@@ -11,13 +11,13 @@
  * @brief  Record a speech segment from microphone to a file
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2001-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -157,6 +157,13 @@ adin_callback_file(SP16 *now, int len)
     fprintf(stderr, "Error: too long input (> %d samples)\n", MAXSPEECHLEN);
     return(FALSE);
     }*/
+
+  /* erase "<<<please speak>>>" text on tty at trigger up */
+  /* (moved from adin-cut.c) */
+  if (speechlen == 0) {
+    j_printerr("\r                    \r");
+  }
+
   /* open output file for the first time */
   if (use_raw) {
     if (fd == -1) {
@@ -253,6 +260,7 @@ main(int argc, char *argv[])
   speechlen = 0;
   
   adin_begin();
+  j_printerr("<<< please speak >>>"); /* moved from adin-cut.c */
   adin_go(adin_callback_file, NULL);
   adin_end();
 
