@@ -1,69 +1,111 @@
-/*
- * Copyright (c) 2001-2003 NAIST
- * All rights reserved
- */
-
-/* 
- * adintool --- AD-in tool to record/split/send/receive speech data
- *
- * $Id: adintool.c,v 1.15 2004/01/08 08:42:45 ri Exp $
+/**
+ * @file   adintool.c
+ * @author Akinobu LEE
+ * @date   Wed Mar 23 20:43:32 2005
  * 
+ * <JA>
+ * @brief  ϤϿ/ʬ/Ѳϥġ
+ *
+ * Υġ Julius β饤֥Ѥ͡ʲϤϤ
+ * ԤޤޥǥХե롦adinnetͥåȥ饤ȡ
+ * ɸϤ鲻ɤ߹(ɬפǤ)ָФԤ
+ * η̤Υǡ缡ե롦adinnetͥåȥСɸ
+ * ʤɤؽϤޤ
+ *
+ * ȤơJulius/Julian ؤΥͥåȥ(=ޥ,=adinnet)
+ * եβ(=ե,=ե)ʤɤ
+ * ѤǤޤ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  AD-in tool to record / split / send / receive speech data
+ *
+ * This tool is to handle speech input and output from/to various devices
+ * using libsetn library in Julius.  It reads input from either of
+ * microphone, file, adinnet network client, or standard input, and
+ * perform speech detection based on level and zero cross (you can disable
+ * this), and output them to either of file, adinnet network server, or
+ * standard output.
+ *
+ * For example, you can send microphone input to Julius running on other host
+ * by specifying input to microphone and output to adinnet (Julius should
+ * be run with "-input adinnet"), or you can long recorded speech data by
+ * long silence by specifying both input and output to file and enabling
+ * segment detection.
+ * </EN>
+ * 
+ * $Revision: 1.3 $
+ * 
+ */
+/*
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2001-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * All rights reserved
  */
 
 #include <sent/stddefs.h>
 #include <sent/speech.h>
 #include <sent/adin.h>
 #include <sent/tcpip.h>
-
-#include <unistd.h>
-#include <sys/types.h>
 #include <sys/stat.h>
-#include <fcntl.h>
 
 /**********************************************************************/
 /* variables about input data and segmentation behavior */
 #ifdef USE_MIC
-static int speech_input = SP_MIC; /* input default is microphone if supported */
+static int speech_input = SP_MIC; ///< input default is microphone if supported
 #else
-static int speech_input = SP_RAWFILE; /* otherwise, file input is default */
+static int speech_input = SP_RAWFILE; ///< otherwise, file input is default
 #endif
-static boolean continuous_segment = TRUE; /* enable/disable successive output */
-static int file_counter = 0;	/* num of input files (for SP_RAWFILE) */
+static boolean continuous_segment = TRUE; ///< enable/disable successive output
+static int file_counter = 0;	///< num of input files (for SP_RAWFILE)
 
 /* values for sentlib audio input library */
 #ifdef USE_NETAUDIO
-static char *netaudio_devname = NULL; /* NetAudio device name */
+static char *netaudio_devname = NULL; ///< NetAudio device name
 #endif
-static boolean strip_zero_sample = TRUE; /* TRUE if strip zero sample */
-static boolean do_segment = TRUE; /* enable/disable long silence cutting */
-static int sfreq = 16000;	/* sampling rate */
-static int level_thres = 2000;	/* input level trigerring threshold */
-static int zero_cross_num = 60;	/* number of zero cross per a second for triggering  */
-static int head_margin = 400;	/* length of silence margin at beginning of a segment */
-static int tail_margin = 400;	/* length of silence margin at end of a segment */
-static boolean use_zmean = FALSE; /* enable zero mean */
+static boolean strip_zero_sample = TRUE; ///< TRUE if strip zero sample
+static boolean do_segment = TRUE; ///< enable/disable long silence cutting
+static int sfreq = 16000;	///< sampling rate
+static int level_thres = 2000;	///< input level trigerring threshold
+static int zero_cross_num = 60;	///< number of zero cross per a second for triggering 
+static int head_margin = 400;	///< length of silence margin at beginning of a segment
+static int tail_margin = 400;	///< length of silence margin at end of a segment
+static boolean use_zmean = FALSE; ///< enable zero mean
 
 /* variables about output data */
-enum{SPOUT_FILE, SPOUT_STDOUT, SPOUT_ADINNET}; /* value for speech_output */
-static int speech_output = SPOUT_FILE; /* output device */
-static int total_speechlen;	/* total samples of recorded segments */
-static int speechlen;		/* samples of one recorded segments */
+enum{SPOUT_FILE, SPOUT_STDOUT, SPOUT_ADINNET}; ///< value for speech_output
+static int speech_output = SPOUT_FILE; ///< output device
+static int total_speechlen;	///< total samples of recorded segments
+static int speechlen;		///< samples of one recorded segments
 
 /* values for successive file output  */
-static boolean use_raw = FALSE;	/* output in RAW format */
-static int fd;			/* file descriptor for RAW output */
-static FILE *fp;		/* file pointer for WAV output */
-static char *filename = NULL;	/* output file name */
-static int startid = 0;		/* output file numbering variable */
+static boolean use_raw = FALSE;	///< output in RAW format
+static int fd;			///< file descriptor for RAW output
+static FILE *fp;		///< file pointer for WAV output
+static char *filename = NULL;	///< output file name
+static int startid = 0;		///< output file numbering variable
 
 /* values for adinnet server output */
-static int adinnet_port = ADINNET_PORT;
-static char *adinnet_serv = NULL;
+static int adinnet_port = ADINNET_PORT; ///< Port number of adinnet 
+static char *adinnet_serv = NULL; ///< Server name to send recorded data
+
+static boolean stop_at_next = FALSE; ///< TRUE if need to stop at next input by server command.  Will be set when PAUSE or TERMINATE command received while input.
 
 
 
 /**********************************************************************/
-/* display usage */
+
+/** 
+ * <JA>
+ * ˡɽƽλ
+ * 
+ * </JA>
+ * <EN>
+ * Display usage and exit.
+ * 
+ * </EN>
+ */
 void
 usage()
 {
@@ -107,7 +149,16 @@ usage()
   exit(1);
 }
 
-/* display input/output configuration */
+/** 
+ * <JA>
+ * ǧΤƥȽϤ
+ * 
+ * </JA>
+ * <EN>
+ * Output input/output configuration in text for a confirmation.
+ * 
+ * </EN>
+ */
 void
 put_status()
 {
@@ -177,7 +228,20 @@ put_status()
   fprintf(stderr,"----\n");
 }    
 
-/* option string parsing */
+/** 
+ * <JA>
+ * ޥɥ饤󥪥ץν
+ * 
+ * @param argc [in] 
+ * @param argv [in] ο
+ * </JA>
+ * <EN>
+ * Parsing command line options
+ * 
+ * @param argc [in] array of option argument strings
+ * @param argv [in] number of arguments
+ * </EN>
+ */
 void
 opt_parse(int argc, char *argv[])
 {
@@ -301,15 +365,30 @@ opt_parse(int argc, char *argv[])
 
 /**********************************************************************/
 /* sentlib ad-in callback functions */
-/* will be called for each incoming samples
-   samples are in now[0..len-1].
-   return value should be:
-   -1 ... error -> exit, terminate 
-   0  ... continue
-   1  ... segmented -> exit, resume (not drop buffer)
-*/
 
-/* record to file specified by file descriptor "fd" */
+/**
+ * <JA>
+ * ɤ߹ץեǥץ "fd" Υե˵Ͽ
+ * 륳Хåؿ
+ * 
+ * @param now [in] Ͽ줿ץ
+ * @param len [in] Ĺʥץ
+ * 
+ * @return 顼 -1 0ֽܶüл 1 ֤
+ * </JA>
+ * <EN>
+ * Callback handler to record the sample fragments to file pointed by
+ * the file descriptor "fd".
+ * 
+ * @param now [in] recorded fragments of speech sample
+ * @param len [in] length of above in samples
+ * 
+ * @return -1 on device error (require caller to exit and terminate input),
+ * 0 on success (allow caller to continue),
+ * 1 on succeeded but segmentation detected (require caller to exit but
+ * input will continue in the next call.
+ * </EN>
+ */
 static int
 adin_callback_file(SP16 *now, int len)
 {
@@ -320,6 +399,12 @@ adin_callback_file(SP16 *now, int len)
     fprintf(stderr, "Error: too long input (> %d samples)\n", MAXSPEECHLEN);
     return(FALSE);
     }*/
+
+  /* erase "<<<please speak>>>" text on tty at trigger up */
+  /* (moved from adin-cut.c) */
+  if (speech_input == SP_MIC && speechlen == 0) {
+    j_printerr("\r                    \r");
+  }
   if (use_raw) {
     /* raw file output */
     count = wrsamp(fd, now, len);
@@ -338,12 +423,46 @@ adin_callback_file(SP16 *now, int len)
   return(0);
 }
 
-/* send to adinnet server pointed by "fd" */
+/**
+ * <JA>
+ * ɤ߹ץ򥽥åȥǥץ "fd" adinnetФ
+ * 륳Хåؿ
+ * 
+ * @param now [in] Ͽ줿ץ
+ * @param len [in] Ĺʥץ
+ * 
+ * @return 顼 -1 0ֽܶüл 1 ֤
+ * </JA>
+ * <EN>
+ * Callback handler to record the sample fragments to adinnet server
+ * pointed by the socket descriptor "fd".
+ * 
+ * @param now [in] recorded fragments of speech sample
+ * @param len [in] length of above in samples
+ * 
+ * @return -1 on device error (require caller to exit and terminate input),
+ * 0 on success (allow caller to continue),
+ * 1 on succeeded but segmentation detected (require caller to exit but
+ * input will continue in the next call.
+ * </EN>
+ */
 static int
 adin_callback_adinnet(SP16 *now, int len)
 {
   int count;
+
+  /* erase "<<<please speak>>>" text on tty at trigger up */
+  /* (moved from adin-cut.c) */
+  if (speech_input == SP_MIC && speechlen == 0) {
+    j_printerr("\r                    \r");
+  }
+#ifdef WORDS_BIGENDIAN
+  swap_sample_bytes(now, len);
+#endif
   count = wt(fd, (char *)now, len * sizeof(SP16));
+#ifdef WORDS_BIGENDIAN
+  swap_sample_bytes(now, len);
+#endif
   if (count < 0) perror("adintool: cannot write");
   /* accumulate sample num of this segment */
   speechlen += len;
@@ -353,23 +472,188 @@ adin_callback_adinnet(SP16 *now, int len)
 }
 
 /**********************************************************************/
-/* send end-of-segment singal to adinnet server */
-/* (for SPOUT_ADINNET only) */
+/** 
+ * <JA>
+ * adinnetФ˥Ƚλ
+ * 
+ * </JA>
+ * <EN>
+ * Send end-of-segment singal to adinnet server.
+ * 
+ * </EN>
+ */
 static void
 adin_send_end_of_segment()
 {
-  char *p;
-  if (wt(fd, p,  0) < 0) perror("adintool: cannot write");
-}  
+  char p;
+  if (wt(fd, &p,  0) < 0) perror("adintool: cannot write");
+}
 
 /**********************************************************************/
-/* main function */
+/* reveice resume/pause command from adinnet server */
+/* (for SPOUT_ADINNET only) */
+/*'1' ... resume  '0' ... pause */
+
+static int unknown_command_counter = 0;	///< Counter to detect broken connection
+/** 
+ * <JA>
+ * ˥Ф/Ƴޥɤ뤿
+ * Хåؿ
+ * 
+ * @return ޥ̵ƳޥɤϿ³Ԥξ 0,
+ * 顼 -2, ǥޥɤϿǤ٤Ȥ -1 ֤
+ * </JA>
+ * <EN>
+ * Callback function for A/D-in processing to check pause/resume
+ * command from adinnet server.
+ * 
+ * @return 0 when no command or RESUME command to tell caller to
+ * continue recording, -1 when received a PAUSE command and tell caller to
+ * stop recording, or -2 when error.
+ * </EN>
+ */
+static int
+adinnet_check_command()
+{
+  fd_set rfds;
+  struct timeval tv;
+  int status;
+  int cnt, ret;
+  char com;
+  
+  /* check if some commands are waiting in queue */
+  FD_ZERO(&rfds);
+  FD_SET(fd, &rfds);
+  tv.tv_sec = 0;
+  tv.tv_usec = 1;
+  status = select(fd+1, &rfds, NULL, NULL, &tv);
+  if (status < 0) {           /* error */
+    j_printerr("adintool: cannot check command from adinnet server\n");
+    return -2;                        /* error return */
+  }
+  if (status > 0) {           /* there are some data */
+    if (FD_ISSET(fd, &rfds)) {
+      ret = rd(fd, &com, &cnt, 1); /* read in command */
+      switch (com) {
+      case '0':                       /* pause */
+	j_printerr("<PAUSE>");
+	stop_at_next = TRUE;	/* mark to pause at the end of this input */
+	/* tell caller to stop recording */
+	return -1;
+      case '1':                       /* resume */
+	j_printerr("<RESUME - already running>");
+	/* we are already running, so just continue */
+	break;
+      case '2':                       /* terminate */
+	j_printerr("<TERMINATE>");
+	stop_at_next = TRUE;	/* mark to pause at the end of this input */
+	/* tell caller to stop recording immediately */
+	return -2;
+	break;
+      default:
+	j_printerr("adintool: unknown command: %d\n", com);
+	unknown_command_counter++;
+	/* avoid infinite loop in that case... */
+	/* this may happen when connection is terminated from server side  */
+	if (unknown_command_counter > 100) {
+	  j_printerr("killed by unknown command flood\n");
+	  exit(1);
+	}
+      }
+    }
+  }
+  return 0;			/* continue ad-in */
+}
+
+/** 
+ * <JA>
+ * ФƳޥɤޤԤġƳޥɤ
+ * λ롥
+ * 
+ * @return 顼 -1, ̾ｪλ 0 ֤
+ * </JA>
+ * <EN>
+ * Wait for resume command from server.
+ * 
+ * @return 0 on normal exit, or -1 on error.
+ * </EN>
+ */
+static int
+adinnet_wait_command()
+{
+  fd_set rfds;
+  int status;
+  int cnt, ret;
+  char com;
+
+  j_printerr("<<< waiting RESUME >>>");
+
+  while(1) {
+    FD_ZERO(&rfds);
+    FD_SET(fd, &rfds);
+    FD_SET(fd, &rfds);
+    status = select(fd+1, &rfds, NULL, NULL, NULL); /* brock if no command */
+    if (status < 0) {         /* error */
+      j_printerr("adintool: cannot check command from adinnet server\n");
+      return -1;                      /* error return */
+    } else {                  /* there are some data */
+      if (FD_ISSET(fd, &rfds)) {
+	ret = rd(fd, &com, &cnt, 1);
+	switch (com) {
+	case '0':                       /* pause */
+	  /* already paused, so just wait for next command */
+	  j_printerr("<PAUSE - already paused>");
+	  break;
+	case '1':                       /* resume */
+	  /* do resume */
+	  j_printerr("<RESUME>\n");
+	  return 1;		/* tell caller to restart */
+	case '2':                       /* terminate */
+	  /* already paused, so just wait for next command */
+	  j_printerr("<TERMINATE - already paused>");
+	  break;
+	default:
+	  j_printerr("adintool: unknown command: %d\n", com);
+	  unknown_command_counter++;
+	  /* avoid infinite loop in that case... */
+	  /* this may happen when connection is terminated from server side  */
+	  if (unknown_command_counter > 100) {
+	    j_printerr("killed by unknown command flood\n");
+	    exit(1);
+	  }
+	}
+      }
+    }
+  }
+  return 0;
+} 
+
+/**********************************************************************/
+
+/** 
+ * <JA>
+ * ᥤؿ
+ * 
+ * @param argc [in] Ĺ
+ * @param argv [in] 
+ * 
+ * @return 
+ * </JA>顼 1̾ｪλ 0 ֤
+ * <EN>
+ * Main function.
+ * 
+ * @param argc [in] number of argument.
+ * @param argv [in] array of arguments.
+ * 
+ * @return 1 on error, 0 on success.
+ * </EN>
+ */
 int
 main(int argc, char *argv[])
 {
   int ret;			/* return value of adin_go() */
-  int sid;			/* current file ID (for SPOUT_FILE) */
-  char *outpath;		/* output file name */
+  int sid = 0;			/* current file ID (for SPOUT_FILE) */
+  char *outpath = NULL;		/* output file name */
   char *arg;
 
   /* process arguments */
@@ -501,7 +785,7 @@ main(int argc, char *argv[])
 	fprintf(stderr,"[%s]", outpath);
 	if (access(outpath, F_OK) == 0) {
 	  if (access(outpath, W_OK) == 0) {
-	    fprintf(stderr, "(override)", filename);
+	    fprintf(stderr, "(override)");
 	  } else {
 	    perror("adintool");
 	    return(1);
@@ -526,21 +810,41 @@ main(int argc, char *argv[])
       */
       /* adin_go() return when input segmented by long silence, or input stream reached to the end */
       speechlen = 0;
+      stop_at_next = FALSE;
+      if (speech_input == SP_MIC) {
+	j_printerr("<<< please speak >>>");
+      }
       if (speech_output == SPOUT_ADINNET) {
-	ret = adin_go(adin_callback_adinnet, NULL);
+	ret = adin_go(adin_callback_adinnet, adinnet_check_command);
       } else {
 	ret = adin_go(adin_callback_file, NULL);
       }
       /* return value of adin_go:
+	 -2: input terminated by pause command from adinnet server
 	 -1: input device read error or callback process error
 	 0:  paused by input stream (end of file, etc..)
 	 >0: detected end of speech segment:
              by adin-cut, or by callback process
 	 (or return value of ad_check (<0) (== not used in this program))
       */
-      j_printf("[%d]\n", ret);
-      j_printerr("\n");
-      if (ret < 0) {
+      /* if PAUSE or TERMINATE command has been received while input,
+	 stop_at_next is TRUE here  */
+      switch(ret) {
+      case -2:			/* terminated by terminate command from server */
+	j_printerr("[terminated by server]\n");
+	break;
+      case -1:			/* device read error or callback error */
+	j_printerr("[error]\n");
+	break;
+      case 0:			/* reached to end of input */
+	j_printerr("[eof]\n");
+	break;
+      default:			/* input segmented by silence or callback process */
+	j_printerr("[segmented]\n", ret);
+	break;
+      }
+      
+      if (ret == -1) {
 	/* error in input device or callback function, so terminate program here */
 	return 1;
       }
@@ -559,12 +863,14 @@ main(int argc, char *argv[])
 	/* output info */
 	j_printf("%s: %d samples (%.2f sec.)\n", outpath, speechlen, (float)speechlen / (float)sfreq);
       } else if (speech_output == SPOUT_ADINNET) {
-	if (ret >= 0) {		/* segmented by adin-cut or end of stream */
-	  /* send end-of-segment ack to client */
-	  adin_send_end_of_segment();
+	if (speechlen > 0) {
+	  if (ret >= 0 || stop_at_next) { /* segmented by adin-cut or end of stream or server-side command */
+	    /* send end-of-segment ack to client */
+	    adin_send_end_of_segment();
+	  }
+	  /* output info */
+	  j_printf("sent: %d samples (%.2f sec.)\n", speechlen, (float)speechlen / (float)sfreq);
 	}
-	/* output info */
-	j_printf("send: %d samples (%.2f sec.)\n", speechlen, (float)speechlen / (float)sfreq);
       }
       
       /*************************************/
@@ -574,6 +880,17 @@ main(int argc, char *argv[])
 	total_speechlen += speechlen;
 	sid++;
       }
+
+      /***************************************************/
+      /* with adinnet server, if terminated by           */
+      /* server-side PAUSE command, wait for RESUME here */
+      /***************************************************/
+      if (speech_output == SPOUT_ADINNET && stop_at_next) {
+	if (adinnet_wait_command() < 0) {
+	  /* command error: terminate program here */
+	  return 1;
+	}
+      }
       
     } while (continuous_segment && ret > 0); /* to the next segment in this input stream */
 
