@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 1991-2003 Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -35,4 +37,4 @@ int dfa_nextterms(NODE *hypo, NEXTWORD **nw);
 boolean dfa_acceptable(NODE *hypo);
 
 
-#endif __NEXT__H__
+#endif /* __NEXT__H__ */
