@@ -11,13 +11,13 @@
  * @brief  Output results in word graph format.
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -25,9 +25,12 @@
 
 #ifdef GRAPHOUT
 
-/// Define if you want debugging output for graph output
+/// Define if you want debugging output for graph generation
 #undef GDEBUG
 
+/// Define if you want much more debugging output for graph generation
+#undef GDEBUG2
+
 
 /**************************************************************/
 /* allocation and free of a WordGraph instance */
@@ -325,6 +328,7 @@ merge_contexts(WordGraph *dst, WordGraph *src)
   /* left context */
   for(s=0;s<src->leftwordnum;s++) {
     adding = src->leftword[s];
+    if (adding->mark) continue;
     /* direct link between dst and src will disapper to avoid unneccesary loop */
     if (adding == dst) {
 #ifdef GDEBUG
@@ -333,6 +337,7 @@ merge_contexts(WordGraph *dst, WordGraph *src)
       continue;
     }
     for(d=0;d<dst->leftwordnum;d++) {
+      if (dst->leftword[d]->mark) continue;
       if (dst->leftword[d] == adding) {
 	break;
       }
@@ -356,6 +361,7 @@ merge_contexts(WordGraph *dst, WordGraph *src)
   /* right context */
   for(s=0;s<src->rightwordnum;s++) {
     adding = src->rightword[s];
+    if (adding->mark) continue;
     /* direct link between dst and src will disapper to avoid unneccesary loop */
     if (adding == dst) {
 #ifdef GDEBUG
@@ -364,6 +370,7 @@ merge_contexts(WordGraph *dst, WordGraph *src)
       continue;
     }
     for(d=0;d<dst->rightwordnum;d++) {
+      if (dst->rightword[d]->mark) continue;
       if (dst->rightword[d] == adding) {
 	break;
       }
@@ -572,6 +579,14 @@ wordgraph_remove_context(WordGraph *wg)
       }
     }
     w->leftwordnum = k;
+#ifdef GDEBUG2
+    if (w->leftwordnum == 0) {
+      j_printf("leftword becomes 0 by remove_context\n");
+      put_wordgraph(w);
+      j_printf("by deleting its left context:\n");
+      put_wordgraph(wg);
+    }
+#endif
   }
 }
 
@@ -590,14 +605,21 @@ wordgraph_remove_context(WordGraph *wg)
 static void
 wordgraph_link_context(WordGraph *wg)
 {
-  WordGraph *w;
   int i,j;
-
+  WordGraph *left, *right;
+  
   if (wg == NULL) return;
   for(i=0;i<wg->leftwordnum;i++) {
+    left = wg->leftword[i];
+    if (left->mark) continue;
+    if (left == wg) continue;
     for(j=0;j<wg->rightwordnum;j++) {
-      wordgraph_check_and_add_leftword(wg->rightword[j], wg->leftword[i]);
-      wordgraph_check_and_add_rightword(wg->leftword[i], wg->rightword[j]);
+      right = wg->rightword[j];
+      if (right->mark) continue;
+      if (right == wg) continue;
+      if (left == right) continue;
+      wordgraph_check_and_add_leftword(right, left);
+      wordgraph_check_and_add_rightword(left, right);
     }
   }
 }
@@ -759,14 +781,42 @@ wordgraph_clean(WordGraph **rootp)
 }
 
 
-/**************************************************************/
+/*********************************************************************/
 /* Post-processing of generated word arcs after search has been done */
 
+/** 
+ * <JA>
+ * ñ쥰տåȤΤ qsort ѥХåfscore_head 
+ * ߽˥Ȥ롥
+ * 
+ * @param x [in] ǣ
+ * @param y [in] ǣ
+ * 
+ * @return qsort ˽स֤
+ * </JA>
+ * <EN>
+ * Callback function for qsort to do word graph depth cutting. Graph
+ * words will be sorted downward based on fscore_head.
+ * 
+ * @param x [in] element 1
+ * @param y [in] element 2
+ * 
+ * @return values for qsort
+ * </EN>
+ */
+static int
+compare_beam(WordGraph **x, WordGraph **y)
+{
+  if ((*x)->fscore_head < (*y)->fscore_head) return 1;
+  else if ((*x)->fscore_head > (*y)->fscore_head) return -1;
+  else return 0;
+}
+
 /** 
  * <JA>
  * @brief  ոΣñ쥰դС
  * 
- * õ줿ñ佸礫顤üϤޤѥ̵ñ
+ * õ줿ñ佸礫顤üϤޤѥ̵leafñ
  * 뤳Ȥǽñ쥰դФ롥
  * 
  * @param rootp [i/o] ñ쥰դΥ롼ȥΡɤؤΥݥ
@@ -784,35 +834,56 @@ void
 wordgraph_purge_leaf_nodes(WordGraph **rootp)
 {
   WordGraph *wg;
+  int i, dst;
   boolean changed;
-  int i, count, erased;
+  int count, erased, del_left, del_right;
 
   /* count whole */
   count = 0;
   for(wg=*rootp;wg;wg=wg->next) count++;
   j_printf("- %d initial word arcs generated\n", count);
-
+  if (count == 0) return;
+  
   j_printf("Step 1: Purge leaf words\n");
+
   /* mark words to be erased */
+  del_left = del_right = 0;
   do {
     changed = FALSE;
     for(wg=*rootp;wg;wg=wg->next) {
+      if (wg->mark == TRUE) continue;
       /* mark if wg has no left context, or all leftwords are marked */
-      if (wg->mark == TRUE || wg->lefttime == 0) continue;
-      for(i=0;i<wg->leftwordnum;i++) {
-	if (wg->leftword[i]->mark == FALSE) break;
+      if (wg->lefttime != 0) {
+	for(i=0;i<wg->leftwordnum;i++) {
+	  if (wg->leftword[i]->mark == FALSE) break;
+	}
+	if (i >= wg->leftwordnum) {
+	  wg->mark = TRUE;
+	  changed = TRUE;
+	  del_left++;
+	  continue;
+	}
       }
-      if (i >= wg->leftwordnum) {
-	wg->mark = TRUE;
-	changed = TRUE;
+      /* mark if wg has no right context, or all rightwords are marked */
+      if (wg->righttime != peseqlen - 1) {
+	for(i=0;i<wg->rightwordnum;i++) {
+	  if (wg->rightword[i]->mark == FALSE) break;
+	}
+	if (i >= wg->rightwordnum) {
+	  wg->mark = TRUE;
+	  changed = TRUE;
+	  del_right++;
+	  continue;
+	}
       }
     }
   } while (changed == TRUE);
 
-  /* do compaction of leftwords */
-  {
-    int dst;
+  j_printf("- %d leaf words found (left_blank=%d, right_blank=%d)\n", del_left + del_right, del_left, del_right);
+
+  /* do compaction of left/rightwords */
     for(wg=*rootp;wg;wg=wg->next) {
+      if (wg->mark) continue;
       dst = 0;
       for(i=0;i<wg->leftwordnum;i++) {
 	if (wg->leftword[i]->mark == FALSE) {
@@ -822,136 +893,396 @@ wordgraph_purge_leaf_nodes(WordGraph **rootp)
       }
       wg->leftwordnum = dst;
     }
-  }
+    for(wg=*rootp;wg;wg=wg->next) {
+      if (wg->mark) continue;
+      dst = 0;
+      for(i=0;i<wg->rightwordnum;i++) {
+	if (wg->rightword[i]->mark == FALSE) {
+	  if (dst != i) wg->rightword[dst] = wg->rightword[i];
+	  dst++;
+	}
+      }
+      wg->rightwordnum = dst;
+    }
 
   /* execute erase of marked words */
   erased = wordgraph_exec_erase(rootp);
   j_printf("- %d words purged, %d words left in lattice\n", erased, count - erased);
+
 }
 
 /** 
  * <JA>
- * @brief  ոΣñ춭Ĵ
+ * @brief  ոΣդοˤñΥå
  * 
- * GRAPHOUT_PRECISE_BOUNDARY ³ñ˰¸Τñ춭
- * 뤿ˡõˤơñȤ˼Ÿ
- * Ūñ춭ư롥ΤᡤñΤġʰưΡ
- * ȤбȤʤʤΤǡõλ˳ññ
- * ñ춭¤뤳ȤȤ롥
+ * GRAPHOUT_DEPTHCUT դοˤñΥåȤԤ
  * 
  * @param rootp [i/o] ñ쥰դΥ롼ȥΡɤؤΥݥ
  * </JA>
  * <EN>
- * @brief  Post-processing step 2: Adjust word boundaries.
+ * @brief  Post-processing step 1.5: word graph depth cutting
  * 
- * When GRAPHOUT_PRECISE_BOUNDARY is defined, the word boundaries will be
- * moved depending on the later word expansion to get context-dependent
- * precise boundaries.  So the precise boundary, modified after generation
- * while search, should be propagated to the context words in the post
- * processing.
+ * If GRAPHOUT_DEPTHCUT is defined, perform word graph depth cutting.
  * 
  * @param rootp [i/o] pointer to root node of a word graph
  * </EN>
  */
 void
-wordgraph_adjust_boundary(WordGraph **rootp)
+wordgraph_depth_cut(WordGraph **rootp)
 {
-#ifdef GRAPHOUT_PRECISE_BOUNDARY
-  WordGraph *wg, *left, *new;
-  int i, j;
-  int new_lefttime;
-  int *framelist;
-  LOGPROB *framescorelist;
-  int maxfnum;
-  int fnum;
-  int mov_num, dup_num, del_num;
-  int count;
+#ifdef GRAPHOUT_DEPTHCUT
+
+  WordGraph *wg;
+  int i, dst;
   boolean changed;
+  int count, erased, del_left, del_right;
+  WordGraph **wlist;
+  boolean f;
+  int *wc;
+  int t;
+  int pruned;
 
-  j_printf("Step 2: Fit word boundaries\n");
-  mov_num = dup_num = del_num = 0;
 
-  /* count number of all words and get maximum number of left context words */
+  if (graphout_cut_depth < 0) return;
+
+  j_printf("Step 1.5: Cut lower words by depth of %d\n", graphout_cut_depth);
+
+  /* count whole */
   count = 0;
-  maxfnum = 0;
+  for(wg=*rootp;wg;wg=wg->next) count++;
+  if (count == 0) return;
+  
+  /* prepare buffer to count words per frame */
+  wc = (int *)mymalloc(sizeof(int) * peseqlen);
+  for (t=0;t<peseqlen;t++) wc[t] = 0;
+  /* sort words by fscore_head */
+  wlist = (WordGraph **)mymalloc(sizeof(WordGraph *) * count);
+  i = 0;
   for(wg=*rootp;wg;wg=wg->next) {
-    count++;
-    if (maxfnum < wg->leftwordnum) maxfnum = wg->leftwordnum;
+    wlist[i++] = wg;
   }
-  /* allocate temporal work area */
-  framelist = (int *)mymalloc(sizeof(int) * maxfnum);
-  framescorelist = (LOGPROB *)mymalloc(sizeof(LOGPROB) * maxfnum);
+  qsort(wlist, count, sizeof(WordGraph *), (int (*)(const void *, const void *))compare_beam);
+  /* count words per frame, and unlink/mark them if below beam width */
+  pruned = 0;
+  for (i=0;i<count;i++) {
+    wg = wlist[i];
+    f = TRUE;
+    for (t=wg->lefttime;t<=wg->righttime;t++) {
+      wc[t]++;
+      if (wc[t] <= graphout_cut_depth) f = FALSE;
+    }
+    if (f) {
+      //wordgraph_remove_context(wg);
+      wg->mark = TRUE;
+      pruned++;
+    }
+  }
+#ifdef GDEBUG2
+  printf("GRAPH DEPTH STATISTICS: NUMBER OF WORDS PER FRAME\n");
+  for(t=0;t<peseqlen;t++) {
+    if (wc[t] > graphout_cut_depth) {
+      printf("*");
+    } else {
+      printf(" ");
+    }
+    printf("%4d: %d\n", t, wc[t]);
+  }
+#endif
+  j_printf("- %d words out of %d are going to be pruned by depth cutting\n", pruned, count);
+  free(wlist);
+  free(wc);
 
+  /* mark words to be erased */
+  del_left = del_right = 0;
   do {
-    /* loop until no word has been changed on the last step */
     changed = FALSE;
     for(wg=*rootp;wg;wg=wg->next) {
-      if (wg->mark) continue;	/* already marked */
-      if (wg->leftwordnum == 0) continue;	/* no leftword */
-      /* correct lefttime variation to framelist[] and framescorelist[] */
-      fnum = 0;
-      for(i=0;i<wg->leftwordnum;i++) {
-	left = wg->leftword[i];
-	if (left->mark) continue;
-	for(j=0;j<fnum;j++) {
-	  if (framelist[j] == left->righttime + 1) break;
+      if (wg->mark == TRUE) continue;
+      /* mark if wg has no left context, or all leftwords are marked */
+      if (wg->lefttime != 0) {
+	for(i=0;i<wg->leftwordnum;i++) {
+	  if (wg->leftword[i]->mark == FALSE) break;
 	}
-	if (j >= fnum) {
-	  framelist[fnum] = left->righttime + 1;
-	  framescorelist[fnum] = left->gscore_tail
-#ifdef USE_NGRAM
-	    /* the tail gscore contains the language score of the word,
-	       so the head gscore of its right context should consider this */
-	    - left->lscore
-#endif
-	    ;
-	  fnum++;
+	if (i >= wg->leftwordnum) {
+	  wg->mark = TRUE;
+	  changed = TRUE;
+	  del_left++;
+	  continue;
 	}
       }
-      if (fnum == 0) continue;	/* no left context */
-      /* one candidate: just move the original (or not move) */
-      if (fnum == 1) {
-	if (wg->lefttime != framelist[0]) {
+      /* mark if wg has no right context, or all rightwords are marked */
+      if (wg->righttime != peseqlen - 1) {
+	for(i=0;i<wg->rightwordnum;i++) {
+	  if (wg->rightword[i]->mark == FALSE) break;
+	}
+	if (i >= wg->rightwordnum) {
+	  wg->mark = TRUE;
+	  changed = TRUE;
+	  del_right++;
+	  continue;
+	}
+      }
+    }
+  } while (changed == TRUE);
+
+  j_printf("- %d new leaves found (left_blank=%d, right_blank=%d)\n", del_left + del_right, del_left, del_right);
+
+  /* do compaction of left/rightwords */
+    for(wg=*rootp;wg;wg=wg->next) {
+      if (wg->mark) continue;
+      dst = 0;
+      for(i=0;i<wg->leftwordnum;i++) {
+	if (wg->leftword[i]->mark == FALSE) {
+	  if (dst != i) wg->leftword[dst] = wg->leftword[i];
+	  dst++;
+	}
+      }
+      wg->leftwordnum = dst;
+    }
+    for(wg=*rootp;wg;wg=wg->next) {
+      if (wg->mark) continue;
+      dst = 0;
+      for(i=0;i<wg->rightwordnum;i++) {
+	if (wg->rightword[i]->mark == FALSE) {
+	  if (dst != i) wg->rightword[dst] = wg->rightword[i];
+	  dst++;
+	}
+      }
+      wg->rightwordnum = dst;
+    }
+
+  /* execute erase of marked words */
+  erased = wordgraph_exec_erase(rootp);
+  j_printf("- total %d words purged, %d words left in lattice\n", erased, count - erased);
+
+#else  /* ~GRAPHOUT_DEPTHCUT */
+
+  j_printf("ERROR: Step 1.5: depth cutting has been disabled, skipped\n");
+
+#endif
+
+}
+
+/** 
+ * <JA>
+ * ñ֤ζΤ¹Ԥ롥ñå,
+ * ³ñ֤Ƕ־ˤ줬ȤϡΤ롥
+ * ʣΥƥȴ֤ǰۤʤ붭¸ߤ,
+ * ԡƤ줾˹碌롥ޤ饤Ȥñ롥
+ * 
+ * @param rootp [i/o] ñꥹȤΥ롼ȥݥ
+ * @param mov_num_ret [out] ֤ưñǼѿؤΥݥ
+ * @param dup_num_ret [out] ԡ줿ñǼѿؤΥݥ
+ * @param del_num_ret [out] 줿ñǼѿؤΥݥ
+ * @param count [in] վñ
+ * 
+ * @return ñ줬İʾѹ TRUEѹʤǤ FALSE
+ * ֤
+ * </JA>
+ * <EN>
+ * Execute adjustment of word boundaries.  It looks through the graph to
+ * check correspondence of word boundary information among context, and if
+ * there is a gap, the beginning frame of right word will be moved to the
+ * end frame of left word.  If several alignment is found among contexts,
+ * the word will be duplicated and each will be fit to each context.  Also,
+ * words with invalid alignment will be eliminated.
+ * 
+ * @param rootp [in] root pointer to the list of graph words
+ * @param mov_num_ret [out] pointer to hold resulted number of moved words
+ * @param dup_num_ret [out] pointer to hold resulted number of duplicated words
+ * @param del_num_ret [out] pointer to hold resulted number of eliminated words
+ * @param mod_num_ret [out] pointer to hold resulted number of modified words
+ * @param count [in] number of words in graph
+ * 
+ * @return TRUE if any word has been changed, or FALSE if no word has been altered.
+ * </EN>
+ */
+static boolean
+wordgraph_adjust_boundary_sub(WordGraph **rootp, int *mov_num_ret, int *dup_num_ret, int *del_num_ret, int *mod_num_ret, int count)
+{
+  WordGraph *wg, *left, *new;
+  int i, j, k;
+  int fnum;
+  int mov_num, dup_num, del_num, mod_num;
+  boolean changed = FALSE;
+  static int *framelist;
+  static LOGPROB *framescorelist;
+  static int maxfnum = 0;
+
+  mov_num = dup_num = del_num = mod_num = 0;
+
+  /* maximum number of left context words does not exceed total word num */
+  /* allocate temporal work area.  these are permanent buffer that will
+     be kept between recognition sessions. */
+  if (maxfnum == 0) {
+    /* when this is called for the first time, allocate buffer */
+    maxfnum = count;
+    framelist = (int *)mymalloc(sizeof(int) * maxfnum);
+    framescorelist = (LOGPROB *)mymalloc(sizeof(LOGPROB) * maxfnum);
+#ifdef GDEBUG
+    j_printerr("Notice: maxfnum starts at %d\n", maxfnum);
+#endif
+  } else if (maxfnum < count) {
+    /* for later call, expand buffer if necessary */
+    maxfnum = count;
+    framelist = (int *)myrealloc(framelist, sizeof(int) * maxfnum);
+    framescorelist = (LOGPROB *)myrealloc(framescorelist, sizeof(LOGPROB) * maxfnum);
+#ifdef GDEBUG
+    j_printerr("Notice: maxfnum expanded by count (%d)\n", maxfnum);
+#endif
+  }
+
+#ifdef GDEBUG2
+  printf("***CHECK LOOP BEGIN***\n");
+#endif
+  for(wg=*rootp;wg;wg=wg->next) {
+    if (wg->mark) continue;	/* already marked */
+#ifdef GDEBUG2
+    printf("  [%d..%d] \"%s\"\n", wg->lefttime, wg->righttime, wchmm->winfo->woutput[wg->wid]);
+#endif
+    if (wg->leftwordnum == 0) {	/* no leftword */
+      if (wg->lefttime != 0) {
+	/* some fraction found by former elimination: remove this */
+#ifdef GDEBUG2
+	printf("  -> no leftword at middle of lattice, eliminate this\n");
+#endif
+	wordgraph_remove_context(wg);
+	wg->mark = TRUE;
+	del_num++;
+	changed = TRUE;
+      }
+      /* if has no leftword, adjustment of this word is not needed */
+      continue;
+    }
+    if (wg->rightwordnum == 0) {	/* no rightword */
+      if (wg->righttime != peseqlen-1) {
+	/* some fraction found by former elimination: remove this */
+#ifdef GDEBUG2
+	printf("  -> no rightword at middle of lattice, eliminate this\n");
+#endif
+	wordgraph_remove_context(wg);
+	wg->mark = TRUE;
+	del_num++;
+	changed = TRUE;
+	continue;
+      }
+      /* if on right edge, continue adjusting */
+    }
+    /* correct lefttime variation to framelist[] and framescorelist[] */
+    fnum = 0;
+    /* check for buffer overrun */
+    if (wg->leftwordnum > maxfnum) {
+      /* expand buffer if necessary */
+      maxfnum = wg->leftwordnum;
+      framelist = (int *)myrealloc(framelist, sizeof(int) * maxfnum);
+      framescorelist = (LOGPROB *)myrealloc(framescorelist, sizeof(LOGPROB) * maxfnum);
+#ifdef GDEBUG
+      j_printerr("Notice: wg->leftwordnum exceeds maxfnum (%d > %d), expanded\n", wg->leftwordnum, maxfnum);
+#endif
+    }
+    for(i=0;i<wg->leftwordnum;i++) {
+      left = wg->leftword[i];
+      if (left->mark) continue;
+      for(j=0;j<fnum;j++) {
+	if (framelist[j] == left->righttime + 1) break;
+      }
+      if (j >= fnum) {
+	framelist[fnum] = left->righttime + 1;
+	framescorelist[fnum] = left->gscore_tail
+#ifdef USE_NGRAM
+	  /* the tail gscore contains the language score of the word,
+	     so the head gscore of its right context should consider this */
+	  - left->lscore
+#endif
+	  ;
+	fnum++;
+      }
+    }
+#ifdef GDEBUG2
+    printf("  possible boundary of left words:");
+    if (fnum == 0) {
+      printf(" (not exist)\n");
+    } else {
+      for(j=0;j<fnum;j++) printf(" %d", framelist[j]);
+      printf("\n");
+    }
+#endif
+    if (fnum == 0) continue;	/* no left context */
+    /* one candidate: just move the original (or not move) */
+    if (fnum == 1) {
+      if (wg->lefttime != framelist[0]) {
+#ifdef GDEBUG2
+	printf("    !moving as [%d..%d]", framelist[0], wg->righttime);
+#endif
+	/* check the time correctness: if the lefttime is larger than
+	   righttime, this graph word has been completely overridden by
+	   the left word (i.e. the aligned frames are absorbed by
+	   re-alignment.  In this case this word should be removed.
+	*/
+	if (framelist[0] > wg->righttime) {
+#ifdef GDEBUG2
+	  printf(" : eliminated");
+#endif
+	  wordgraph_link_context(wg);
+	  wordgraph_remove_context(wg);
+	  wg->mark = TRUE;
+	  del_num++;
+	} else {
+#ifdef GDEBUG2
+	  printf(" : ok");
+#endif
 	  /* adjust time and score */
-	  //printf("    moved: %d -> %d\n", wg->lefttime, framelist[0]);
 	  wg->lefttime = framelist[0];
 	  wg->gscore_head = framescorelist[0];
-	  /* check the time correctness: if the lefttime is larger than
-	     righttime, this graph word has been completely overridden by
-	     the left word (i.e. the aligned frames are absorbed by
-	     re-alignment.  In this case this word should be removed.
-	  */
-	  if (wg->lefttime > wg->righttime) {
-	    wordgraph_link_context(wg);
-	    wordgraph_remove_context(wg);
-	    wg->mark = TRUE;
-	    del_num++;
-	  } else {
-	    mov_num++;
-	  }
-	  changed = TRUE;
-	} else if (wg->gscore_head != framescorelist[0]) {
-	  /* adjust only score */
-	  wg->gscore_head = framescorelist[0];
-	  changed = TRUE;
+	  mov_num++;
 	}
+#ifdef GDEBUG2
+	printf("\n");
+#endif
+	changed = TRUE;
+      } else if (wg->gscore_head != framescorelist[0]) {
+	/* adjust only score */
+#ifdef GDEBUG2
+	printf("    !ghead score changed: %f -> %f\n", wg->gscore_head, framescorelist[0]);
+#endif
+	wg->gscore_head = framescorelist[0];
+	mod_num++;
+	changed = TRUE;
       }
-      if (fnum > 1) {
-	/* multiple candidate: make copy for each (fnum)*/
-	for(j=0;j<fnum;j++) {
-	  //printf("    duped: (%d) %d -> %d\n", j+1, wg->lefttime, framelist[j]);
-	  dup_num++;
-	  /* check the time correctness: if the lefttime is larger than
-	     righttime, this graph word has been completely overridden by
-	     the left word (i.e. the aligned frames are absorbed by
-	     re-alignment.  In this case this word should not be duplicated
-	  */
-	  if (framelist[j] > wg->righttime) {
-	    del_num++;
-	    continue;
+    }
+    if (fnum > 1) {
+      /* multiple candidate: make copy for each (fnum)*/
+      for(j=0;j<fnum;j++) {
+	/* duplicate */
+	dup_num++;
+#ifdef GDEBUG2
+	printf("    !duping as [%d..%d]", framelist[j], wg->righttime);
+#endif
+	
+	if (framelist[j] > wg->righttime) {
+	  /* bogus link: link leftwords and rightwords, and delete this */
+#ifdef GDEBUG2
+	  printf(" : eliminated");
+#endif
+	  for(i=0;i<wg->leftwordnum;i++) {
+	    left = wg->leftword[i];
+	    if (left->mark) continue;
+	    if (left->righttime + 1 == framelist[j]) {
+	      for(k=0;k<wg->rightwordnum;k++) {
+		if ((wg->rightword[k])->mark) continue;
+		if (wg->rightword[k] == left) continue;
+		wordgraph_check_and_add_leftword(wg->rightword[k], left);
+		wordgraph_check_and_add_rightword(left, wg->rightword[k]);
+	      }
+	    }
 	  }
+	  del_num++;
 	  
-	  /* duplicate */
+	} else {
+	  /* really duplicate */
+#ifdef GDEBUG2
+	  printf(" : ok");
+#endif
 	  new = wordgraph_new(wg->wid, framelist[j], wg->righttime, wg->fscore_head, wg->fscore_tail, framescorelist[j], wg->gscore_tail
 #ifdef USE_NGRAM
 			      , wg->lscore
@@ -981,24 +1312,180 @@ wordgraph_adjust_boundary(WordGraph **rootp)
 	  new->next = *rootp;
 	  *rootp = new;
 	}
-	/* ñΥƥȤWGʴؿ */
-	wordgraph_remove_context(wg);
-	/* remove the original */
-	wg->mark = TRUE;
-	dup_num--;
-	changed = TRUE;
+#ifdef GDEBUG2
+	printf("\n");
+#endif
+      }
+      
+      /* remove the original */
+#ifdef GDEBUG2
+      printf("    !delete original [%d..%d]\n", wg->lefttime, wg->righttime);
+#endif
+      wordgraph_remove_context(wg);
+      wg->mark = TRUE;
+      dup_num--;
+      
+      changed = TRUE;
+    }
+  }
+
+  *mov_num_ret = mov_num;
+  *dup_num_ret = dup_num;
+  *del_num_ret = del_num;
+  *mod_num_ret = mod_num;
+
+#ifdef GDEBUG2
+  if (changed) {
+    printf("*** some graph has been altered, check loop continues\n");
+  } else {
+    printf("*** graph not changed at last loop, check ends here\n");
+  }
+#endif
+
+  return (changed);
+}
+
+/** 
+ * <JA>
+ * ˶䥹Ʊñ줬礽ޡ롥
+ * 
+ * @param rootp [i/o] ñꥹȤΥ롼ȥݥ
+ * @param rest_ret [out] ޡΥñ֤ݥ
+ * @param merged_ret [out] ޡ줿ñ֤ݥ
+ * </JA>
+ * <EN>
+ * Merge duplicated words with exactly the same scores and alignments.
+ * 
+ * @param rootp [i/o] root pointer to the list of graph words
+ * @param rest_ret [out] pointer to hold resulted number of words left in graph
+ * @param merged_ret [out] pointer to hold resuled number of merged words
+ * </EN>
+ */
+static void
+wordgraph_compaction_thesame_sub(WordGraph **rootp, int *rest_ret, int *merged_ret)
+{
+  WordGraph *wg, *we;
+  int i, count, erased, merged;
+
+  count = 0;
+  merged = 0;
+  for(wg=*rootp;wg;wg=wg->next) {
+    count++;
+    if (wg->mark == TRUE) continue;
+    for(we=wg->next;we;we=we->next) {
+      if (we->mark == TRUE) continue;
+      /* find the word with exactly the same time and score */
+      if (wg->wid == we->wid &&
+	  wg->lefttime == we->lefttime &&
+	  wg->righttime == we->righttime &&
+	  wg->fscore_head == we->fscore_head &&
+	  wg->fscore_tail == we->fscore_tail) {
+	/* merge contexts */
+	merge_contexts(wg, we);
+	/* swap contexts of left / right contexts */
+	for(i=0;i<we->leftwordnum;i++) {
+	  if (we->leftword[i]->mark) continue;
+	  //if (we->leftword[i] == wg) continue;
+	  swap_rightword(we->leftword[i], we, wg);
+	}
+	for(i=0;i<we->rightwordnum;i++) {
+	  if (we->rightword[i]->mark) continue;
+	  //if (we->rightword[i] == wg) continue;
+	  swap_leftword(we->rightword[i], we, wg);
+	}
+	we->mark = TRUE;
+	merged++;
       }
     }
-  } while (changed == TRUE);
+  }
+
+  erased = wordgraph_exec_erase(rootp);
+
+  for(wg=*rootp;wg;wg=wg->next) {
+    uniq_leftword(wg);
+    uniq_rightword(wg);
+  }
+
+  *rest_ret = count - erased;
+  *merged_ret = merged;
+}
+
+/** 
+ * <JA>
+ * @brief  ոΣñ춭Ĵ
+ * 
+ * GRAPHOUT_PRECISE_BOUNDARY ³ñ˰¸Τñ춭
+ * 뤿ˡõˤơñȤ˼Ÿ
+ * Ūñ춭ư롥ΤᡤñΤġʰưΡ
+ * ȤбȤʤʤΤǡõλ˳ññ
+ * ñ춭¤뤳ȤȤ롥
+ *
+ * ñ춭Τñ֤¤뤿ᡤ٤Ƥñ춭ưʤʤޤ
+ * Ĵ֤롥ʥդǤûñʨǽʤ
+ * 礬뤬ξ GRAPHOUT_LIMIT_BOUNDARY_LOOP ꤹ뤳Ȥǡ
+ * ֤ξ¤ graphout_limit_boundary_loop_num ¤Ǥ롥
+ * 
+ * @param rootp [i/o] ñ쥰դΥ롼ȥΡɤؤΥݥ
+ * </JA>
+ * <EN>
+ * @brief  Post-processing step 2: Adjust word boundaries.
+ * 
+ * When GRAPHOUT_PRECISE_BOUNDARY is defined, the word boundaries will be
+ * moved depending on the later word expansion to get context-dependent
+ * precise boundaries.  So the precise boundary, modified after generation
+ * while search, should be propagated to the context words in the post
+ * processing.
+ *
+ * Since the affect of word boundaries may propagate to the context words,
+ * the adjustment procedure has to be executed iteratively until all the
+ * boundaries are fixated.  However, when graph is large, the oscillation of
+ * short words will results in very long loop.  By defining
+ * GRAPHOUT_LIMIT_BOUNDARY_LOOP, the number of the adjustment loop can be
+ * up to the number specified by graphout_limit_bounrady_loop_num.
+ * 
+ * @param rootp [i/o] pointer to root node of a word graph
+ * </EN>
+ */
+void
+wordgraph_adjust_boundary(WordGraph **rootp)
+{
+#ifdef GRAPHOUT_PRECISE_BOUNDARY
+  WordGraph *wg;
+  int mov_num, dup_num, del_num, mod_num;
+  int count, merged;
+  boolean flag;
+  int loopcount;
+
+  loopcount = 0;
+
+  j_printf("Step 2: Fit word boundaries\n");
+  mov_num = dup_num = del_num = 0;
+
+  /* count number of all words */
+  count = 0;
+  for(wg=*rootp;wg;wg=wg->next) count++;
+
+  do {
+    /* do adjust */
+    flag = wordgraph_adjust_boundary_sub(rootp, &mov_num, &dup_num, &del_num, &mod_num, count);
+    /* do compaction */
+    wordgraph_compaction_thesame_sub(rootp, &count, &merged);
+    /*j_printf("- %d moved, %d duplicated, %d purged (bad align), %d words left\n", mov_num, dup_num, del_num, count + dup_num - del_num);*/
+    j_printf("- #%d: %d moved, %d duplicated, %d purged, %d modified, %d idential, %d left\n", loopcount + 1, mov_num, dup_num, del_num, mod_num, merged, count);
+#ifdef GRAPHOUT_LIMIT_BOUNDARY_LOOP
+    if (++loopcount >= graphout_limit_boundary_loop_num) {
+      j_printf("*** loop count reached %d, terminate loop now\n", graphout_limit_boundary_loop_num);
+      break;
+    }
+#endif
+  } while (flag);
+
   /* execute erase of marked words */
   wordgraph_exec_erase(rootp);
-  free(framelist);
-  free(framescorelist);
-  j_printf("- %d moved, %d duplicated, %d purged (bad align), %d words left in lattice\n", mov_num, dup_num, del_num, count + dup_num - del_num);
 
 #else
 
-  j_printf("# Step 2: SKIP (adjusting re-aligned word boundaries)\n");
+  j_printf("# Step 2: SKIP (adjusting word boundaries)\n");
 
 #endif /* GRAPHOUT_PRECISE_BOUNDARY */
 
@@ -1025,43 +1512,11 @@ wordgraph_adjust_boundary(WordGraph **rootp)
 void
 wordgraph_compaction_thesame(WordGraph **rootp)
 {
-  WordGraph *wg, *we;
-  int i, count, erased;
-  WordGraph *wtmp;
+  int rest, erased;
 
   j_printf("Step 3: Merge same words with exactly same time and score\n");
-
-  count = 0;
-  for(wg=*rootp;wg;wg=wg->next) {
-    count++;
-    if (wg->mark == TRUE) continue;
-    for(we=wg->next;we;we=we->next) {
-      if (we->mark == TRUE) continue;
-      /* find the word with exactly the same time and score */
-      if (wg->wid == we->wid &&
-	  wg->lefttime == we->lefttime &&
-	  wg->righttime == we->righttime &&
-	  wg->fscore_head == we->fscore_head &&
-	  wg->fscore_tail == we->fscore_tail) {
-	/* merge contexts */
-	merge_contexts(wg, we);
-	/* swap contexts of left / right contexts */
-	for(i=0;i<we->leftwordnum;i++) {
-	  swap_rightword(we->leftword[i], we, wg);
-	}
-	for(i=0;i<we->rightwordnum;i++) {
-	  swap_leftword(we->rightword[i], we, wg);
-	}
-	we->mark = TRUE;
-      }
-    }
-  }
-  erased = wordgraph_exec_erase(rootp);
-  j_printf("- %d words merged, %d words left in lattice\n", erased, count - erased);
-  for(wg=*rootp;wg;wg=wg->next) {
-    uniq_leftword(wg);
-    uniq_rightword(wg);
-  }
+  wordgraph_compaction_thesame_sub(rootp, &rest, &erased);
+  j_printf("- %d words merged, %d words left in lattice\n", erased, rest);
 }
 
 /** 
@@ -1070,7 +1525,8 @@ wordgraph_compaction_thesame(WordGraph **rootp)
  * 
  * ñ춭郎פƱñɤĤ«ͤ롥
  * ƱǤʤƤ«ͤ롥ξ硤ʬʸǤ⤤䤬
- * Ĥ롥
+ * Ĥ롥graph_merge_neighbor_range   ξϼ¹Ԥʤ
+
  * 
  * @param rootp [i/o] ñ쥰դΥ롼ȥΡɤؤΥݥ
  * </JA>
@@ -1079,7 +1535,8 @@ wordgraph_compaction_thesame(WordGraph **rootp)
  * 
  * This function bundles the same words which have exactly the same
  * boundaries, allowing having different scores.  The word with
- * the best partial sentence score will be adopted.
+ * the best partial sentence score will be adopted.  This function
+ * will not take effect when graph_merge_neightbor_range is lower than 0.
  * 
  * @param rootp [i/o] pointer to root node of a word graph
  * </EN>
@@ -1091,6 +1548,11 @@ wordgraph_compaction_exacttime(WordGraph **rootp)
   int i, count, erased;
   WordGraph *wtmp;
 
+  if (graph_merge_neighbor_range < 0) {
+    j_printf("# Step 4: SKIP (Merge same words at same time, keep max score)\n");
+    return;
+  }
+
   j_printf("Step 4: Merge same words at same time, keep max score\n");
 
   count = 0;
@@ -1143,7 +1605,7 @@ wordgraph_compaction_exacttime(WordGraph **rootp)
  * @brief  ոΣñ«͡ʶ˵֡
  * 
  * ñ춭ƱñɤĤ«ͤ롥
- * graph_merge_neighbor_range Ϳ롥
+ * graph_merge_neighbor_range Ϳ줬 0 Ǥϼ¹Ԥʤ
  * 
  * @param rootp [i/o] ñ쥰դΥ롼ȥΡɤؤΥݥ
  * </JA>
@@ -1153,6 +1615,8 @@ wordgraph_compaction_exacttime(WordGraph **rootp)
  * This function bundles the same words which appears at similar place.
  * If the difference of both the left boundary and right right boundary
  * is under graph_merge_neighbor_range, it will be bundled.
+ * If its value is lower than or equal to 0, this function does not take
+ * effect.
  * 
  * @param rootp [i/o] pointer to root node of a word graph
  * </EN>
@@ -1162,9 +1626,8 @@ wordgraph_compaction_neighbor(WordGraph **rootp)
 {
   WordGraph *wg, *we;
   int i, count, erased;
-  WordGraph *wtmp;
 
-  if (graph_merge_neighbor_range == 0) {
+  if (graph_merge_neighbor_range <= 0) {
     j_printf("# Step 5: SKIP (merge same words at near place)\n");
     return;
   }
@@ -1409,6 +1872,31 @@ wordgraph_check_merge(WordGraph *now, WordGraph **root, WORD_ID next_wid, boolea
 	*merged_p = TRUE;
       }
 #endif /* GRAPHOUT_SEARCH */
+#ifdef GRAPHOUT_OVERWRITE
+      /*  if current hypothesis score is higher than saved,
+	  overwrite the scores and not terminate */
+      if (
+#ifdef GRAPHOUT_OVERWRITE_GSCORE
+	  wg->gscore_head < now->gscore_head
+#else
+	  wg->fscore_head < now->fscore_head
+#endif
+	  ) {
+	  wg->fscore_head = now->fscore_head;
+	  wg->fscore_tail = now->fscore_tail;
+	  wg->gscore_head = now->gscore_head;
+	  wg->gscore_tail = now->gscore_tail;
+#ifdef USE_NGRAM
+	  wg->lscore = now->lscore;
+#endif
+#ifdef CM_SEARCH
+	  wg->cmscore = now->cmscore;
+#endif
+#ifdef GRAPHOUT_SEARCH
+	  *merged_p = FALSE;
+#endif
+      }
+#endif /* GRAPHOUT_OVERWRITE */
       /* the merged word should be discarded for later merging from
 	 another word, so disable this */
       now->purged = TRUE;
