@@ -1,38 +1,90 @@
+/**
+ * @file   hmm_check.c
+ * @author Akinobu LEE
+ * @date   Thu Mar 17 20:50:07 2005
+ * 
+ * <JA>
+ * @brief  ȥ饤եǥΥǥХåѴؿ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Functions for checking triphone model
+ * </EN>
+ * 
+ * $Revision: 1.2 $
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* hmm_check.c --- debug routine for triphone model check */
-
-/* $Id: hmm_check.c,v 1.6 2003/09/29 06:01:22 ri Exp $ */
-
 #include <julius.h>
 
-
-#define PHSTEP 10
+#define PHSTEP 10		///< Malloc step for phoneme conversion
 
-/* $B2;AGL>"*(BHMM$BL>JQ49%A%'%C%+!<(B
-
-   $B2;AGNs$NJ8;zNs(B($B6uGr6h@Z$j(B)$B$r(B triphone HMM $BNs$KJQ49$9$k!%(B
-   $BC18l6h@Z$j(B('|')$B$rF~$l$?>l9g$OC18lFb(B/$BC18l4V$r9MN8$9$k!%(B
-   $B$3$l$G<B:]$K$I$N$h$&$J(B triphone $B$,E,MQ$5$l$k$N$+$r%A%'%C%/$G$-$k!%(B
-
-   $BK\Mh$N(Btriphone$BI=5-"*(Blogical HMM$B"*(Bphysical/pseudo HMM $B$N%^%C%T%s%0$r(B
-   $BI8=`=PNO$K=PNO$9$k!%(B */
-/* phoneme -> HMM conversion checker: convert phoneme string (each separated
-   by space, word boundary by '|') into HMM sequence.  Inter-word/cross-word
-   triphone conversion is considered, and the logical phone's mapping to
-   pseudo/physical HMM is output in stdout. */
+/** 
+ * <JA>
+ * @brief  󤫤HMMؤѴԤʤ̤ɽ롥
+ *
+ * Υ롼ϡJulius/Julian Ϳ줿ǥ
+ * HMMList եˤơ󤫤HMMؤѴƥȤ
+ * δؿǤ롥
+ * 
+ * Ƕڤ줿ʸФơȥ饤եǥѻˤ
+ * ƥȤθ졤ǽŪб HMM Ѵ롥
+ * θ塤Ѵ̤
+ *   - 󤫤ƳŬѤ٤ǥ̾
+ *   - 嵭 HMMList ˤäѴ HMM ̾
+ *   - ºݤ˷׻ѤʪHMM̾ޤ pseudo HMM ̾
+ * ν˽Ϥ롥
+ * 
+ * ʤʸ "|" ޤ뤳ȤǡñڤȤư
+ * ȥ饤եˤñ֤Ÿθ뤳ȤǤ롥
+ * 
+ * @param str [i/o] Ƕڤ줿ʸ
+ * @param hmminfo [in] HMM¤
+ * @param len_ret [out] ֤ͤ HMM ǿ
+ * 
+ * @return ˥դ줿ѴHMMΥݥ
+ * </JA>
+ * <EN>
+ * @brief  Convert phoneme sequences to logical HMM sequences, and output the
+ * result.
+ *
+ * This function is for testing procedure to convert words in dictionary
+ * to corresponding HMM sequences in Julius/Julian, given an HMMList and
+ * HTK HMM definition.
+ *
+ * Given a space-separated list of phoneme names in a string, each phonemes
+ * will be converted to context-dependent style (if using triphone model),
+ * and then converted to HMM sequence that will finally be used for
+ * recognition.  Then, the following data will be output for all HMM:
+ *   - Original phone HMM name,
+ *   - Logical HMM name that is converted from above,
+ *   - Physical or pseudo HMM name that will actually be used.
+ *
+ * Additionally, specifying '|' in the string gives a word boundary between
+ * phonemes, and triphone conversion will consider the cross-word expansion.
+ * 
+ * @param str [i/o] string that contains space-saparated phoneme sequence.
+ * @param hmminfo [in] HMM definition structure
+ * @param len_ret [out] num of elements in the return value
+ * 
+ * @return the newly allocated pointer array to the converted logical HMMs.
+ * </EN>
+ */
 static HMM_Logical **
-new_str2phseq(char *str, int *len_return, HTK_HMM_INFO *hmminfo)
+new_str2phseq(char *str, HTK_HMM_INFO *hmminfo, int *len_ret)
 {
   char **tokens;
   boolean *word_end;
   int phnum;
   boolean word_mode = FALSE;
   HMM_Logical **new;
+  static char buf[MAX_HMMNAME_LEN];
   
   /* read in string and divide into token unit */
   {
@@ -89,7 +141,6 @@ new_str2phseq(char *str, int *len_return, HTK_HMM_INFO *hmminfo)
     int i;
     char *hmmstr;
     HMM_Logical *lg;
-    char buf[50];
     boolean ok_flag = TRUE;
 
     new = (HMM_Logical **)mymalloc(sizeof(HMM_Logical *) * phnum);
@@ -178,42 +229,67 @@ new_str2phseq(char *str, int *len_return, HTK_HMM_INFO *hmminfo)
     free(word_end);
   }
 
+  *len_ret = phnum;
+
   return new;
 }
 
-/* read in 1 line as phoneme sequence and try to convert it to triphone */
+/** 
+ * <JA>
+ * ɸϤ1ԤɽȤɤ߹ߡȥ饤եؤѴå
+ * Ԥʤ
+ * 
+ * @param hmminfo [in] HMM¤
+ * </JA>
+ * <EN>
+ * Read in line from stdin as phoneme sequence and try convertion to
+ * triphone for checking.
+ * 
+ * @param hmminfo [in] HMM definition structure
+ * </EN>
+ */
 static void
 test_expand_triphone(HTK_HMM_INFO *hmminfo)
 {
   char *buf;
   int newline;
-  HMM *whmm;
   HMM_Logical **phseq;
   int phlen;
 
+  buf = (char *)mymalloc(4096);
   for(;;) {
     /* read in phoneme sequence from stdin */
     j_printf(">>> input phone sequence (word delimiter is `|', blank to return)\n");
-    buf = (char *)mymalloc(4096);
     if (fgets(buf, 4096, stdin) == NULL) break;
     newline = strlen(buf)-1;    /* chop newline */
     if (buf[newline] == '\n') buf[newline] = '\0';
     if (buf[0] == '\0') break;
     /* convert string to phseq and output */
-    phseq = new_str2phseq(buf, &phlen, hmminfo);
+    phseq = new_str2phseq(buf, hmminfo, &phlen);
     if (phseq == NULL) continue;
-    /* make HMM from phseq */
-    /*whmm = new_make_word_hmm(hmminfo, phseq, phlen);*/
   }
   free(buf);
 }
 
-/* interactive triphone conversion check for given hmmdefs/hmmlist */
+/** 
+ * <JA>
+ * ޥɥ饤ǥȥ饤եΥåԤʤ⡼ ("-check hmm")
+ * 
+ * @param hmminfo [in] HMM¤
+ * @param winfo [in] ñ켭
+ * </JA>
+ * <EN>
+ * Mode to do interactive triphone conversion check ("-check hmm").
+ * 
+ * @param hmminfo [in] HMM definition structure
+ * @param winfo [in] word dictionary
+ * </EN>
+ */
 void
 hmm_check(HTK_HMM_INFO *hmminfo, WORD_INFO *winfo)
 {
   boolean endflag;
-  char cmd[50];
+  static char cmd[MAX_HMMNAME_LEN];
   int newline;
   
   j_printf("\n\n");
@@ -235,7 +311,7 @@ hmm_check(HTK_HMM_INFO *hmminfo, WORD_INFO *winfo)
 
   for(endflag = FALSE; endflag == FALSE;) {
     j_printf("===== command (\"H\" for help) > ");
-    if (fgets(cmd, 50, stdin) == NULL) continue;
+    if (fgets(cmd, MAX_HMMNAME_LEN, stdin) == NULL) continue;
     newline = strlen(cmd)-1;    /* chop newline */
     if (cmd[newline] == '\n') cmd[newline] = '\0';
     if (cmd[0] == '\0') continue; /* if blank line, read next */
