@@ -11,13 +11,13 @@
  * @brief  Read all models and build data for recognition.
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.4 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -40,40 +40,30 @@ initialize_HMM()
   hmminfo = hmminfo_new();
   /* load hmmdefs */
   init_hmminfo(hmminfo, hmmfilename, mapfilename);
-  /* check training parameter type */
-  /* for direct speech input,
-     only MFCC_{0|E}_D[_Z][_N] with {25|26} dimension is supported */
+  /* check parameter type of this acoustic HMM */
+  /* only MFCC is supported for audio input */
+  /* MFCC_{0|E}[_D][_A][_Z][_N] is supported */
+  /* Decode parameter extraction type according to the training
+     parameter type in the header of the given acoustic HMM */
   if (speech_input != SP_MFCFILE) {
     if ((hmminfo->opt.param_type & F_BASEMASK) != F_MFCC) {
       j_error("Error: for direct speech input, only HMM trained by MFCC is supported\n");
     }
-    if (! (hmminfo->opt.param_type & F_DELTA)) {
-      j_error("Error: for direct speech input, delta coef should be used in this HMM\n");
-    }
-    if (hmminfo->opt.param_type & F_ACCL) {
-      j_error("Error: for direct speech input, accel coef should not be used in this HMM\n");
-    }
-    if (hmminfo->opt.param_type & F_ENERGY) {
-      c0_required = FALSE;
-    } else if (hmminfo->opt.param_type & F_ZEROTH) {
-      c0_required = TRUE;
-    } else {
-      j_error("Error: for direct speech input, either _E or _0 should be used in this HMM\n");
-    }
-    if (hmminfo->opt.param_type & F_ENERGY_SUP) {
-      dim = 25;
-    } else {
-      dim = 26;
-    }
-    if (hmminfo->opt.vec_size != dim) {
-      j_error("Error: for direct speech input, MFCC dimension must be 13 in this HMM\n");
-    }
-    /* check if CMN is required on this model */
-    if (hmminfo->opt.param_type & F_CEPNORM) {
-      cmn_required = TRUE;
-    } else {
-      cmn_required = FALSE;
-    }
+    /* decode required parameter extraction types */
+    delta_required = (hmminfo->opt.param_type & F_DELTA) ? TRUE : FALSE;
+    acc_required = (hmminfo->opt.param_type & F_ACCL) ? TRUE : FALSE;
+    energy_required = (hmminfo->opt.param_type & F_ENERGY) ? TRUE : FALSE;
+    c0_required = (hmminfo->opt.param_type & F_ZEROTH) ? TRUE : FALSE;
+    abs_energy_suppress = (hmminfo->opt.param_type & F_ENERGY_SUP) ? TRUE : FALSE;
+    cmn_required = (hmminfo->opt.param_type & F_CEPNORM) ? TRUE : FALSE;
+    /* guess MFCC dimension from the vector size and parameter type in the
+       acoustic HMM */
+    dim = hmminfo->opt.vec_size;
+    if (abs_energy_suppress) dim++;
+    dim /= 1 + (delta_required ? 1 : 0) + (acc_required ? 1 : 0);
+    if (energy_required) dim--;
+    if (c0_required) dim--;
+    model_mfcc_dim = dim;
   }
   /* check if tied_mixture */
   if (hmminfo->is_tied_mixture && hmminfo->codebooknum <= 0) {
@@ -86,6 +76,9 @@ initialize_HMM()
     if (make_cdset(hmminfo) == FALSE) {
       j_error("\nError: failed to make context-dependent state set\n");
     }
+    /* add those `pseudo' biphone and monophone to the logical HMM names */
+    /* they points not to the defined HMM, but to the CD_Set structure */
+    hmm_add_pseudo_phones(hmminfo);
     j_printerr("done\n");
   }
 #endif
