@@ -1,29 +1,55 @@
+/**
+ * @file   m_info.c
+ * @author Akinobu Lee
+ * @date   Thu May 12 14:14:01 2005
+ * 
+ * <JA>
+ * @brief  ưǧƥϤ롥
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Output all information of recognition system to standard out.
+ * </EN>
+ * 
+ * $Revision: 1.6 $
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* m_info.c --- output system information */
-
-/* $Id: m_info.c,v 1.32 2004/01/21 12:23:12 ri Exp $ */
-
 #include <julius.h>
 
-/* output filenames */
+/** 
+ * <JA>
+ * ɤ߹ǥե̾Ϥ롥
+ * 
+ * </JA>
+ * <EN>
+ * Output all file names of the models.
+ * 
+ * </EN>
+ */
 void
 print_setting()
 {
 #ifdef USE_NETAUDIO
   char *p;
 #endif
+#ifdef USE_DFA
+  GRAMLIST *g;
+  int n;
+#endif
   
   j_printf("    hmmfilename=%s\n",hmmfilename);
   if (mapfilename != NULL) {
     j_printf("    hmmmapfilename=%s\n",mapfilename);
   }
-  j_printf("    vocabulary filename=%s\n",dictfilename);
 #ifdef USE_NGRAM
+  j_printf("    vocabulary filename=%s\n",dictfilename);
   if (ngram_filename != NULL) {
     j_printf("    n-gram  filename=%s (binary format)\n",ngram_filename);
   } else {
@@ -33,14 +59,31 @@ print_setting()
     }
   }
 #else  /* USE_DFA */
-  j_printf("    DFA grammar filename=%s\n", dfa_filename);
+  n = 1;
+  for(g = gramlist_root; g; g = g->next) {
+    j_printf("    grammar #%d:\n", n++);
+    j_printf("        dfa  = %s\n", g->dfafile);
+    j_printf("        dict = %s\n", g->dictfile);
+  }
 #endif
   if (hmm_gs_filename != NULL) {
     j_printf("    hmmfile for Gaussian Selection: %s\n", hmm_gs_filename);
   }
+  if (gmm_filename != NULL) {
+    j_printf("    GMM file for utterance verification: %s\n", gmm_filename);
+  }
 }
 
-/* print full system information */
+/** 
+ * <JA>
+ * ƤΥƥϤ롥
+ * 
+ * </JA>
+ * <EN>
+ * Output full system information.
+ * 
+ * </EN>
+ */
 void
 print_info()
 {
@@ -63,29 +106,49 @@ print_info()
 
   /* for backward compatibility with scoring tool (IPA99)... :-( */
   if (speech_input == SP_RAWFILE) {
-    j_printf("Speech input source: file\n");
+    j_printf("Speech input source: file\n\n");
   } else if (speech_input == SP_MFCFILE) {
-    j_printf("Speech input source: MFCC parameter file (HTK format)\n");
+    j_printf("Speech input source: MFCC parameter file (HTK format)\n\n");
   }
 
   if (speech_input != SP_MFCFILE) {
     j_printf("Acoustic analysis condition:\n");
+    j_printf("\t       parameter = MFCC");
+    if (c0_required) j_printf("_0");
+    if (energy_required) j_printf("_E");
+    if (delta_required) j_printf("_D");
+    if (acc_required) j_printf("_A");
+    if (abs_energy_suppress) j_printf("_N");
+    if (cmn_required) j_printf("_Z");
+    j_printf(" (%d dimension from %d cepstrum)\n", hmminfo->opt.vec_size, model_mfcc_dim);
     j_printf("\t   sample period = %4d ns      (%5d Hz)\n", smpPeriod, smpFreq);
     j_printf("\t     window size = %4d samples (%.1f ms)\n", fsize,
 	     (float)smpPeriod * (float)fsize / 10000.0);
     j_printf("\t     frame shift = %4d samples (%.1f ms)\n", fshift,
 	     (float)smpPeriod * (float)fshift / 10000.0);
-    j_printf("\t    delta window = %4d frames  (%.1f ms) around\n", delwin,
-	     (float)delwin * (float)smpPeriod * (float)fshift / 10000.0);
+    j_printf("\t    pre-emphasis = %.2f\n", preemph);
+    j_printf("\t    # filterbank = %d\n", fbank_num);
+    j_printf("\t   cepst. lifter = %d\n", ceplifter);
+    j_printf("\t      raw energy = %s\n", rawe_required ? "True" : "False");
+    if (enormal_required) {
+      j_printf("\tenergy normalize = True (scale = %.1f, silence floor = %.1f dB)\n", enormal_escale, enormal_silfloor);
+    } else {
+      j_printf("\tenergy normalize = False\n");
+    }
+    if (delta_required) {
+      j_printf("\t    delta window = %d frames (%.1f ms) around\n", delwin,
+	       (float)delwin * (float)smpPeriod * (float)fshift / 10000.0);
+    }
+    if (acc_required) {
+      j_printf("\t      acc window = %d frames (%.1f ms) around\n", accwin,
+	       (float)accwin * (float)smpPeriod * (float)fshift / 10000.0);
+    }
     j_printf("\t        hi freq. = ");
     if (hipass < 0) j_printf("OFF\n"); 
     else j_printf("%5d Hz\n", hipass);
     j_printf("\t        lo freq. = ");
     if (lopass < 0) j_printf("OFF\n"); 
     else j_printf("%5d Hz\n", lopass);
-    j_printf("\t  energy measure = ");
-    if (c0_required) j_printf("(_0) 0'th cepstral\n");
-    else j_printf("(_E) log energy\n");
 
     j_printf("    spectral subtraction = ");
     if (ssload_filename || sscalc) {
@@ -112,13 +175,19 @@ print_info()
     j_printf("GS ");
     print_hmmdef_info(hmm_gs); j_printf("\n");
   }
-  print_voca_info(winfo); j_printf("\n");
-  print_wchmm_info(wchmm); j_printf("\n");
+  if (winfo != NULL) {
+    print_voca_info(winfo); j_printf("\n");
+  }
+  if (wchmm != NULL) {
+    print_wchmm_info(wchmm); j_printf("\n");
+  }
 #ifdef USE_NGRAM
   print_ngram_info(ngram);
 #else  /* USE_DFA */
-  print_dfa_info(dfa);
-  if (debug2_flag) print_dfa_cp(dfa);
+  if (dfa != NULL) {
+    print_dfa_info(dfa);
+    if (debug2_flag) print_dfa_cp(dfa);
+  }
 #endif /* USE_NGRAM */
 
 #ifdef USE_NGRAM
@@ -165,6 +234,12 @@ print_info()
 #else
   j_printf("\t(-cmalpha)CM alpha coef = %f\n", cm_alpha);
 #endif
+#ifdef CM_SEARCH_LIMIT
+  j_printf("\t(-cmthres) CM cut thres = %f for hypo generation\n", cm_cut_thres);
+#endif
+#ifdef CM_SEARCH_LIMIT_POP
+  j_printf("\t(-cmthres2)CM cut thres = %f for popped hypo\n", cm_cut_thres_pop);
+#endif
 #endif /* CONFIDENCE_MEASURE */
   j_printf("\t(-sp)shortpause HMM name= \"%s\" specified", spmodel_name);
   if (hmminfo->sp != NULL) {
@@ -177,7 +252,7 @@ print_info()
   }
   j_printf("\n");
 #ifdef USE_DFA
-  {
+  if (dfa != NULL) {
     int i;
     j_printf("\t  found sp category IDs =");
     for(i=0;i<dfa->term_num;i++) {
@@ -188,39 +263,68 @@ print_info()
     j_printf("\n");
   }
 #endif
+#ifdef MULTIPATH_VERSION
+  if (enable_iwsp) {
+    j_printf("\t inter-word short pause = on (append \"%s\" for each word tail)\n", hmminfo->sp->name);
+    j_printf("\t  sp transition penalty = %+2.1f\n", iwsp_penalty);
+  }
+#endif
 #ifdef USE_NGRAM
   if (enable_iwspword) {
     j_printf("\tIW-sp word added to dict= \"%s\"\n", iwspentry);
   }
 #endif
   
+  if (gmm != NULL) {
+    j_printf("\nUtterance verification by GMM\n");
+    j_printf("           GMM defs file = %s\n", gmm_filename);
+    j_printf("          GMM gprune num = %d\n", gmm_gprune_num);
+    if (gmm_reject_cmn_string != NULL) {
+      j_printf("   GMM names to reject = %s\n", gmm_reject_cmn_string);
+    }
+    j_printf("    ");
+   print_hmmdef_info(gmm);
+  }
+
+  if (realtime_flag && cmn_required) {
+    j_printf("\nMAP-CMN on realtime input: \n");
+    if (cmnload_filename) {
+      if (cmn_loaded) {
+	j_printf("\t      initial CMN param = from \"%s\"\n", cmnload_filename);
+      } else {
+	j_printf("\t      initial CMN param = from \"%s\" (failed, ignored)\n", cmnload_filename);
+      }
+    } else {
+      j_printf("\t      initial CMN param = not specified\n");
+    }
+    j_printf("\t    initial mean weight = %6.2f\n", cmn_map_weight);
+    if (cmn_update) {
+      j_printf("\t       CMN param update = yes, update from last inputs\n");
+    } else {
+      j_printf("\t       CMN param update = no, keep initial\n");
+    }
+    if (cmnsave_filename) {
+      if (realtime_flag) {
+	j_printf("\t      save CMN param to = %s\n", cmnsave_filename);
+      } else {
+	j_printf("\t      save CMN param to = %s (not realtime CMN, ignored)\n", cmnsave_filename);
+      }
+    }
+  }
+
   j_printf("\nSearch parameters: \n");
     
   j_printf("\t      1st pass decoding = ");
   if (force_realtime_flag) j_printf("(forced) ");
   if (realtime_flag) {
     j_printf("on-the-fly");
-    if (cmn_required) j_printf(" with last CMN");
+    if (cmn_required) j_printf(" with MAP-CMN");
     j_printf("\n");
   } else {
     j_printf("batch");
     if (cmn_required) j_printf(" with sentence CMN");
     j_printf("\n");
   }
-  if (cmnload_filename) {
-    if (cmn_loaded) {
-      j_printf("\t initial CMN param from = %s\n", cmnload_filename);
-    } else {
-      j_printf("\t initial CMN param from = %s (failed, ignored)\n", cmnload_filename);
-    }
-  }
-  if (cmnsave_filename) {
-    if (realtime_flag) {
-      j_printf("\t      save CMN param to = %s\n", cmnsave_filename);
-    } else {
-      j_printf("\t      save CMN param to = %s (not realtime CMN, ignored)\n", cmnsave_filename);
-    }
-  }
   j_printf("\t        1st pass method = ");
 #ifdef WPAIR
 # ifdef WPAIR_KEEP_NLIMIT
@@ -236,13 +340,34 @@ print_info()
 #else
   j_printf("generating indexed trellis\n");
 #endif
-  
-  j_printf("\t(-b) trellis beam width = %d\n", trellis_beam_width);
+
+  j_printf("\t(-b) trellis beam width = %d", trellis_beam_width);
+  if (specified_trellis_beam_width == -1) {
+    j_printf(" (-1 or not specified - guessed)\n");
+  } else if (specified_trellis_beam_width == 0) {
+    j_printf(" (0 - full)\n");
+  } else {
+    j_printf("\n");
+  }
   j_printf("\t(-n)search candidate num= %d\n", nbest);
   j_printf("\t(-s)  search stack size = %d\n", stack_size);
   j_printf("\t(-m)    search overflow = after %d hypothesis poped\n", hypo_overflow);
+  j_printf("\t        2nd pass method = ");
+#ifdef GRAPHOUT
+#ifdef GRAPHOUT_DYNAMIC
+#ifdef GRAPHOUT_SEARCH
+  j_printf("searching graph, generating dynamic graph\n");
+#else
+  j_printf("searching sentence, generating dynamic graph\n");
+#endif /* GRAPHOUT_SEARCH */
+#else  /* ~GRAPHOUT_DYNAMIC */
+  j_printf("searching sentence, generating static graph from N-best\n");
+#endif
+#else  /* ~GRAPHOUT */
+  j_printf("searching sentence, generating N-best\n");
+#endif
   if (enveloped_bestfirst_width >= 0) {
-  j_printf("\t(-b2)  pass2 beam width = %d\n", enveloped_bestfirst_width);
+    j_printf("\t(-b2)  pass2 beam width = %d\n", enveloped_bestfirst_width);
   }
   j_printf("\t(-lookuprange)lookup range= %d  (tm-%d <= t <tm+%d)\n",lookup_range,lookup_range,lookup_range);
 #ifdef SCAN_BEAM
@@ -264,6 +389,35 @@ print_info()
 
   j_printf("\t(-n)        search till = %d candidates found\n", nbest);
   j_printf("\t(-output)    and output = %d candidates out of above\n", output_hypo_maxnum);
+#ifdef GRAPHOUT
+  j_printf("\t(-graphrange)    margin = %d frames", graph_merge_neighbor_range);
+  if (graph_merge_neighbor_range < 0) {
+    j_printf(" (distinguish same words with different scores)\n");
+  } else if (graph_merge_neighbor_range == 0) {
+    j_printf(" (merge same words at the same time)\n");
+  } else {
+    j_printf(" (merge same words around this frame)\n");
+  }
+#ifdef GRAPHOUT_DEPTHCUT
+  j_printf("\t(-graphcut)cutoff depth = ");
+  if (graphout_cut_depth < 0) {
+    j_printf("disabled (-1)\n");
+  } else {
+    j_printf("%d words\n",graphout_cut_depth);
+  }
+#endif
+#ifdef GRAPHOUT_LIMIT_BOUNDARY_LOOP
+  j_printf("\t(-graphboundloop)loopmax= %d for boundary adjustment\n",graphout_limit_boundary_loop_num);
+#endif
+#ifdef GRAPHOUT_SEARCH_DELAY_TERMINATION
+  j_printf("\tInhibit graph search termination before 1st sentence found = ");
+  if (graphout_search_delay) {
+    j_printf("enabled\n");
+  } else {
+    j_printf("disabled\n");
+  }
+#endif
+#endif /* GRAPHOUT */
   
   if (ccd_flag) {
     j_printf("\t IWCD handling:\n");
@@ -417,8 +571,11 @@ print_info()
     } else {
       j_printf("\t        silence cutting = off\n");
     }
-    if (use_zmean) {
+    if (use_zmean || zmean_frame) {
       j_printf("\t       remove DC offset = on");
+      if (zmean_frame) {
+	j_printf(" (frame-wise)\n");
+      }
       if (speech_input == SP_RAWFILE) {
 	j_printf(" (will compute for each file)\n");
       } else {
@@ -429,6 +586,12 @@ print_info()
       j_printf("\t       remove DC offset = off\n");
     }
   }
+  j_printf("\t     reject short input = ");
+  if (rejectshortlen > 0) {
+    j_printf("< %d msec\n", rejectshortlen);
+  } else {
+    j_printf("off\n");
+  }
 #ifdef SP_BREAK_CURRENT_FRAME
   j_printf("\tshort pause segmentation= on\n");
   j_printf("\t     sp duration length = %d frames\n", sp_frame_duration);
@@ -450,6 +613,19 @@ print_info()
       j_printf("\tspeech data stored to = %s/\n", record_dirname);
     }
   }
+  j_printf("\t   output charset conv. = ");
+#ifdef CHARACTER_CONVERSION
+  if (to_code == NULL) {
+    j_printf("disabled\n");
+  } else {
+    if (from_code != NULL) {
+      j_printf("from \"%s\" ", from_code);
+    }
+    j_printf("to \"%s\"\n", to_code);
+  }
+#else
+  j_printf("not supported\n");
+#endif /* CHARACTER_CONVERSION */
   j_printf("\n------------- System Info end -------------\n");
 
 #ifdef USE_MIC
