@@ -1,31 +1,86 @@
+/**
+ * @file   m_options.c
+ * @author Akinobu Lee
+ * @date   Thu May 12 18:52:07 2005
+ * 
+ * <JA>
+ * @brief  jconf䥳ޥɤΥץ롥
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Process options and configurations from jconf or command argument.
+ * </EN>
+ * 
+ * $Revision: 1.6 $
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* m_options.c --- process options and set values */
-
-/* $Id: m_options.c,v 1.21 2004/01/08 08:42:17 ri Exp $ */
-
 #include <julius.h>
 
-/* change relative path to absolute path */
-static char *
+/** 
+ * <JA>
+ * @brief  ХѥեѥѴ롥
+ * 
+ * եΥѥ̾ХѥǤСȥǥ쥯ȥĤ
+ * եѥѴ֤ХѥǤСΤޤ֤
+ * 
+ * @param filename [in] եΥѥ̾
+ * @param dirname [in] ȥǥ쥯ȥΥѥ̾
+ * 
+ * @return Хѥ̾ä˳դ줿Хåե
+ * </JA>
+ * <EN>
+ * @brief  Change relative path to full path.
+ *
+ * If the file path is given as relative, prepend the dirname to it.
+ * If the file path is full, just copy it to new buffer and return.
+ * 
+ * @param filename [in] file path name
+ * @param dirname [in] full path of current directory
+ * 
+ * @return newly malloced buffer holding the full path name.
+ * </EN>
+ */
+char *
 filepath(char *filename, char *dirname)
 {
   char *p;
-  if (dirname != NULL && filename[0] != '/') {
+  if (dirname != NULL && filename[0] != '/'
+#if defined(_WIN32)
+      && filename[0] != '\\' && !(strlen(filename) >= 3 && filename[1] == ':')
+#endif
+      ) {
     p = (char *)mymalloc(strlen(filename) + strlen(dirname) + 1);
     strcpy(p, dirname);
     strcat(p, filename);
   } else {
-    p = filename;
+    p = strcpy((char *)mymalloc(strlen(filename)+1), filename);
   }
   return p;
 }
 
-/* error exit when required argument was not specified */
+/** 
+ * <JA>
+ * ɬפʰͿʤä˥顼λ롥
+ * 
+ * @param opt [in] ץʸ
+ * 
+ * @return NULL.
+ * </JA>
+ * <EN>
+ * Error exit when required argument was not specified.
+ * 
+ * @param opt [in] option string
+ * 
+ * @return NULL.
+ * </EN>
+ */
 static char *
 args_needed_exit(char *opt)
 {
@@ -34,14 +89,29 @@ args_needed_exit(char *opt)
   return(NULL);
 }
 
-/* parse options and set values */
+/** 
+ * <JA>
+ * @brief  ץϤбͤ򥻥åȤ롥
+ *
+ * @param argc [in] @a argv ˴ޤޤο
+ * @param argv [in] ɽʸ
+ * @param cwd [in] ȥǥ쥯ȥʸ
+ * </JA>
+ * <EN>
+ * Parse options and set values.
+ * 
+ * @param argc [in] number of elements in @a argv
+ * @param argv [in] array of argument strings
+ * @param cwd [in] current directory string
+ * </EN>
+ */
 void
 opt_parse(int argc, char *argv[], char *cwd)
 {
   char *tmparg;
   int i;
 
-#define NEXTARG (++i >= argc) ? (char *)args_needed_exit(argv[i-1]) : argv[i]
+#define NEXTARG (++i >= argc) ? (char *)args_needed_exit(argv[i-1]) : argv[i] ///< Points to next argument, or error if not exist
 
   if (argc == 1) {		/* no argument */
     usage();
@@ -92,6 +162,9 @@ opt_parse(int argc, char *argv[], char *cwd)
     } else if (strmatch(argv[i],"-record")) {	/* record speech data*/
       record_dirname = NEXTARG;
       continue;
+    } else if (strmatch(argv[i],"-rejectshort")) { /* short input rejection */
+      rejectshortlen = atoi(NEXTARG);
+      continue;
     } else if (strmatch(argv[i],"-module")) { /* enable module mode */
       module_mode = TRUE;
       if (i+1 < argc) {
@@ -172,10 +245,39 @@ opt_parse(int argc, char *argv[], char *cwd)
     } else if (strmatch(argv[i],"-lookuprange")) { /* trellis neighbor range */
       lookup_range = atoi(NEXTARG);
       continue;
+#ifdef GRAPHOUT
+    } else if (strmatch(argv[i],"-graphrange")) { /* neighbor merge range frame */
+      graph_merge_neighbor_range = atoi(NEXTARG);
+      continue;
+#ifdef GRAPHOUT_DEPTHCUT
+    } else if (strmatch(argv[i],"-graphcut")) { /* cut graph word by depth */
+      graphout_cut_depth = atoi(NEXTARG);
+      continue;
+#endif
+#ifdef GRAPHOUT_LIMIT_BOUNDARY_LOOP
+    } else if (strmatch(argv[i],"-graphboundloop")) { /* neighbor merge range frame */
+      graphout_limit_boundary_loop_num = atoi(NEXTARG);
+      continue;
+#endif
+#ifdef GRAPHOUT_SEARCH_DELAY_TERMINATION
+    } else if (strmatch(argv[i],"-graphsearchdelay")) { /* not do graph search termination before the 1st sentence is found */
+      graphout_search_delay = TRUE;
+      continue;
+    } else if (strmatch(argv[i],"-nographsearchdelay")) { /* not do graph search termination before the 1st sentence is found */
+      graphout_search_delay = FALSE;
+      continue;
+#endif
+#endif /* GRAPHOUT */
 #ifdef USE_DFA
     } else if (strmatch(argv[i],"-looktrellis")) { /* activate loopuprange */
       looktrellis_flag = TRUE;
       continue;
+    } else if (strmatch(argv[i],"-multigramout")) { /* enable per-grammar decoding on 2nd pass */
+      multigramout_flag = TRUE;
+      continue;
+    } else if (strmatch(argv[i],"-nomultigramout")) { /* disable per-grammar decoding on 2nd pass */
+      multigramout_flag = FALSE;
+      continue;
 #endif
 #ifdef CATEGORY_TREE
     } else if (strmatch(argv[i],"-oldtree")) { /* use old tree function */
@@ -241,9 +343,39 @@ opt_parse(int argc, char *argv[], char *cwd)
     } else if (strmatch(argv[i],"-fshift")) { /* Frame shiht */
       fshift = atoi(NEXTARG);
       continue;
+    } else if (strmatch(argv[i],"-preemph")) {
+      preemph = atof(NEXTARG);
+      continue;
+    } else if (strmatch(argv[i],"-fbank")) {
+      fbank_num = atoi(NEXTARG);
+      continue;
+    } else if (strmatch(argv[i],"-ceplif")) {
+      ceplifter = atoi(NEXTARG);
+      continue;
+    } else if (strmatch(argv[i],"-rawe")) {
+      rawe_required = TRUE;
+      continue;
+    } else if (strmatch(argv[i],"-norawe")) {
+      rawe_required = FALSE;
+      continue;
+    } else if (strmatch(argv[i],"-enormal")) {
+      enormal_required = TRUE;
+      continue;
+    } else if (strmatch(argv[i],"-noenormal")) {
+      enormal_required = FALSE;
+      continue;
+    } else if (strmatch(argv[i],"-escale")) {
+      enormal_escale = atof(NEXTARG);
+      continue;
+    } else if (strmatch(argv[i],"-silfloor")) {
+      enormal_silfloor = atof(NEXTARG);
+      continue;
     } else if (strmatch(argv[i],"-delwin")) { /* Delta window length */
       delwin = atoi(NEXTARG);
       continue;
+    } else if (strmatch(argv[i],"-accwin")) { /* Acceleration window length */
+      accwin = atoi(NEXTARG);
+      continue;
     } else if (strmatch(argv[i],"-ssalpha")) { /* alpha coef. for SS */
       ssalpha = atof(NEXTARG);
       continue;
@@ -266,6 +398,37 @@ opt_parse(int argc, char *argv[], char *cwd)
     } else if (strmatch(argv[i],"-proginterval")) { /* interval for -progout */
       progout_interval = atoi(NEXTARG);
       continue;
+    } else if (strmatch(argv[i],"-nocharconv")) { /* disable character set conversion */
+#ifdef CHARACTER_CONVERSION
+      from_code = NULL;
+      to_code = NULL;
+#else
+      j_printerr("Warning: character set conversion disabled, option \"-nocharconv\" ignored\n");
+#endif
+      continue;
+    } else if (strmatch(argv[i],"-charconv")) {	/* enable character set conversion */
+#ifdef CHARACTER_CONVERSION
+      from_code = NEXTARG;
+      to_code = NEXTARG;
+#else
+      j_printerr("Warning: character set conversion disabled, option \"-charconv\" ignored\n");
+      i+=2;
+#endif
+      continue;
+    } else if (strmatch(argv[i],"-kanji")) {
+#ifdef CHARACTER_CONVERSION
+      from_code = NULL;
+      tmparg = NEXTARG;
+      if (strmatch(tmparg, "noconv")) {
+	to_code = NULL;
+      } else {
+	to_code = tmparg;
+      }
+#else
+      j_printerr("Warning: character set conversion disabled, option \"-kanji\" ignored\n");
+      i++;
+#endif
+      continue;
     } else if (strmatch(argv[i],"-demo")) { /* quiet + progout */
       debug2_flag = verbose_flag = FALSE;
       progout_flag = TRUE;
@@ -311,6 +474,23 @@ opt_parse(int argc, char *argv[], char *cwd)
       lm_penalty_trans = (LOGPROB)atof(NEXTARG);
       continue;
 #else  /* USE_DFA */
+    } else if (strmatch(argv[i],"-gram")) { /* comma-separatedlist of grammar prefix */
+      multigram_add_prefix_list(NEXTARG, cwd);
+      continue;
+    } else if (strmatch(argv[i],"-gramlist")) { /* file of grammar prefix list */
+      multigram_add_prefix_filelist(filepath(NEXTARG, cwd));
+      continue;
+    } else if (strmatch(argv[i],"-nogram")) { /* remove grammar list */
+      multigram_remove_gramlist();
+      if (dfa_filename != NULL) {
+	free(dfa_filename);
+	dfa_filename = NULL;
+      }
+      if (dictfilename != NULL) {
+	free(dictfilename);
+	dictfilename = NULL;
+      }
+      continue;
     } else if (strmatch(argv[i],"-dfa")) { /* DFA filename */
       dfa_filename = filepath(NEXTARG, cwd);
       continue;
@@ -324,6 +504,14 @@ opt_parse(int argc, char *argv[], char *cwd)
     } else if (strmatch(argv[i],"-spmodel") || strmatch(argv[i], "-sp")) { /* name of short pause word */
       spmodel_name = NEXTARG;
       continue;
+#ifdef MULTIPATH_VERSION
+    } else if (strmatch(argv[i],"-iwsp")) { /* enable inter-word short pause handing */
+      enable_iwsp = TRUE;
+      continue;
+    } else if (strmatch(argv[i],"-iwsppenalty")) { /* set inter-word short pause transition penalty */
+      iwsp_penalty = atof(NEXTARG);
+      continue;
+#endif
 #ifdef USE_NGRAM
     } else if (strmatch(argv[i],"-silhead")) { /* head silence word name */
       head_silname = NEXTARG;
@@ -351,6 +539,7 @@ opt_parse(int argc, char *argv[], char *cwd)
       separate_wnum = atoi(NEXTARG);
 #else
       j_printerr("Warning: option \"-sepnum\" ignored\n");
+      i++;
 #endif
       continue;
 #endif /* USE_NGRAM */
@@ -371,6 +560,12 @@ opt_parse(int argc, char *argv[], char *cwd)
     } else if (strmatch(argv[i],"-nozmean")) { /* disable DC offset by zero mean */
       use_zmean = FALSE;
       continue;
+    } else if (strmatch(argv[i],"-zmeanframe")) { /* enable frame-wise DC offset by zero mean */
+      zmean_frame = TRUE;
+      continue;
+    } else if (strmatch(argv[i],"-nozmeanframe")) { /* disable frame-wise DC offset by zero mean */
+      zmean_frame = FALSE;
+      continue;
 #ifdef SP_BREAK_CURRENT_FRAME
     } else if (strmatch(argv[i],"-spdur")) { /* short-pause duration threshold */
       sp_frame_duration = atoi(NEXTARG);
@@ -447,6 +642,15 @@ opt_parse(int argc, char *argv[], char *cwd)
     } else if (strmatch(argv[i],"-cmnsave")) { /* save CMN parameter to file */
       cmnsave_filename = filepath(NEXTARG, cwd);
       continue;
+    } else if (strmatch(argv[i],"-cmnupdate")) { /* update CMN parameter */
+      cmn_update = TRUE;
+      continue;
+    } else if (strmatch(argv[i],"-cmnnoupdate")) { /* not update CMN parameter */
+      cmn_update = FALSE;
+      continue;
+    } else if (strmatch(argv[i],"-cmnmapweight")) { /* CMN weight for MAP */
+      cmn_map_weight = (float)atof(NEXTARG);
+      continue;
     } else if (strmatch(argv[i],"-sscalc")) { /* do spectral subtraction (SS) for raw file input */
       sscalc = TRUE;
       ssload_filename = NULL;
@@ -470,7 +674,28 @@ opt_parse(int argc, char *argv[], char *cwd)
       cm_alpha = (LOGPROB)atof(NEXTARG);
 #endif
       continue;
+#ifdef CM_SEARCH_LIMIT
+    } else if (strmatch(argv[i],"-cmthres")) { /* CM cut threshold for CM decoding */
+      cm_cut_thres = (LOGPROB)atof(NEXTARG);
+      continue;
+#endif
+#ifdef CM_SEARCH_LIMIT_POP
+    } else if (strmatch(argv[i],"-cmthres2")) { /* CM cut threshold for CM decoding */
+      cm_cut_thres_pop = (LOGPROB)atof(NEXTARG);
+      continue;
+#endif
 #endif /* CONFIDENCE_MEASURE */
+    } else if (strmatch(argv[i],"-gmm")) { /* load SS parameter from file */
+      gmm_filename = filepath(NEXTARG, cwd);
+      continue;
+    } else if (strmatch(argv[i],"-gmmnum")) { /* num of Gaussian pruning for GMM */
+      gmm_gprune_num = atoi(NEXTARG);
+      continue;
+    } else if (strmatch(argv[i],"-gmmreject")) {
+      tmparg = NEXTARG;
+      gmm_reject_cmn_string = (char *)mymalloc(strlen(tmparg)+1);
+      strcpy(gmm_reject_cmn_string, tmparg);
+      continue;
     } else if (strmatch(argv[i],"-help")) { /* output version and option */
       detailed_help();
     } else if (strmatch(argv[i],"--help")) {
@@ -493,7 +718,7 @@ opt_parse(int argc, char *argv[], char *cwd)
 	break;
 #endif
       case 'b':			/* beam width in 1st pass */
-	trellis_beam_width = atoi(NEXTARG);
+	specified_trellis_beam_width = atoi(NEXTARG);
 	break;
       case 's':			/* stack size in 2nd pass */
 	stack_size = atoi(NEXTARG);
