@@ -1,24 +1,54 @@
+/**
+ * @file   m_usage.c
+ * @author Akinobu Lee
+ * @date   Fri May 13 15:04:34 2005
+ * 
+ * <JA>
+ * @brief  إפɽ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Print help.
+ * </EN>
+ * 
+ * $Revision: 1.8 $
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* m_usage.c --- print help */
-
-/* $Id: m_usage.c,v 1.25 2004/01/08 08:42:17 ri Exp $ */
-
 #include <julius.h>
 
-
-/* error no argument specified */
-/* terminate on option error */
+/** 
+ * <JA>
+ * åϤƥ顼λʥץꥨ顼ѡ
+ * 
+ * </JA>
+ * <EN>
+ * Output error message and error exit (for option parse error)
+ * 
+ * </EN>
+ */
 void
 opt_terminate()
 {
   j_error("Try `-help' for more information.\n");
 }
 
+/** 
+ * <JA>
+ * СȥåϤƽλʥץ̵¹Ի
+ * 
+ * </JA>
+ * <EN>
+ * Output version messages and exit (for exec with no option)
+ * 
+ * </EN>
+ */
 void
 usage()
 {
@@ -28,7 +58,16 @@ usage()
   j_exit();
 }
 
-/* output detailed info */
+/** 
+ * <JA>
+ * С󡤥󥸥ꤪӾܺ٤ʥץΥإפɽƽλ
+ * 
+ * </JA>
+ * <EN>
+ * Output version info, engine setting and detailed help for options, and exit.
+ * 
+ * </EN>
+ */
 void
 detailed_help()
 {
@@ -57,10 +96,12 @@ detailed_help()
 #endif
   j_printf("    [-adport portnum]   adinnet portnum to listen             (%d)\n", adinnet_port);
   j_printf("    [-smpFreq freq]     sample period (Hz)                    (%d)\n", smpFreq);
-  j_printf("    [-smpPeriod period] sample period (ns)                    (%d)\n", smpPeriod);
+  j_printf("    [-smpPeriod period] sample period (100ns)                    (%d)\n", smpPeriod);
   j_printf("    [-zmean/-nozmean]   enable/disable DC offset removal      (OFF)\n");
+  j_printf("    [-zmeanframe/-nozmeanframe] frame-wise DC offset removal  (OFF)\n");
   j_printf("    [-nostrip]          not strip off zero samples\n");
   j_printf("    [-record dir]       save recognized speech data to dir\n");
+  j_printf("    [-rejectshort msec] reject short input\n");
   
   j_printf("\n Speech Detection: (default: on=mic/net off=files)\n");
   /*j_printf("    [-pausesegment]     turn on (force) pause detection\n");*/
@@ -74,10 +115,19 @@ detailed_help()
 
   j_printf("\n Acoustic analysis:\n");
   j_printf("    [-smpFreq freq]     sample period (Hz)                    (%d)\n", smpFreq);
-  j_printf("    [-smpPeriod period] sample period (ns)                    (%d)\n", smpPeriod);
+  j_printf("    [-smpPeriod period] sample period (100ns)                    (%d)\n", smpPeriod);
   j_printf("    [-fsize sample]     window size (sample)                  (%d)\n", fsize);
   j_printf("    [-fshift sample]    frame shift (sample)                  (%d)\n", fshift);
+  j_printf("    [-preemph]          pre-emphasis coef.                    (%.2f)\n", preemph);
+  j_printf("    [-fbank]            number of filterbank channels         (%d)\n", fbank_num);
+  j_printf("    [-ceplif]           cepstral liftering coef.              (%d)\n", ceplifter);
+  j_printf("    [-rawe] [-norawe]    toggle using raw energy              (not use)\n");
+  j_printf("    [-enormal] [-noenormal]  toggle normalizing log energy    (not normalize)\n", preemph);
+  j_printf("                        (cannot set -enormal when live input)\n");
+  j_printf("    [-escale]           scaling log energy for enormal        (%.1f)\n", enormal_escale);
+  j_printf("    [-silfloor]         energy silence floor in dB            (%.1f)\n", enormal_silfloor);
   j_printf("    [-delwin frame]     delta windows length (frame)          (%d)\n", delwin);
+  j_printf("    [-accwin frame]     accel windows length (frame)          (%d)\n", accwin);
   j_printf("    [-hifreq freq]      freq. of upper band limit, off if <0  (%d)\n", hipass);
   j_printf("    [-lofreq freq]      freq. of lower band limit, off if <0  (%d)\n", lopass);
   j_printf("    [-sscalc]           do spectral subtraction (file input only)\n");
@@ -86,6 +136,11 @@ detailed_help()
   j_printf("    [-ssalpha value]    alpha coef. for SS                    (%f)\n", ssalpha);
   j_printf("    [-ssfloor value]    spectral floor for SS                 (%f)\n", ssfloor);
 
+  j_printf("\n GMM utterance verification:\n");
+  j_printf("    -gmm filename       GMM definition file\n");
+  j_printf("    -gmmnum num         GMM Gaussian pruning num              (%d)\n", gmm_gprune_num);
+  j_printf("    -gmmreject string   list of GMMs for rejection (cancel pass2, keep CM)\n");
+
 #ifdef USE_NGRAM
   j_printf("\n N-gram Language model (\"-d\" or \"-nlr -nrl\" needed):\n");
   j_printf("    -d file.bingram     n-gram file name in Julius binary format\n");
@@ -97,6 +152,8 @@ detailed_help()
 #else  /* USE_DFA */
   j_printf("\n Grammar:\n");
   j_printf("    -dfa file.dfa       DFA grammar file\n");
+  j_printf("    -gram file[,file2...] (list of) grammar prefix\n");
+  j_printf("    -gramlist filename  filename of grammar list\n");
   j_printf("    [-penalty1 float]   word insertion penalty (1st pass)     (%.1f)\n", penalty1);
   j_printf("    [-penalty2 float]   word insertion penalty (2nd pass)     (%.1f)\n", penalty1);
 #endif
@@ -160,6 +217,7 @@ detailed_help()
 
   j_printf("\n Search Parameters (First Pass):\n");
   j_printf("    [-b beamwidth]      beam width (by state num)             (guessed)\n");
+  j_printf("                        (0: full search, -1: force guess)\n");
 #ifdef WPAIR
 # ifdef WPAIR_KEEP_NLIMIT
   j_printf("    [-nlimit N]         keeps only N tokens on each state     (%d)\n", wpair_keep_nlimit);
@@ -188,6 +246,7 @@ detailed_help()
   j_printf("    [-lookuprange N]    frame lookup range in word expansion  (%d)\n", lookup_range);
 #ifdef USE_DFA
   j_printf("    [-looktrellis]      expand only backtrellis words\n");
+  j_printf("    [-[no]multigramout] output per-grammar results\n");
 #endif
 #ifdef CATEGORY_TREE
   j_printf("    [-oldtree]          use old build_wchmm()\n");
@@ -195,34 +254,66 @@ detailed_help()
   j_printf("    [-oldiwcd]          use full lcdset\n");
 #endif
 #endif
+#ifdef MULTIPATH_VERSION
+  j_printf("    [-iwsp]             turn on inter-word short pause handling (off)\n");
+  j_printf("    [-iwsppenalty]      trans. penalty for inter-word sp      (%.1f)\n", iwsp_penalty);
+#endif 
+
+#ifdef GRAPHOUT
+  j_printf("\n Graph Output:\n");
+  j_printf("    [-graphrange N]     merge same words in graph (%d)\n", graph_merge_neighbor_range);
+  j_printf("                        -1: not merge, leave same loc. with diff. score\n");
+  j_printf("                         0: merge same words at same location\n");
+  j_printf("                        >0: merge same words around the margin\n");
+#ifdef GRAPHOUT_DEPTHCUT
+  j_printf("    [-graphcut num]     graph cut depth at postprocess (-1: disable)(%d)\n", graphout_cut_depth);
+#endif
+#ifdef GRAPHOUT_LIMIT_BOUNDARY_LOOP
+  j_printf("    [-graphboundloop num] max. num of boundary adjustment loop (%d)\n", graphout_limit_boundary_loop_num);
+#endif
+#ifdef GRAPHOUT_SEARCH_DELAY_TERMINATION
+  j_printf("    [-graphsearchdelay] inhibit search termination until 1st sent. found\n");
+  j_printf("    [-nographsearchdelay] disable it (default)\n");
+#endif
+#endif /* GRAPHOUT */
 
   j_printf("\n On-the-fly Decoding: (default: on=mic/net off=files)\n");
-  j_printf("    [-realtime]         turn on, input streamed with last CMN\n");
+  j_printf("    [-realtime]         turn on, input streamed with MAP-CMN\n");
   j_printf("    [-norealtime]       turn off, input buffered with sentence CMN\n");
   j_printf("    [-cmnload file]     load initial CMN param from file on startup\n");
-  j_printf("    [-cmnsave file]     save CMN param to file (update per input)\n");
+  j_printf("    [-cmnsave file]     save CMN param to file after each input\n");
+  j_printf("    [-cmnnoupdate]      not update CMN param while recog. (use with -cmnload)\n");
+  j_printf("    [-cmnmapweight]     weight value of initial cm for MAP-CMN (%6.2f)\n", cmn_map_weight);
+
   j_printf("\n Forced Alignment:\n");
   j_printf("    [-walign]           optionally output word alignments\n");
   j_printf("    [-palign]           optionally output phoneme alignments\n");
   j_printf("    [-salign]           optionally output state alignments\n");
 #ifdef CONFIDENCE_MEASURE
-#ifdef CM_MULTIPLE_ALPHA
   j_printf("\n Confidence Score:\n");
+#ifdef CM_MULTIPLE_ALPHA
   j_printf("    [-cmalpha f t s]    CM smoothing factor        (from, to, step)\n");
 #else
-  j_printf("\n Confidence Score:\n");
   j_printf("    [-cmalpha value]    CM smoothing factor                    (%f)\n", cm_alpha);
 #endif
+#ifdef CM_SEARCH_LIMIT
+  j_printf("    [-cmthres value]    CM threshold to cut hypo on 2nd pass   (%f)\n", cm_cut_thres);
+#endif
 #endif /* CONFIDENCE_MEASURE */
   j_printf("\n Output:\n");
   j_printf("    [-result devname]   output recognition result to:         (tty)\n");
-  j_printf("             tty        terminal (standard out)\n");
+  j_printf("             tty        plain text to standard out\n");
   j_printf("             msock      module socket\n");
   j_printf("\n Server Module Mode:\n");
   j_printf("    [-module [port]]    enable server module mode             (%d)\n", module_port);
   j_printf("                        (implies \"-result msock\")\n");
+#ifdef CONFIDENCE_MEASURE
+  j_printf("    [-outcode WLPSCwlps] select info to output to the module\n");
+  j_printf("                        (Word,Lang,Phone,Score,Confidence low=1st pass)\n");
+#else
   j_printf("    [-outcode WLPSwlps] select info to output to the module\n");
   j_printf("                        (Word,Lang,Phone,Score, lower == 1st pass)\n");
+#endif
   j_printf("\n Message Output:\n");
 #ifdef USE_NGRAM
   j_printf("    [-separatescore]    output LM and AM score independently\n");
@@ -231,6 +322,13 @@ detailed_help()
   j_printf("    [-progout]          progressive output in 1st pass\n");
   j_printf("    [-proginterval]     interval of progout in msec           (%d)\n", progout_interval);
   j_printf("    [-demo]             equal to \"-quiet -progout\"\n");
+#ifdef CHARACTER_CONVERSION
+  j_printf("    [-charconv from to] convert output character set from [from] to [to]\n");
+#ifdef USE_LIBJCODE
+  j_printf("                        (support only Japanese sets: euc,sjis,jis)\n");
+#endif
+  j_printf("    [-nocharconv]       disable output code conversion (default)\n");
+#endif
   j_printf("\n Others:\n");
   j_printf("    [-C jconffile]      load options from jconf file\n");
   j_printf("    [-debug]            (for debug) dump numerous log\n");
