@@ -11,20 +11,20 @@
  * @brief  Main loop for module mode (server mode)
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.4 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
 #include <julius.h>
 #include <stdarg.h>
 
-#define MAXBUFLEN 4096
+#define MAXBUFLEN 4096 ///< Maximum line length of a message sent from a client
 static int listen_sd = -1;	///< Socket to listen to a client
 static char mbuf[MAXBUFLEN];	///< Work buffer for message output
 
@@ -328,8 +328,9 @@ msock_exec_command(char *command)
 #ifdef USE_DFA
   DFA_INFO *new_dfa;
   WORD_INFO *new_winfo;
-  static char *buf = NULL, *p;
+  static char *buf = NULL, *p, *q;
   int gid;
+  char namebuf[MAXGRAMNAMELEN];
 
   if (buf == NULL) buf = mymalloc(MAXBUFLEN);
 #endif
@@ -360,28 +361,36 @@ msock_exec_command(char *command)
 		PRODUCTNAME, VERSION, SETUP);
   } else if (strmatch(command, "PAUSE")) {
     /* pause recognition: will stop when the current input ends */
-    process_want_terminate = FALSE;
-    process_want_reload = TRUE;
-    process_active = FALSE;
+    if (process_active) {
+      process_want_terminate = FALSE;
+      process_want_reload = TRUE;
+      process_active = FALSE;
+    }
     if (speech_input == SP_ADINNET) {
       /* when taking speech from adinnet client,
-	 tell the client to stop recording */
+	 always tell the client to stop recording */
       adin_tcpip_send_pause();
     }
   } else if (strmatch(command, "TERMINATE")) {
     /* terminate recognition: input will terminate immidiately */
-    process_want_terminate = TRUE;
-    process_want_reload = TRUE;
-    process_active = FALSE;
+    /* set flags to stop adin to terminate immediately, and
+       stop process */
+    if (process_active) {
+      process_want_terminate = TRUE;
+      process_want_reload = TRUE;
+      process_active = FALSE;
+    }
     if (speech_input == SP_ADINNET) {
-      /* when taking speech from adinnet client,
-	 tell the client to stop recording */
-      adin_tcpip_send_pause();
+      /* when taking speech input from adinnet client,
+	 always tell the client to stop recording immediately */
+      adin_tcpip_send_terminate();
     }
   } else if (strmatch(command, "RESUME")) {
     /* restart the paused/terminated recognition process */
-    process_want_terminate = FALSE;
-    process_active = TRUE;
+    if (process_active == FALSE) {
+      process_want_terminate = FALSE;
+      process_active = TRUE;
+    }
     if (speech_input == SP_ADINNET) {
       /* when taking speech from adinnet client,
 	 tell the client to restart recording */
@@ -408,8 +417,19 @@ msock_exec_command(char *command)
     } else {
       j_error("Error: msock(INPUTONCHANGE): unknown method [%s]\n", buf);
     }
-  } else if (strmatch(command, "CHANGEGRAM")) {
+  } else if (strnmatch(command, "CHANGEGRAM", strlen("CHANGEGRAM"))) {
     /* receive grammar (DFA + DICT) from the socket, and swap the whole grammar  */
+    /* read grammar name if any */
+    p = &(command[strlen("CHANGEGRAM")]);
+    while (*p == ' ' && *p != '\r' && *p != '\n' && *p != '\0') p++;
+    if (*p != '\r' && *p != '\n' && *p != '\0') {
+      q = buf;
+      while (*p != ' ' && *p != '\r' && *p != '\n' && *p != '\0') *q++ = *p++;
+      *q = '\0';
+      p = buf;
+    } else {
+      p = NULL;
+    }
     /* read a new grammar via socket */
     if (msock_read_grammar(&new_dfa, &new_winfo) == FALSE) {
       module_send(module_sd, "<GRAMMAR STATUS=\"ERROR\" REASON=\"WRONG DATA\"/>\n.\n");
@@ -417,20 +437,31 @@ msock_exec_command(char *command)
       /* delete all existing grammars */
       multigram_delete_all();
       /* register the new grammar to multi-gram tree */
-      multigram_add(new_dfa, new_winfo, NULL);
+      multigram_add(new_dfa, new_winfo, p);
       /* need to rebuild the global lexicon */
       /* set engine flag to tell how to switch the grammar when active */
       set_grammar_switch_timing_flag();
       module_send(module_sd, "<GRAMMAR STATUS=\"RECEIVED\"/>\n.\n");
     }
-  } else if (strmatch(command, "ADDGRAM")) {
+  } else if (strnmatch(command, "ADDGRAM", strlen("ADDGRAM"))) {
     /* receive grammar and add it to the current grammars */
+    /* read grammar name if any */
+    p = &(command[strlen("ADDGRAM")]);
+    while (*p == ' ' && *p != '\r' && *p != '\n' && *p != '\0') p++;
+    if (*p != '\r' && *p != '\n' && *p != '\0') {
+      q = buf;
+      while (*p != ' ' && *p != '\r' && *p != '\n' && *p != '\0') *q++ = *p++;
+      *q = '\0';
+      p = buf;
+    } else {
+      p = NULL;
+    }
     /* read a new grammar via socket */
     if (msock_read_grammar(&new_dfa, &new_winfo) == FALSE) {
       module_send(module_sd, "<GRAMMAR STATUS=\"ERROR\" REASON=\"WRONG DATA\"/>\n.\n");
     } else {
       /* add it to multi-gram tree */
-      multigram_add(new_dfa, new_winfo, NULL);
+      multigram_add(new_dfa, new_winfo, p);
       /* need to rebuild the global lexicon */
       /* set engine flag to tell how to switch the grammar when active */
       set_grammar_switch_timing_flag();
@@ -678,7 +709,14 @@ msock_check_in_adin()
   if (module_is_connected()) {
     /* module: check command and terminate recording when requested */
     msock_check_and_process_command();
-    if (module_wants_terminate()) {/* TERMINATE ... force termination */
+    /* With audio input via adinnet, TERMINATE command will issue terminate
+       command to the adinnet client.  The client then stops recording
+       immediately and return end-of-segment ack.  Then it will cause this
+       process to stop recognition as normal.  So we need not to
+       perform immediate termination at this callback, but just ignore the
+       results in the main.c.  */
+    if (module_wants_terminate() /* TERMINATE ... force termination */
+	&& speech_input != SP_ADINNET) {
       return(-2);
     }
     if (process_want_reload) {
