@@ -35,13 +35,13 @@
  * corresponding functions in dfa_decode.c will be used instead.
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
    
@@ -178,6 +178,7 @@ set_word_context(WORD_ID *cseq, int n, WORD_INFO *winfo)
  * @param nw [i/o] ñꥹȡз̤ @a oldnum ʹߤɲä
  * @param oldnum [in] @a nw ˤǤ˳ǼƤ뼡ñο
  * @param hypo [in] Ÿʸ
+ * @param t [in] ե졼
  * 
  * @return ХꥹȤɲäȤ @a nw ˴ޤޤ뼡ñ
  * </JA>
@@ -195,6 +196,7 @@ set_word_context(WORD_ID *cseq, int n, WORD_INFO *winfo)
  * @param nw [in] list of next word candidates (new words will be appended at @a oldnum)
  * @param oldnum [in] number of words already stored in @a nw
  * @param hypo [in] the source sentence hypothesis
+ * @param t [in] specified frame
  * 
  * @return the total number of words currently stored in the @a nw.
  * </EN>
@@ -205,10 +207,8 @@ pick_backtrellis_words(BACKTRELLIS *bt, WORD_INFO *winfo, NGRAM_INFO *ngram, NEX
   int i;
   WORD_ID w;
   LOGPROB rawscore;
-#ifndef WORD_GRAPH
 #ifdef WPAIR
   int w_old = WORD_INVALID;
-#endif
 #endif
   int num;
 
@@ -221,9 +221,7 @@ pick_backtrellis_words(BACKTRELLIS *bt, WORD_INFO *winfo, NGRAM_INFO *ngram, NEX
 #ifdef WORD_GRAPH
     /* only words on the word graphs are expanded */
     if (!(bt->rw[t][i])->within_wordgraph) continue;
-    /* word connection is restricted by the graph structure */
-    if (w != ((hypo->tre)->last_tre)->wid) continue;
-#else /* not WORD_GRAPH */
+#endif /* not WORD_GRAPH */
 #ifdef WPAIR
     /* some word have same word ID with different previous word, so
        only one will be opened (best word will be selected later
@@ -231,7 +229,6 @@ pick_backtrellis_words(BACKTRELLIS *bt, WORD_INFO *winfo, NGRAM_INFO *ngram, NEX
     if (w == w_old) continue;	/* backtrellis is sorted by word ID */
     else w_old = w;
 #endif /* WPAIR */
-#endif /* not WORD_GRAPH */
     /* skip if already exist */
     if (search_nw(nw, w, oldnum) != NULL) continue;
     switch(cnnum) {		/* length of valid context */
@@ -464,10 +461,6 @@ limit_nw(NEXTWORD **nw, NODE *hypo, int num)
 int
 ngram_firstwords(NEXTWORD **nw, int peseqlen, int maxnw, WORD_INFO *winfo, BACKTRELLIS *bt)
 {
-#ifdef WORD_GRAPH
-  int last_time;
-#endif
-
 #ifdef SP_BREAK_CURRENT_FRAME
   if (rest_param != NULL) {
     /*  ǽե졼˻Ĥäñȥꥹκñ */
@@ -488,19 +481,6 @@ ngram_firstwords(NEXTWORD **nw, int peseqlen, int maxnw, WORD_INFO *winfo, BACKT
 #else
   nw[0]->lscore = lm_penalty2;
 #endif
-#ifdef WORD_GRAPH
-  /* TRELLIS_ATOM򤳤Ƿꤹ */
-  /* in word-graph mode, the first trellis atom should be defined here */
-  for (last_time = peseqlen - 1; last_time >= 0; last_time--) {
-    nw[0]->tre = bt_binsearch_atom(&backtrellis, last_time, winfo->tail_silwid);
-    if (nw[0]->tre != NULL) break;
-  }
-  if (nw[0]->tre == NULL) {		/* </s> not found */
-    j_printerr("no wordend left in beam!\n");
-    j_printerr("beam width too small or input speech too short\n");
-    return 0;
-  }
-#endif
 
   return 1;			/* number of words = 1 */
 }
