@@ -11,18 +11,52 @@
  * @brief  Output recoginition result to standard output
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.4 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
 #include <julius.h>
 
+/**********************************************************************/
+/* process online/offline status  */
+
+/** 
+ * <JA>
+ * ǧǽʾ֤ˤʤäȤ˸ƤФ
+ * 
+ * </JA>
+ * <EN>
+ * Called when it becomes ready to recognize the input.
+ * 
+ * </EN>
+ */
+static void
+ttyout_status_process_online()
+{
+  /* no output */
+}
+/** 
+ * <JA>
+ * ǧǾ֤ˤʤäȤ˸ƤФ
+ * 
+ * </JA>
+ * <EN>
+ * Called when process paused and recognition is stopped.
+ * 
+ * </EN>
+ */
+static void
+ttyout_status_process_offline()
+{
+  /* no output */
+}
+
 /**********************************************************************/
 /* 1st pass output */
 
@@ -42,6 +76,8 @@ static void
 ttyout_pass1_begin()
 {
   wst = 0;
+  /* moved from adin-cut.c */
+  j_printerr("\r                    \r");
 }
 
 
@@ -464,7 +500,7 @@ static void
 ttyout_graph(WordGraph *root, WORD_INFO *winfo)
 {
   WordGraph *wg;
-  int k, tw1, tw2, i;
+  int tw1, tw2, i;
 
   j_printf("-------------------------- begin wordgraph show -------------------------\n");
   for(wg=root;wg;wg=wg->next) {
@@ -542,7 +578,10 @@ ttyout_rejected(const char *s)
 void
 ttyout_status_recready()
 {
-  /* do noting */
+  if (speech_input == SP_MIC || speech_input == SP_NETAUDIO) {
+    /* moved from adin-cut.c */
+    j_printerr("<<< please speak >>>");
+  }
 }
 /** 
  * <JA>
@@ -610,6 +649,8 @@ ttyout_status_param(HTK_Param *param)
 void
 setup_result_tty()
 {
+  status_process_online = ttyout_status_process_online;
+  status_process_offline = ttyout_status_process_offline;
   status_recready      = ttyout_status_recready;
   status_recstart      = ttyout_status_recstart;
   status_recend        = ttyout_status_recend;
