@@ -40,13 +40,13 @@
  * within this file.
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -1062,6 +1062,7 @@ result_reorder_and_output(NODE **r_start, NODE **r_bottom, int *r_stacknum, int
  * @param backmax [in] 1ѥ줿Ѳ⥹κ
  * @param stacksize [in] ⥹åΥ
  * @param ncan [in] ٤ʸ
+ * @param maxhypo [in] õǰŸ
  * @param cate_bgn [in] ŸоݤȤ٤ƥγֹJuliusǤ̵
  * @param cate_num [in] ŸоݤȤ٤ƥοJuliusǤ̵
  * </JA>
@@ -1073,6 +1074,7 @@ result_reorder_and_output(NODE **r_start, NODE **r_bottom, int *r_stacknum, int
  * @param backmax [in] maximum score in the word trellis
  * @param stacksize [in] maximum size of hypothesis stack in the search
  * @param ncan [in] num of hypothesis to be found in the search
+ * @param maxhypo [in] threshold num of hypothesis expansion to be treated as "search overflow"
  * @param cate_bgn [in] category id to allow word expansion from (ignored in Julius)
  * @param cate_num [in] num of category to allow word expansion from (ignored in Julius)
  * </EN>
@@ -1384,7 +1386,13 @@ wchmm_fbs(HTK_Param *param, BACKTRELLIS *backtrellis, LOGPROB backmax, int stack
     }
 #ifdef GRAPHOUT_SEARCH
     /* if recent hypotheses are included in the existing graph, terminate */
-    if (merged_p && now->endflag == FALSE) {
+    if (merged_p && now->endflag == FALSE
+#ifdef GRAPHOUT_SEARCH_DELAY_TERMINATION
+	/* Do not apply search termination by graph merging
+	   until the first sentence candidate is found. */
+	&& (graphout_search_delay == FALSE || finishnum > 0)
+#endif
+	) {
       terminate_search_num++;
       free_node(now);
       continue;
@@ -2022,6 +2030,10 @@ wchmm_fbs(HTK_Param *param, BACKTRELLIS *backtrellis, LOGPROB backmax, int stack
   /* garbage collection and word merging */
   /* words with no following word (except end of sentence) will be erased */
   wordgraph_purge_leaf_nodes(&wordgraph_root);
+#ifdef GRAPHOUT_DEPTHCUT
+  /* perform word graph depth cutting here */
+  wordgraph_depth_cut(&wordgraph_root);
+#endif
   /* if GRAPHOUT_PRECISE_BOUNDARY defined,
      propagate exact time boundary to the right context.
      words of different boundary will be duplicated here.
