@@ -48,13 +48,13 @@
  * even non-promising ones.
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -1129,41 +1129,6 @@ next_word(NODE *now, NODE *new, NEXTWORD *nword, HTK_Param *param, BACKTRELLIS *
   /* (裲ѥ),(裱ѥ)ȥꥹ³³򸫤Ĥ */
   /* connect forward/backward trellis to look for the best connection time   */
   /***************************************************************************/
-#ifdef WORD_GRAPH
-  /*-----------------------------------------------------------------*/
-  /* ñ쥰ե⡼ɤǤƱե졼ƱñǤľñ˥
-     㤦bestt   */
-  /* there are several trellis word with the same word ID in the same
-     frame, with different word context in WORD_GRAPH mode.  And the
-     best t is fixed (no re-esimation) */
-  /*-----------------------------------------------------------------*/
-  /* ³, Ǹ˻ȤTRELLIS_ATOMνü˸ */
-  /* the best connection point is fixed to the end time of last trellis word (no re-alignment */
-  new->bestt = (nword->tre)->endtime;
-  /* ŸñΤοü֤ϡ嵭λü */
-  /* the estimated next beginning frame for the next word expansion
-     corresponds to the endtime above (this is also fixed) */
-  new->estimated_next_t = (nword->tre)->begintime - 1;
-  
-  /* ΤΥ׻ */
-#ifdef MULTIPATH_VERSION
-  new->score = new->g[new->bestt] + (nword->tre)->backscore;
-#else
-  /* ³ξ֤νϳΨû */
-  /* add output prob of connection state */
-  if (newphone->is_pseudo) {
-    tmpp = outprob_cd(new->bestt, &(newphone->body.pseudo->stateset[newphone->body.pseudo->state_num-2]), param);
-  } else {
-    tmpp = outprob_state(new->bestt, newphone->body.defined->s[newphone->body.defined->state_num-2], param);
-  }
-  new->score = new->g[new->bestt] + (nword->tre)->backscore + tmpp;
-#endif
-  
-  /* ŸñΥȥꥹξ */
-  new->tre  =  nword->tre;
-  
-#else  /* not WORD_GRAPH */
-  
   /*-----------------------------------------------------------------*/
   /* ñȥꥹõ, ñκ³ȯ */
   /* determine the best connection time of the new word, seeking the word
@@ -1250,8 +1215,6 @@ next_word(NODE *now, NODE *new, NEXTWORD *nword, HTK_Param *param, BACKTRELLIS *
   }
 #endif
 
-#endif /* WORD_GRAPH */
-
 #ifdef USE_NGRAM
   /* set current LM score */
   new->lscore = nword->lscore;
@@ -1323,14 +1286,6 @@ start_word(NODE *new, NEXTWORD *nword, HTK_Param *param, BACKTRELLIS *backtrelli
   new->g[peseqlen-1] = 0;
 #endif
   
-#ifdef WORD_GRAPH
-  new->score = new->g[peseqlen-1] + (nword->tre)->backscore;
-  new->tre  =  nword->tre;
-  new->bestt = peseqlen-1;
-  new->estimated_next_t = (nword->tre)->begintime - 1;
-  
-#else
-  
   for (t=peseqlen-1; t>=0; t--) {
     tre = bt_binsearch_atom(backtrellis, t, word);
     if (tre != NULL) {
@@ -1357,7 +1312,6 @@ start_word(NODE *new, NEXTWORD *nword, HTK_Param *param, BACKTRELLIS *backtrelli
   if (tre == NULL) {		/* no word in backtrellis */
     new->score = LOG_ZERO;
   }
-#endif /* WORD_GRAPH */
 }
 
 /** 
