@@ -1,9 +1,33 @@
-/* Copyright (c) 2003 Speech and Acoustics Processing Lab., NAIST */
-/*   All rights reserved   */
-
-/* visual.c --- visualize search space */
-
-/* $Id: visual.c,v 1.3 2003/12/15 02:35:51 ri Exp $ */
+/**
+ * @file   visual.c
+ * @author Akinobu Lee
+ * @date   Mon Sep 12 01:49:44 2005
+ * 
+ * <JA>
+ * @brief  õ֤βĻ벽 (GTK/X11 )
+ *
+ * õֲĻ벽ǽ Linux ǥݡȤƤޤ
+ * "--enable-visualize"  ON ˤƥѥǤޤ
+ * ʤѤˤ gtk ΥС 1.x ɬפǤ
+ * ưǧ gtk-1.2 ǹԤޤ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Visualization of search space using GTK/X11.
+ *
+ * This visualization feature is supported on Linux.
+ * To enable, specify "--enable-visualize" at configure.
+ * It needs gtk version = 1.x, (tested on gtk-1.2).
+ * </EN>
+ * 
+ * $Revision: 1.2 $
+ * 
+ */
+/*
+ * Copyright (c) 2003-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * All rights reserved
+ */
 
 #include <julius.h>
 
@@ -12,43 +36,43 @@
 #include <gtk/gtk.h>
 
 /* Window constant properties */
-#define WINTITLE "Julius word trellis viewer"
-#define DEFAULT_WINDOW_WIDTH 800
-#define DEFAULT_WINDOW_HEIGHT 600
-#define CANVAS_MARGIN 16
-#define WAVE_HEIGHT 48
-#define WAVE_MARGIN 6
-#define RESULT_HEIGHT 0
-#define FONTSET "-*-fixed-medium-r-normal--10-*-*-*-*-*-*-*"
+#define WINTITLE "Julius word trellis viewer" ///< Window title
+#define DEFAULT_WINDOW_WIDTH 800 ///< Default window width
+#define DEFAULT_WINDOW_HEIGHT 600 ///< Default window height
+#define CANVAS_MARGIN 16 ///< Margin of drawable
+#define WAVE_HEIGHT 48 ///< Height of wave drawing canvas
+#define WAVE_MARGIN 6 ///< Height margin of wave drawing canvas
+#define RESULT_HEIGHT 0 ///< Offset of text output for each node
+#define FONTSET "-*-fixed-medium-r-normal--10-*-*-*-*-*-*-*" ///< Fontset
 
 /* global valuables for window handling */
-static GdkFont *fontset;
-static GdkPixmap *pixmap = NULL;
-static gint canvas_width;
-static gint canvas_height;
+static GdkFont *fontset;	///< Fontset
+static GdkPixmap *pixmap = NULL; ///< Pixmap for the drawable
+static gint canvas_width;	///< Current width of the drawable
+static gint canvas_height;	///< Current height of the drawable
+
 
-
 /**********************************************************************/
 /* view configuration switches */
 /**********************************************************************/
-static boolean sw_wid_axis = TRUE; /* Y axis is wid (FALSE: score) */
-static boolean sw_score_beam = FALSE; /* Y axis is beam score (FALSE: normalized accumulated score */
-static boolean sw_text = TRUE;	/* text display on/off */
-static boolean sw_line = TRUE;	/* arc line display on/off */
-static boolean sw_level_thres = FALSE; /* show level thres on waveform */
-static boolean sw_hypo = FALSE;		/* Y axis is hypothesis score */
+static boolean sw_wid_axis = TRUE; ///< Y axis is wid (FALSE: score)
+static boolean sw_score_beam = FALSE; ///< Y axis is beam score (FALSE: normalized accumulated score
+static boolean sw_text = TRUE;	///< Text display on/off
+static boolean sw_line = TRUE;	///< Arc line display on/off
+static boolean sw_level_thres = FALSE; ///< Show level thres on waveform
+static boolean sw_hypo = FALSE;		///< Y axis is hypothesis score
 
 /**********************************************************************/
 /* data to plot (1st pass / 2nd pass) */
 /**********************************************************************/
 /* data to plot on 1st pass */
-static BACKTRELLIS *btlocal = NULL;
+static BACKTRELLIS *btlocal = NULL; ///< Local pointer to the word trellis
 /* data to plot on 2nd pass */
-static POPNODE *popped = NULL;	/* list of information for popped nodes */
-static int pnum;		/* maximum pop num */
-static POPNODE *lastpop = NULL;
+static POPNODE *popped = NULL;	///< List of information for popped nodes
+static int pnum;		///< Total number of popped nodes
+static POPNODE *lastpop = NULL;	///< Pointer to the last popped node
+
 
-
 /**********************************************************************/
 /* GTK color allocation */
 /**********************************************************************/
@@ -73,10 +97,18 @@ static GdkColor cols[NCOLS] = {
 };
 typedef enum {C_BG, C_WAVEFORM, C_LEVELTHRES, C_BGN, C_END, C_LINE, C_TEXT, C_LINE_FAINT, C_LINE_BEST, C_END_BEST, C_TEXT_BEST, C_PASS2_NEXT, C_PASS2, C_PASS2_BEST, C_SHADOW} UserColors;
 
-static GdkGC *dgc = NULL;		/* gc for drawing */
+static GdkGC *dgc = NULL;		///< gc for drawing
 
-/* assign color */
-static color_init()
+/** 
+ * <JA>
+ * γ
+ * </JA>
+ * <EN>
+ * Assign color.
+ * </EN>
+ */
+static void
+color_init()
 {
   GdkColormap *defcolmap;
   gboolean success[NCOLS];
@@ -88,18 +120,29 @@ static color_init()
 
 }
 
-
+
 /**********************************************************************/
 /* graph scaling */
 /**********************************************************************/
 
-static LOGPROB *ftop = NULL;	/* top score for each frame */
-static LOGPROB *fbottom = NULL;	/* bottom maximum score for each frame */
-static LOGPROB lowest;		/* lowest value (lower bound) */
-static LOGPROB maxrange;	/* maximum value of (top - bottom) */
-static LOGPROB maxrange2;	/* maximum distance from normalization line */
+static LOGPROB *ftop = NULL;	///< Top score for each frame
+static LOGPROB *fbottom = NULL;	///< Bottom maximum score for each frame
+static LOGPROB lowest;		///< Lowest value (lower bound)
+static LOGPROB maxrange;	///< Maximum value of (top - bottom)
+static LOGPROB maxrange2;	///< Maximum distance from normalization line
 
-/* get top and bottom score for scaling */
+/** 
+ * <JA>
+ * Ѥ˺祹ȺǾ롥
+ * 
+ * @param bt [in] ñȥꥹ
+ * </JA>
+ * <EN>
+ * Get the top and bottom scores for scaling.
+ * 
+ * @param bt [in] word trellis
+ * </EN>
+ */
 static void
 get_max_frame_score(BACKTRELLIS *bt)
 {
@@ -142,14 +185,46 @@ get_max_frame_score(BACKTRELLIS *bt)
   }
 }
 
-/* scale X axis by time to fill in the canvas width */
+/** 
+ * <JA>
+ * ֥ե졼 X ɸͤѴ롥
+ * 
+ * @param t [in] ֥ե졼
+ * 
+ * @return б X ɸ֤ͤ
+ * </JA>
+ * <EN>
+ * Scale X axis by time to fullfill in the canvas width.
+ * 
+ * @param t [in] time frame
+ * 
+ * @return the converted X position.
+ * </EN>
+ */
 static gint
 scale_x(int t)
 {
   return(t * (canvas_width - CANVAS_MARGIN * 2) / btlocal->framelen + CANVAS_MARGIN);
 }
 
-/* scale Y axis to fill in the canvas height */
+/** 
+ * <JA>
+ *  Y ɸͤѴ롥
+ * 
+ * @param s [in] 
+ * @param t [in] б֥ե졼
+ * 
+ * @return Y ɸ֤ͤ
+ * </JA>
+ * <EN>
+ * Scale Y axis from score to fulfill in the canvas height.
+ * 
+ * @param s [in] score to plot
+ * @param t [in] corresponding time frame
+ * 
+ * @return the converted Y position.
+ * </EN>
+ */
 static gint
 scale_y(LOGPROB s, int t)
 {
@@ -181,7 +256,22 @@ scale_y(LOGPROB s, int t)
   return(y);
 }
 
-/* scale Y axis by the word id */
+/** 
+ * <JA>
+ * ñID Y ɸͤѴ롥
+ * 
+ * @param wid [in] ñID
+ * 
+ * @return Y ɸ֤ͤ
+ * </JA>
+ * <EN>
+ * Scale Y axis from word id.
+ * 
+ * @param wid [in] word id
+ * 
+ * @return the converted Y position.
+ * </EN>
+ */
 static gint
 scale_y_wid(WORD_ID wid)
 {
@@ -198,23 +288,66 @@ scale_y_wid(WORD_ID wid)
   return(y);
 }
 
-
+
 /**********************************************************************/
 /* Draw wave data */
 /**********************************************************************/
-static SP16 max_level;
+static SP16 max_level;		///< Maximum level of input waveform
 
+/** 
+ * <JA>
+ * ȷɽѤ˻֤XɸѴ롥
+ * 
+ * @param t [in] ֥ե졼
+ * 
+ * @return Ѵ X ɸ֤
+ * </JA>
+ * <EN>
+ * Scale time to X position.
+ * 
+ * @param t [in] time frame
+ * 
+ * @return converted X position.
+ * </EN>
+ */
 static gint
 scale_x_wave(int t)
 {
   return(t * (canvas_width - CANVAS_MARGIN * 2) / speechlen + CANVAS_MARGIN);
 }
+
+/** 
+ * <JA>
+ * ȷɽѤ˿YɸѴ롥
+ * 
+ * @param x [in] ȷο
+ * 
+ * @return Ѵ X ɸ֤
+ * </JA>
+ * <EN>
+ * Scale wave level to Y position
+ * 
+ * @param x [in] wave level
+ * 
+ * @return converted Y position.
+ * </EN>
+ */
 static gint
 scale_y_wave(SP16 x)
 {
   return(WAVE_HEIGHT / 2 + WAVE_MARGIN - (x * WAVE_HEIGHT / (max_level * 2)));
 }
 
+/** 
+ * <JA>
+ * κͤ speech[] 롥
+ * 
+ * </JA>
+ * <EN>
+ * Get the maximum level of input waveform from speech[].
+ * 
+ * </EN>
+ */
 static void
 get_max_waveform_level()
 {
@@ -232,7 +365,20 @@ get_max_waveform_level()
 
   max_level = maxl;
   if (max_level < 3000) max_level = 3000;
-}	     
+}
+
+/** 
+ * <JA>
+ * ȷ speech[] 褹롥
+ * 
+ * @param widget [in] 襦å
+ * </JA>
+ * <EN>
+ * Draw input waveform in speech[].
+ * 
+ * @param widget [in] drawing widget
+ * </EN>
+ */
 static void
 draw_waveform(GtkWidget *widget)
 {
@@ -291,12 +437,33 @@ draw_waveform(GtkWidget *widget)
   }
 }
 
-
+
 /**********************************************************************/
 /* GTK primitive functions to draw a trellis atom */
 /**********************************************************************/
 
-/* draw word arc */
+/** 
+ * <JA>
+ * 褹롥
+ * 
+ * @param widget [in] 襦å
+ * @param x1 [in] 1Xɸ
+ * @param y1 [in] 1Yɸ
+ * @param x2 [in] 2Xɸ
+ * @param y2 [in] 2Yɸ
+ * @param sw [in] ٥λ
+ * </JA>
+ * <EN>
+ * Draw an arc.
+ * 
+ * @param widget [in] drawing widget
+ * @param x1 [in] x of 1st point
+ * @param y1 [in] y of 1st point
+ * @param x2 [in] x of 2nd point
+ * @param y2 [in] y of 2nd point
+ * @param sw [in] draw strength level
+ * </EN>
+ */
 static void
 mygdk_draw_arc(GtkWidget *widget, int x1, int y1, int x2, int y2, int sw)
 {
@@ -367,7 +534,24 @@ mygdk_draw_arc(GtkWidget *widget, int x1, int y1, int x2, int y2, int sw)
   
 }
 
-/* compute axis and draw the trellis word from last_tre to tre */
+/** 
+ * <JA>
+ * ȥꥹñ褹륵ִؿ
+ * 
+ * @param widget [in] 襦å
+ * @param tre [in] 褹ȥꥹñ
+ * @param last_tre [in] @a tre ľΥȥꥹñ
+ * @param sw [in] ζ
+ * </JA>
+ * <EN>
+ * Sub-function to draw a trellis word.
+ * 
+ * @param widget [in] drawing widget
+ * @param tre [in] trellis word to be drawn
+ * @param last_tre [in] previous word of @a tre
+ * @param sw [in] drawing strength
+ * </EN>
+ */
 static void
 draw_atom_sub(GtkWidget *widget, TRELLIS_ATOM *tre, TRELLIS_ATOM *last_tre, int sw)
 {
@@ -407,7 +591,22 @@ draw_atom_sub(GtkWidget *widget, TRELLIS_ATOM *tre, TRELLIS_ATOM *last_tre, int
   }
 }
 
-/* draw the trellis word at the 1st pass */
+/** 
+ * <JA>
+ * 1ѥΤȥꥹñ褹
+ * 
+ * @param widget [in] 襦å
+ * @param tre [in] ȥꥹñ
+ * @param sw [in] ζ
+ * </JA>
+ * <EN>
+ * Draw a trellis word at the 1st pass.
+ * 
+ * @param widget [in] drawing widget
+ * @param tre [in] trellis word
+ * @param sw [in] strength of drawing
+ * </EN>
+ */
 static void
 draw_atom(GtkWidget *widget, TRELLIS_ATOM *tre, int sw)
 {
@@ -415,6 +614,23 @@ draw_atom(GtkWidget *widget, TRELLIS_ATOM *tre, int sw)
 }
 
 /* draw word output text */
+/** 
+ * <JA>
+ * ȥꥹññɤߤ褹롥
+ * 
+ * @param widget [in] 襦å
+ * @param tre [in] ȥꥹñ
+ * @param sw [in] ζ
+ * </JA>
+ * <EN>
+ * Draw a word output string of a trellis word.
+ * 
+ * @param widget [in] drawing widget
+ * @param tre [in] trellis word
+ * @param sw [in] strength of drawing
+ * </EN>
+ */
+static void
 draw_atom_text(GtkWidget *widget, TRELLIS_ATOM *tre, int sw)
 {
   gint text_width;
@@ -459,14 +675,29 @@ draw_atom_text(GtkWidget *widget, TRELLIS_ATOM *tre, int sw)
   }
 }
 
-
+
 /**********************************************************************/
 /* wrapper for narrowing atoms to be drawn */
 /**********************************************************************/
-static WORD_ID *wordlist = NULL;
-static WORD_ID wordlistnum = 0; 
+static WORD_ID *wordlist = NULL; ///< List of drawn words for search
+static WORD_ID wordlistnum = 0;	///< Length of @a wordlist
 
-/* check if the word is in the wordlist */
+/** 
+ * <JA>
+ * ñꥹȤñ줬뤫ɤ롥
+ * 
+ * @param wid [in] ñID
+ * 
+ * @return ꥹȤ˸Ĥ TRUEĤʤ FALSE ֤
+ * </JA>
+ * <EN>
+ * Check if the given word exists in the drawn word list.
+ * 
+ * @param wid [in] word id
+ * 
+ * @return TRUE if found in list, or FALSE if not.
+ * </EN>
+ */
 static boolean 
 wordlist_find(WORD_ID wid)
 {
@@ -488,13 +719,28 @@ wordlist_find(WORD_ID wid)
   return FALSE;
 }
 
-
+
 /**********************************************************************/
-/* top trellis atom drawing functions including wrapper */
-/* all functions below should call this */
+/* Top trellis atom drawing functions including wrapper */
+/* All functions below should call this */
 /**********************************************************************/
 
-/* draw a word (if any wid specified in sw_wid, draw only that one) */
+/** 
+ * <JA>
+ * ñ褹ñꥹȤˤΤΤߡˡ
+ * 
+ * @param widget [in] 襦å
+ * @param tre [in] 褹ȥꥹñ
+ * @param sw [in] ζ
+ * </JA>
+ * <EN>
+ * Draw a word on canvas (only words in wordlist).
+ * 
+ * @param widget [in] drawing widget
+ * @param tre [in] trellis word to be drawn
+ * @param sw [in] strength of drawing
+ * </EN>
+ */
 static void
 draw_atom_top(GtkWidget *widget, TRELLIS_ATOM *tre, int sw)
 {
@@ -503,7 +749,24 @@ draw_atom_top(GtkWidget *widget, TRELLIS_ATOM *tre, int sw)
   }
 }
 
-/* draw a word text (if any wid specified in sw_wid, draw only that one) */
+/** 
+ * <JA>
+ * ñ褹ñꥹȤˤΤΤߡˡ
+ * ñΥƥȤ褹ʥꥹȤˤʤ褷ʤˡ
+ * 
+ * @param widget [in] 襦å
+ * @param tre [in] 褹ȥꥹñ
+ * @param sw [in] ζ
+ * </JA>
+ * <EN>
+ * ñΥƥȤ褹ñꥹȤˤΤΤߡˡ
+ * Draw text of a word on canvas (only words in wordlist).
+ * 
+ * @param widget [in] drawing widget
+ * @param tre [in] trellis word to be drawn
+ * @param sw [in] strength of drawing
+ * </EN>
+ */
 static void
 draw_atom_text_top(GtkWidget *widget, TRELLIS_ATOM *tre, int sw)
 {
@@ -512,12 +775,23 @@ draw_atom_text_top(GtkWidget *widget, TRELLIS_ATOM *tre, int sw)
   }
 }
 
-
+
 /**********************************************************************/
 /* Draw a set of atom according to their properties */
 /**********************************************************************/
 
-/* draw all survived trellis words  */
+/** 
+ * <JA>
+ * ƤΥȥꥹñ褹롥
+ * 
+ * @param widget [in] 襦å
+ * </JA>
+ * <EN>
+ * Draw all survived words in trellis.
+ * 
+ * @param widget [in] drawing widget
+ * </EN>
+ */
 static void
 draw_all_atom(GtkWidget *widget)
 {
@@ -540,7 +814,18 @@ draw_all_atom(GtkWidget *widget)
   }
 }
 
-/* draw only words whose next word was survived on the 1st pass */
+/** 
+ * <JA>
+ * 1ѥˤƤμñ줬ĤäȥꥹñΤ褹롥
+ * 
+ * @param widget [in] 襦å
+ * </JA>
+ * <EN>
+ * Draw words whose next word was survived on the 1st pass.
+ * 
+ * @param widget [in] drawing widget
+ * </EN>
+ */
 static void
 draw_context_valid_atom(GtkWidget *widget)
 {
@@ -568,7 +853,18 @@ draw_context_valid_atom(GtkWidget *widget)
 }
 
 #ifdef WORD_GRAPH
-/* draw word graph (score scaled, arcs other than the best path will not be displayed as connected) */
+/** 
+ * <JA>
+ * ñ쥰դȤ褹
+ * 
+ * @param widget [in] 襦å
+ * </JA>
+ * <EN>
+ * Draw words as word graph.
+ * 
+ * @param widget [in] drawing widget
+ * </EN>
+ */
 static void
 draw_word_graph(GtkWidget *widget)
 {
@@ -599,7 +895,18 @@ draw_word_graph(GtkWidget *widget)
 }
 #endif
 
-/* draw the best path on the 1st pass */
+/** 
+ * <JA>
+ * 1ѥΰʸ褹롥
+ * 
+ * @param widget [in] 襦å
+ * </JA>
+ * <EN>
+ * Draw the best path at the 1st pass.
+ * 
+ * @param widget [in] drawing widget
+ * </EN>
+ */
 static void
 draw_best_path(GtkWidget *widget)
 {
@@ -655,7 +962,18 @@ draw_best_path(GtkWidget *widget)
    referred to in the search process of the 2nd pass */
 /**********************************************************************/
 
-/* initialize */
+/** 
+ * <JA>
+ * 2ѥĻ벽ΤνԤ
+ * 
+ * @param maxhypo [in] 2ѥˤƥݥåפ줿κ
+ * </JA>
+ * <EN>
+ * Initialize for visualization of the 2nd pass.
+ * 
+ * @param maxhypo [in] maximum number of popped hypothesis on the 2nd pass.
+ * </EN>
+ */
 void
 visual2_init(int maxhypo)
 {
@@ -691,7 +1009,20 @@ visual2_init(int maxhypo)
   lastpop = NULL;
 }
 
-/* store popped node */
+/** 
+ * <JA>
+ * åФ¸롥
+ * 
+ * @param n [in] 
+ * @param popctr [in] ߤΥݥå׿
+ * </JA>
+ * <EN>
+ * Store popped nodes to local buffer.
+ * 
+ * @param n [in] hypothesis node
+ * @param popctr [in] current number of popped hypo.
+ * </EN>
+ */
 void
 visual2_popped(NODE *n, int popctr)
 {
@@ -705,7 +1036,22 @@ visual2_popped(NODE *n, int popctr)
   n->popnode = &(popped[popctr]);
 }
   
-/* store generated nodes */
+/** 
+ * <JA>
+ * 줿ΥΡɤ¸롥
+ * 
+ * @param next [in] 줿ñ첾
+ * @param prev [in] Ÿβ
+ * @param popctr [in] ߤ
+ * </JA>
+ * <EN>
+ * Store generated nodes.
+ * 
+ * @param next [in] generated next word hypothesis
+ * @param prev [in] source hypothesis from which @a next was expanded
+ * @param popctr [in] current popped num 
+ * </EN>
+ */
 void
 visual2_next_word(NODE *next, NODE *prev, int popctr)
 {
@@ -723,7 +1069,20 @@ visual2_next_word(NODE *next, NODE *prev, int popctr)
   popped[popctr].next = new;
 }
 
-/* store last popped hypothesis of best hypothesis */
+/** 
+ * <JA>
+ * ݥåפ줿¸롥
+ * 
+ * @param now [in] ʸ
+ * @param winfo [in] ñ켭
+ * </JA>
+ * <EN>
+ * Store last popped hypothesis of best hypothesis.
+ * 
+ * @param now [in] ʸ
+ * @param winfo [in] ñ켭
+ * </EN>
+ */
 void
 visual2_best(NODE *now, WORD_INFO *winfo)
 {
@@ -742,6 +1101,18 @@ visual2_best(NODE *now, WORD_INFO *winfo)
 /**********************************************************************/
 
 /* draw 2nd pass results */
+/** 
+ * <JA>
+ * 2ѥõˡåФ줿Ȥμñ콸褹롥
+ * 
+ * @param widget 襦å
+ * </JA>
+ * <EN>
+ * Draw popped hypotheses and their next candidates appeared while search.
+ * 
+ * @param widget 襦å
+ * </EN>
+ */
 static void
 draw_final_results(GtkWidget *widget)
 {
@@ -762,9 +1133,18 @@ draw_final_results(GtkWidget *widget)
   }
 }
 
-/* get maximum and minimum score */
-static LOGPROB maxscore;
-static LOGPROB minscore;
+static LOGPROB maxscore;	///< Maximum score of popped hypotheses while search
+static LOGPROB minscore;	///< Minimum score of popped hypotheses while search
+/** 
+ * <JA>
+ * 2ѥǽиŸΥκͤȺǾͤ롥
+ * 
+ * </JA>
+ * <EN>
+ * Get the maximum and minumum score of popped hypotheses appeared while search.
+ * 
+ * </EN>
+ */
 static void
 get_max_hypo_score()
 {
@@ -779,7 +1159,22 @@ get_max_hypo_score()
   }
 }
 
-/* scaling for hypo score */
+/** 
+ * <JA>
+ * ɽ⥹ Y ɸͤѴ롥
+ * 
+ * @param s [in] ⥹
+ * 
+ * @return бYɸ֤
+ * </JA>
+ * <EN>
+ * Scale hypothesis score to Y position.
+ * 
+ * @param s [in] hypothesis score
+ * 
+ * @return the corresponding Y position.
+ * </EN>
+ */
 static gint
 scale_hypo_y(LOGPROB s)
 {
@@ -793,6 +1188,26 @@ scale_hypo_y(LOGPROB s)
 }
 
 /* draw popped words */
+/** 
+ * <JA>
+ * åФ줿褹롥
+ * 
+ * @param widget [in] 襦å
+ * @param p [in] åФ줿ξ
+ * @param c [in] ο
+ * @param width [in] 
+ * @param style [in] Υ
+ * </JA>
+ * <EN>
+ * Draw a popped hypothesis.
+ * 
+ * @param widget [in] drawing widget
+ * @param p [in] information of popped hypothesis
+ * @param c [in] line color
+ * @param width [in] line width
+ * @param style [in] line style
+ * </EN>
+ */
 static void
 draw_popped(GtkWidget *widget, POPNODE *p, UserColors c, int width, int style)
 {
@@ -850,7 +1265,23 @@ draw_popped(GtkWidget *widget, POPNODE *p, UserColors c, int width, int style)
 }
 
 /* draw popped words at one hypothesis expantion */
-static int old_popctr;
+
+static int old_popctr;		///< popctr of previously popped hypo.
+
+/** 
+ * <JA>
+ * 2ѥñŸͻҤ褹롥
+ * 
+ * @param widget [in] 襦å
+ * @param popctr [in] Ÿֹ
+ * </JA>
+ * <EN>
+ * Draw a popped word and their expanded candidates for 2nd pass replay.
+ * 
+ * @param widget [in] drawing widget
+ * @param popctr [in] counter of popped hypothesis to draw
+ * </EN>
+ */
 static void
 draw_popnodes(GtkWidget *widget, int popctr)
 {
@@ -876,6 +1307,18 @@ draw_popnodes(GtkWidget *widget, int popctr)
 
   old_popctr = popctr;
 }
+/** 
+ * <JA>
+ * ľ draw_popnodes() 褷Τ񤭤ƾä
+ * 
+ * @param widget [in] 襦å
+ * </JA>
+ * <EN>
+ * Erase previous one drawn at draw_popnodes(), by overwriting.
+ * 
+ * @param widget [in] drawing widget
+ * </EN>
+ */
 static void
 draw_popnodes_old(GtkWidget *widget)
 {
@@ -895,19 +1338,28 @@ draw_popnodes_old(GtkWidget *widget)
   draw_popped(widget, porg, C_PASS2, 3, 2);
 }
 
-
 /**********************************************************************/
 /* GTK TopLevel draw/redraw functions */
 /* will be called for each exposure/configure event */
 /**********************************************************************/
-static boolean fitscreen = TRUE;
+static boolean fitscreen = TRUE; ///< եåȥ꡼ TRUE
 
-/**********************************************************************/
-/* draw background to pixmap */
+/** 
+ * <JA>
+ * Хطʤ pixmap ˺
+ * 
+ * @param widget [in] 襦å
+ * </JA>
+ * <EN>
+ * Make the white background to the pixmap of the given canvas widget.
+ * 
+ * @param widget [in] drawing widget
+ * </EN>
+ */
 static void
 draw_background(GtkWidget *widget)
 {
-  char buf[50];
+  static char buf[MAX_HMMNAME_LEN];
   
   /* first time, make gc for drawing */
   if (dgc == NULL) {
@@ -932,7 +1384,18 @@ draw_background(GtkWidget *widget)
   gdk_draw_string(pixmap, fontset, dgc, 0, canvas_height - 3, buf);
 }
 
-/* (re)draw all the contents to internal pixmap */
+/** 
+ * <JA>
+ * 襭ѥɽ٤Ƥñ pixmap 褹롥
+ * 
+ * @param widget 
+ * </JA>
+ * <EN>
+ * Draw all the contents to the pixmap.
+ * 
+ * @param widget 
+ * </EN>
+ */
 static void
 drawarea_draw(GtkWidget *widget)
 {
@@ -967,7 +1430,18 @@ drawarea_draw(GtkWidget *widget)
   }
 }
 
-/* tell X to issue expose event on this window */
+/** 
+ * <JA>
+ * Expose ٥ȤȯԤ롥
+ * 
+ * @param widget [in] 襦å
+ * </JA>
+ * <EN>
+ * Tell X to issue expose event on this window.
+ * 
+ * @param widget [in] drawing widget
+ * </EN>
+ */
 static void
 drawarea_expose(GtkWidget *widget)
 {
@@ -980,7 +1454,26 @@ drawarea_expose(GtkWidget *widget)
   gtk_widget_draw(widget, &r);
 }
 
-/* expose event handler: show the drawn internal pixmap to the screen */
+/** 
+ * <JA>
+ * Expose ٥Ƚ餫褵줿 pixmap ̤ɽ롥
+ * 
+ * @param widget [in] 襦å
+ * @param event [in] ٥Ⱦ
+ * @param user_data [in] 桼ǡ̤ѡ
+ * 
+ * @return gboolean ֤
+ * </JA>
+ * <EN>
+ * Expose event handler: show the already drawn pixmap to the screen.
+ * 
+ * @param widget [in] drawing widget
+ * @param event [in] event information
+ * @param user_data [in] user data (unused)
+ * 
+ * @return gboolean value.
+ * </EN>
+ */
 static gboolean
 event_drawarea_expose(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
 {
@@ -994,7 +1487,26 @@ event_drawarea_expose(GtkWidget *widget, GdkEventExpose *event, gpointer user_da
   }
 }
 
-/* configure event handler: (re) draw objects when resized or mapped */
+/** 
+ * <JA>
+ * Configure ٥Ƚresize ޤ map ˺褹롥
+ * 
+ * @param widget [in] 襦å
+ * @param event [in] ٥Ⱦ
+ * @param user_data [in] 桼ǡ̤ѡ
+ * 
+ * @return gboolean ֤
+ * </JA>
+ * <EN>
+ * Configure event handler: redraw objects when resized or mapped.
+ * 
+ * @param widget [in] drawing widget
+ * @param event [in] event information
+ * @param user_data [in] user data (unused)
+ * 
+ * @return gboolean value.
+ * </EN>
+ */
 static gboolean
 event_drawarea_configure(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
 {
@@ -1008,12 +1520,25 @@ event_drawarea_configure(GtkWidget *widget, GdkEventExpose *event, gpointer user
   drawarea_draw(widget);
 }
 
-
+
 /**********************************************************************/
 /* GTK callbacks for buttons */
 /**********************************************************************/
 
-/* callback when [show threshold] button is clicked */
+/** 
+ * <JA>
+ * [show threshold] ܥ󥯥å callback: ȥꥬ
+ *  ON/ OFF
+ * 
+ * @param widget [in] 襦å
+ * </JA>
+ * <EN>
+ * Callback when [show threshold] button is clicked: toggle trigger
+ * threshold line.
+ * 
+ * @param widget [in] drawing widget
+ * </EN>
+ */
 static void
 action_toggle_thres(GtkWidget *widget)
 {
@@ -1031,7 +1556,18 @@ action_toggle_thres(GtkWidget *widget)
 }
 
 #ifdef PLAYCOMMAND
-/* callback when [play] button is clicked */
+/** 
+ * <JA>
+ * [play] ܥ󥯥å callback: 롥
+ * 
+ * @param widget [in] 襦å
+ * </JA>
+ * <EN>
+ * Callback when [play] button is clicked: play waveform.
+ * 
+ * @param widget [in] drawing widget
+ * </EN>
+ */
 static void
 action_play_waveform(GtkWidget *widget)
 {
@@ -1061,7 +1597,20 @@ action_play_waveform(GtkWidget *widget)
 }
 #endif
 
-/* callback when [Word view] button is clicked */
+/** 
+ * <JA>
+ * [Word view] ܥ󥯥å callback: YñIDˤ롥
+ * 
+ * @param button [in] ܥ󥦥å
+ * @param widget [in] 襦å
+ * </JA>
+ * <EN>
+ * Callback when [Word view] button is clicked: set Y axis to word ID.
+ * 
+ * @param button [in] button widget
+ * @param widget [in] drawing widget
+ * </EN>
+ */
 static void
 action_view_wid(GtkWidget *button, GtkWidget *widget)
 {
@@ -1078,7 +1627,20 @@ action_view_wid(GtkWidget *button, GtkWidget *widget)
   }
 }
 
-/* callback when [Score view] button is clicked */
+/** 
+ * <JA>
+ * [Score view] ܥ󥯥å callback: Y򥹥ˤ롥
+ * 
+ * @param button [in] ܥ󥦥å
+ * @param widget [in] 襦å
+ * </JA>
+ * <EN>
+ * Callback when [Score view] button is clicked: set Y axis to score.
+ * 
+ * @param button [in] button widget
+ * @param widget [in] drawing widget
+ * </EN>
+ */
 static void
 action_view_score(GtkWidget *button, GtkWidget *widget)
 {
@@ -1093,7 +1655,22 @@ action_view_score(GtkWidget *button, GtkWidget *widget)
   }
 }
 
-/* callback when [Beam view] button is clicked */
+/** 
+ * <JA>
+ * [Beam view] ܥ󥯥å callback: Yե졼ऴȤκ祹
+ * κʬˤ롥
+ * 
+ * @param button [in] ܥ󥦥å
+ * @param widget [in] 襦å
+ * </JA>
+ * <EN>
+ * Callback when [Beam view] button is clicked: set Y axis to offsets
+ * from the maximum score on each frame.
+ * 
+ * @param button [in] button widget
+ * @param widget [in] drawing widget
+ * </EN>
+ */
 static void
 action_view_beam(GtkWidget *button, GtkWidget *widget)
 {
@@ -1108,7 +1685,22 @@ action_view_beam(GtkWidget *button, GtkWidget *widget)
   }
 }
 
-/* callback when [Arc on/off] button is clicked */
+/** 
+ * <JA>
+ * [Arc on/off] ܥ󥯥å callback: ñƬñ콪üδ֤
+ *  on/off 롥
+ * 
+ * @param button [in] ܥ󥦥å
+ * @param widget [in] 襦å
+ * </JA>
+ * <EN>
+ * Callback when [Arc on/off] button is clicked: toggle lines between
+ * word head node and word tail node.
+ * 
+ * @param button [in] button widget
+ * @param widget [in: drawing widget
+ * </EN>
+ */
 static void
 action_toggle_arc(GtkWidget *button, GtkWidget *widget)
 {
@@ -1125,7 +1717,22 @@ action_toggle_arc(GtkWidget *button, GtkWidget *widget)
   drawarea_expose(widget);
 }
 
-/* callback when word ID entry is entered */
+/** 
+ * <JA>
+ * ƥȥåȤñ̾Ϥ줿Ȥcallback: ñꥹȤ
+ * 롥
+ * 
+ * @param widget [in] ƥȥå
+ * @param draw [in] 襦å
+ * </JA>
+ * <EN>
+ * Callback when a word string is entered on the text widget: update the
+ * drawing word list to show only the word.
+ * 
+ * @param widget [in] text widget
+ * @param draw [in] drawing widget
+ * </EN>
+ */
 static void
 action_set_wid(GtkWidget *widget, GtkWidget *draw)
 {
@@ -1163,8 +1770,20 @@ action_set_wid(GtkWidget *widget, GtkWidget *draw)
   drawarea_expose(draw);
 }
 
-/* when zooming, enter no-fitscreen mode: resizing does not cause
-   re-scaling in X axis */
+/** 
+ * <JA>
+ * [x2] ܥ󥯥åcallback: X2ܤ˿ĥ롥ʤFITSCREEN
+ * OFFˤʤ롥
+ * 
+ * @param widget [in] 襦å
+ * </JA>
+ * <EN>
+ * Callback when [x2] zoom button is clicked: expand X axis to "x2".
+ * If FITSCREEN is enabled, it will be disabled.
+ * 
+ * @param widget [in] drawing widget
+ * </EN>
+ */
 static void
 action_zoom(GtkWidget *widget)
 {
@@ -1177,6 +1796,21 @@ action_zoom(GtkWidget *widget)
   drawarea_draw(widget);
   drawarea_expose(widget);
 }
+
+/** 
+ * <JA>
+ * [x4] ܥ󥯥åcallback: X4ܤ˿ĥ롥ʤFITSCREEN
+ * OFFˤʤ롥
+ * 
+ * @param widget [in] 襦å
+ * </JA>
+ * <EN>
+ * Callback when [x4] zoom button is clicked: expand X axis to "x4".
+ * If FITSCREEN is enabled, it will be disabled.
+ * 
+ * @param widget [in] drawing widget
+ * </EN>
+ */
 static void
 action_zoom_4(GtkWidget *widget)
 {
@@ -1189,6 +1823,21 @@ action_zoom_4(GtkWidget *widget)
   drawarea_draw(widget);
   drawarea_expose(widget);
 }
+
+/** 
+ * <JA>
+ * [x8] ܥ󥯥åcallback: X8ܤ˿ĥ롥ʤFITSCREEN
+ * OFFˤʤ롥
+ * 
+ * @param widget [in] 襦å
+ * </JA>
+ * <EN>
+ * Callback when [x8] zoom button is clicked: expand X axis to "x8".
+ * If FITSCREEN is enabled, it will be disabled.
+ * 
+ * @param widget [in] drawing widget
+ * </EN>
+ */
 static void
 action_zoom_8(GtkWidget *widget)
 {
@@ -1202,7 +1851,20 @@ action_zoom_8(GtkWidget *widget)
   drawarea_expose(widget);
 }
 
-/* fit screen button: disable zoom and make the canvas automatically fit the window size */
+/** 
+ * <JA>
+ * [Fit] ܥ󥯥åΥХåѥ򥦥ɥ
+ * ưŪ˹碌褦ˤ롥¾ zoom  off ˤ롥
+ * 
+ * @param widget 
+ * </JA>
+ * <EN>
+ * Callback for [Fit] button: make canvas automatically fit to the window size.
+ * If other zoom mode is enabled, they will be disabled.
+ * 
+ * @param widget 
+ * </EN>
+ */
 static void
 action_fit_screen(GtkWidget *widget)
 {
@@ -1214,7 +1876,21 @@ action_fit_screen(GtkWidget *widget)
   drawarea_expose(widget);
 }
 
-/* pass2 replay: toggle popctr */
+/** 
+ * <JA>
+ * 2ѥƸѤ [start] ܥcallback: 2ѥƸѤ˽롥
+ * 
+ * @param button [in] ܥ󥦥å
+ * @param widget [in] 襦å
+ * </JA>
+ * <EN>
+ * Callback for [start] button for pass2 replay: prepare flag and canvas
+ * for the replaying of the 2nd pass.
+ * 
+ * @param button [in] button widget
+ * @param widget [in] drawing widget
+ * </EN>
+ */
 static void
 action_toggle_popctr(GtkWidget *button, GtkWidget *widget)
 {
@@ -1227,7 +1903,23 @@ action_toggle_popctr(GtkWidget *button, GtkWidget *widget)
   drawarea_expose(widget);
 }
 
-/* pass2 replay: popctr scale */
+/** 
+ * <JA>
+ * 2ѥƸѤΥcallback: ͤ N ѹ줿Ȥˡ
+ * N ܤñŸͻҤ褹롥
+ * 
+ * @param adj [in] 㥹
+ * @param widget [in] 襦å
+ * </JA>
+ * <EN>
+ * Callback for pop counter scale for pass2 replay: when the scale value
+ * was changed to N, draw the details of the Nth word expansion on the
+ * 2nd pass.
+ * 
+ * @param adj [in] adjuster of the scale
+ * @param widget [in] drawing widget
+ * </EN>
+ */
 static void
 action_change_popctr(GtkAdjustment *adj, GtkWidget *widget)
 {
@@ -1244,23 +1936,65 @@ action_change_popctr(GtkAdjustment *adj, GtkWidget *widget)
 /**********************************************************************/
 /* GTK delete/destroy event handler */
 /**********************************************************************/
-/* delete event */
+
+/** 
+ * <JA>
+ * GTKκ٥ȥϥɥ
+ * 
+ * @param widget [in] å
+ * @param event [in] ٥
+ * @param data [in] 桼ǡ
+ * 
+ * @return FALSE֤destroy signalȯԤ뤿ˡ
+ * </JA>
+ * <EN>
+ * GTK delete event handler.
+ * 
+ * @param widget [in] widget
+ * @param event [in] event
+ * @param data [in] user data
+ * 
+ * @return always FALSE, to emit destroy signal.
+ * </EN>
+ */
 static gint
 delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
   return (FALSE);		/* emit destroy signal */
 }
+
+/** 
+ * <JA>
+ * GTKνλ٥ȥϥɥ顥ץꥱλ롥
+ * 
+ * @param widget [in] å
+ * @param data [in] 桼ǡ
+ * </JA>
+ * <EN>
+ * GTK destroy event handler.  Quit application here.
+ * 
+ * @param widget [in] widget
+ * @param data [in] user data
+ * </EN>
+ */
 static void
 destroy(GtkWidget *widget, gpointer data)
 {
   gtk_main_quit();
 }
 
-
 /**********************************************************************/
-/* visualization main */
+/* Main public functions for visualization */
 /**********************************************************************/
-/* initialize: called once on startup */
+
+/** 
+ * <JA>
+ * ưĻ벽ǽ롥
+ * </JA>
+ * <EN>
+ * Initialize visualization functions at startup.
+ * </EN>
+ */
 void
 visual_init()
 {
@@ -1290,8 +2024,18 @@ visual_init()
 
 }
 
-/* show widget and start browsing */
-/* call each time the 1st pass ended */
+/** 
+ * <JA>
+ * ǧ̤򸵤ˡõ֤βĻ벽¹Ԥ롥
+ * 
+ * @param bt [in] ñȥꥹ
+ * </JA>
+ * <EN>
+ * Start visualization of recognition result.
+ * 
+ * @param bt [in] word trellis
+ * </EN>
+ */
 void
 visual_show(BACKTRELLIS *bt)
 {
