@@ -20,13 +20,13 @@
  * ٷ׻ϤξǹԤޤ
  * </JA>
  *
- * $Revision: $
+ * $Revision: 1.3 $
  *
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -126,20 +126,25 @@ outprob_init(HTK_HMM_INFO *hmminfo,
 	     int gprune_method, int gprune_mixnum
 	     );
 boolean outprob_prepare(int framenum);
+void outprob_free();
 /* outprob.c */
 boolean outprob_cache_init();
 boolean outprob_cache_prepare();
+void outprob_cache_free();
 LOGPROB outprob_state(int t, HTK_HMM_State *stateinfo, HTK_Param *param);
 void outprob_cd_nbest_init(int num);
+void outprob_cd_nbest_free();
 LOGPROB outprob_cd(int t, CD_State_Set *lset, HTK_Param *param);
 LOGPROB outprob(int t, HMM_STATE *hmmstate, HTK_Param *param);
 /* gms.c */
 boolean gms_init(int nbest);
 boolean gms_prepare(int framelen);
+void gms_free();
 LOGPROB gms_state();
 /* gms_gprune.c */
 void gms_gprune_init(HTK_HMM_INFO *hmminfo, int gsset_num);
 void gms_gprune_prepare();
+void gms_gprune_free();
 void compute_gs_scores(GS_SET *gsset, int gsset_num, LOGPROB *scores_ret);
 
 /* calc_mix.c */
@@ -147,6 +152,7 @@ LOGPROB calc_mix();
 /* calc_tied_mix.c */
 boolean calc_tied_mix_init();
 boolean calc_tied_mix_prepare(int framenum);
+void calc_tied_mix_free();
 LOGPROB calc_tied_mix();
 
 /* hmminfo/put_htkdata_info */
