@@ -15,13 +15,13 @@
  * Ƥޤ
  * </JA>
  *
- * $Revision:$
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -372,6 +372,10 @@ typedef struct {
   int maxcodebooksize;		///< Maximum size of codebook on tied-mixture model
   int maxmixturenum;		///< Maximum number of Gaussian per mixture
   int maxstatenum;		///< Maximum number of state per model
+
+  BMALLOC_BASE *mroot;		///< Pointer for block memory allocation
+  BMALLOC_BASE *lroot;		///< Pointer for block memory allocation for logical HMM
+
   //@}
 } HTK_HMM_INFO;
 
@@ -382,7 +386,6 @@ void htk_hmm_set_pause_model(HTK_HMM_INFO *hmminfo, char *spmodel_name);
 /* rdhmmdef.c */
 void rderr(char *str);
 char *read_token(FILE *fp);
-void init_hmm(HTK_HMM_INFO *);
 boolean rdhmmdef(FILE *, HTK_HMM_INFO *);
 /* rdhmmdef_options.c */
 void set_global_opt(FILE *fp, HTK_HMM_INFO *hmm);
@@ -408,7 +411,7 @@ void def_var_macro(char *, FILE *, HTK_HMM_INFO *);
 void var_add(HTK_HMM_INFO *hmm, HTK_HMM_Var *new);
 /* rdhmmdef_data.c */
 void def_HMM(char *, FILE *, HTK_HMM_INFO *);
-HTK_HMM_Data *htk_hmmdata_new();
+HTK_HMM_Data *htk_hmmdata_new(HTK_HMM_INFO *);
 void htk_hmmdata_add(HTK_HMM_INFO *hmm, HTK_HMM_Data *new);
 /* rdhmmdef_tiedmix.c */
 void tmix_read(FILE *fp, HTK_HMM_State *state, HTK_HMM_INFO *hmm);
@@ -426,6 +429,7 @@ void put_htk_hmm(HTK_HMM_Data *h);
 void put_logical_hmm(HMM_Logical *l);
 void print_hmmdef_info(HTK_HMM_INFO *);
 HTK_HMM_INFO *hmminfo_new();
+boolean hmminfo_free(HTK_HMM_INFO *);
 void init_hmminfo(HTK_HMM_INFO *hmminfo, char *filename, char *mapfile);
 HTK_HMM_Data *htk_hmmdata_lookup_physical(HTK_HMM_INFO *, char *);
 HMM_Logical *htk_hmmdata_lookup_logical(HTK_HMM_INFO *, char *);
@@ -456,6 +460,7 @@ char *rightcenter_name(char *hmmname, char *buf);
 boolean regist_cdset(APATNODE **root, HTK_HMM_Data *d, char *cdname);
 boolean make_cdset(HTK_HMM_INFO *hmminfo);
 void put_all_cdinfo(HTK_HMM_INFO *hmminfo);
+void free_cdset(APATNODE **root);
 CD_Set *cdset_lookup(HTK_HMM_INFO *hmminfo, char *cdstr);
 CD_Set *lcdset_lookup_by_hmmname(HTK_HMM_INFO *hmminfo, char *hmmname);
 CD_Set *rcdset_lookup_by_hmmname(HTK_HMM_INFO *hmminfo, char *hmmname);
@@ -470,6 +475,6 @@ HTK_Param *new_param_check_and_adjust(HTK_HMM_INFO *hmm, HTK_Param *pinfo, boole
 
 /* binary format */
 boolean write_binhmm(FILE *fp, HTK_HMM_INFO *hmm);
-boolean read_binhmm(FILE *fp, HTK_HMM_INFO *hmm);
+boolean read_binhmm(FILE *fp, HTK_HMM_INFO *hmm, boolean gzfile_p);
 
 #endif /* __SENT_HTK_HMM_2_H__ */
