@@ -70,13 +70,13 @@
  * Ϳ줿N-gram嵭ʤ硤Julius ϥ顼Фޤ
  * </JA>
  * 
- * $Revision:$
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -111,10 +111,11 @@ typedef int NNID;	       ///< Type definition for N-gram word ID
  * 
  */
 typedef struct {
+  int version;			///< version number
   boolean from_bin;		///< TRUE if source is bingram, otherwise ARPA
   WORD_ID max_word_num;		///< N-gram vocabulary size
   NNID ngram_num[MAX_N];	///< Total number of tuples for each N
-  NNID bigram_bo_num;		///< Total number of bigram tuples that has back-off weight (i.e. context of upper 3-gram)
+  NNID bigram_bo_num;		///< Total number of bigram tuples that has back-off weight (i.e. context of upper 3-gram) (v4)
   /**
    * @brief Unknown word ID
    *
@@ -142,13 +143,14 @@ typedef struct {
   WORD_ID *n2tonid;		///< Mapping each 2-gram index ID (n2) to its last word ID (nid)
   LOGPROB *p_lr;		///< LR 2-gram log probabilities [n2]
   LOGPROB *p_rl;		///< RL 2-gram log probabilities [n2]
-  NNID_UPPER *n2bo_upper; ///< Mapping each 2-gram index ID (n2) to bigram back-off index (n2-bo)
-  NNID_LOWER *n2bo_lower; ///< Mapping each 2-gram index ID (n2) to bigram back-off index (n2-bo)
+  NNID_UPPER *n2bo_upper; ///< Mapping each 2-gram index ID (n2) to bigram back-off index (n2-bo) (v4)
+  NNID_LOWER *n2bo_lower; ///< Mapping each 2-gram index ID (n2) to bigram back-off index (n2-bo) (v4)
   
   /* 2-gram back-off values (separated from  rev.3.5) */
   LOGPROB *bo_wt_rrl;		///< Back-off weights for RL 3-gram [n2-bo]
-  NNID_UPPER *n3_bgn_upper; ///< 3-gram IDs (n3) representing beginning point of 3-gram entries that have the left context
-  NNID_LOWER *n3_bgn_lower; ///< 3-gram IDs (n3) representing beginning point of 3-gram entries that have the left context
+  NNID *n3_bgn;			///< 3-gram IDs (n3) representing beginning point of 3-gram entries that have the left context (v3)
+  NNID_UPPER *n3_bgn_upper; ///< upper 8-bit 3-gram IDs (n3) representing beginning point of 3-gram entries that have the left context (v4)
+  NNID_LOWER *n3_bgn_lower; ///< lower 16-bit 3-gram IDs (n3) representing beginning point of 3-gram entries that have the left context (v4)
   WORD_ID *n3_num;		///< Number of 3-gram that have the left context of above
   
   /* 3-gram (n3: 0 - ngram_num[2] - 1) */
@@ -185,7 +187,7 @@ typedef struct {
 
 /* function declaration */
 NNID search_bigram(NGRAM_INFO *ndata, WORD_ID w_l, WORD_ID w_r);
-NNID search_trigram(NGRAM_INFO *ndata,  NNID n2, WORD_ID wkey);
+// NNID search_trigram(NGRAM_INFO *ndata,  NNID n2, WORD_ID wkey);
 LOGPROB uni_prob(NGRAM_INFO *ndata, WORD_ID w);
 LOGPROB bi_prob_lr(NGRAM_INFO *ndata, WORD_ID w1, WORD_ID w2);
 LOGPROB bi_prob_rl(NGRAM_INFO *ndata, WORD_ID w1, WORD_ID w2);
