@@ -1,22 +1,56 @@
+/**
+ * @file   adin_mic_freebsd.c
+ * @author Akinobu LEE
+ * @date   Sun Feb 13 16:18:26 2005
+ *
+ * <JA>
+ * @brief  ޥ (FreeBSD)
+ *
+ * ޥϤΤ٥ؿǤFreeBSDǤΥե뤬Ѥޤ
+ *
+ * ɥɤ 16bit Υ ϿǤ뤳ȤɬܤǤ
+ * 
+ * JuliusFreeBSDǥߥǥХڹԤޤϿǥХ
+ * ʥޥ/饤ˤϿܥ塼Ĵ¾Υġ
+ * ԤʤäƲ
+ *
+ * ǥեȤΥǥХ̾ "/dev/dsp" ǤĶѿ AUDIODEV 
+ * ǥХ̾ꤹ뤳Ȥǡ¾ΥǥХ̾ѤǤޤ
+ *
+ * ưǧFreeBSD 3.2-RELEASE ǹԤʤޤɥɥ饤Ф
+ * snd ѤƤޤ
+ * </JA>
+ * <EN>
+ * @brief  Microphone input on FreeBSD
+ *
+ * Low level I/O functions for microphone input on FreeBSD.
+ *
+ * To use microphone input in FreeBSD, sound card and sound driver must
+ * support 16bit monaural recording.
+ *
+ * Julius does not alter any mixer device setting at all.  You should
+ * configure the mixer for recording source (mic/line) and recording volume
+ * correctly using other audio tool.
+ *
+ * The default device name is "/dev/dsp", which can be changed by setting
+ * environment variable AUDIODEV.
+ *
+ * Tested on FreeBSD 3.2-RELEASE with snd driver.
+ * </EN>
+ *
+ * $Revision: 1.2 $
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* adin_mic_freebsd.c --- adin lib for FreeBSD/snd */
-
-/* $Id: adin_mic_freebsd.c,v 1.3 2003/09/29 06:01:22 ri Exp $ */
-
 /* Thanks to Kentaro Nagatomo for information */
 /* All functions are the same as OSS version, except the header filename */
 
-/*
- * These function will not change mixer setting.
- * Use other mixer program to setup mic device.
- * (mute/unmute, volume control, etc.)
- */
-
 #include <sent/stddefs.h>
 #include <sent/adin.h>
 
@@ -24,36 +58,46 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <unistd.h>
 #include <poll.h>
 
 /* sound header */
-/* #include <sys/soundcard.h> */
 #include <machine/soundcard.h>
 
-static int audio_fd;		/* audio descriptor */
-static boolean need_swap;	/* whether samples need byte swap */
-static int fmt;			/* signed 16bit */
-struct pollfd fds[1];		/* structure for polling */
+/// Default device name, can be overridden by AUDIODEV environment variable
+#define DEFAULT_DEVICE "/dev/dsp"
 
-#define FREQALLOWRANGE 200	/* acceptable sampling frequency width around 16kHz */
-#define POLLINTERVAL 200	/* in miliseconds */
+static int audio_fd;		///< Audio descriptor
+static boolean need_swap;	///< Whether input samples need byte-swapping
+struct pollfd fds[1];		///< Workarea for polling device
 
+#define FREQALLOWRANGE 200	///< Acceptable width of sampling frequency
+#define POLLINTERVAL 200	///< Polling interval in miliseconds
 
-/* check audio port resource and initialize */
-/* will be called once at startup time */
+/** 
+ * Device initialization: check device capability and open for recording.
+ * 
+ * @param sfreq [in] required sampling frequency.
+ * @param arg [in] a dummy data
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 adin_mic_standby(int sfreq, void *arg)
 {
-  int fmt_can;
-  int fmt1, fmt2;                /* sampling format */
-  int rfmt;
-  int samplerate;
+  int fmt, fmt_can, fmt1, fmt2, rfmt; /* sampling format */
+  int samplerate;		/* actual sampling rate */
   int stereo;		/* mono */
+  char *defaultdev = DEFAULT_DEVICE; /* default device */
+  char *devname;
 
+  /* set device name */
+  if ((devname = getenv("AUDIODEV")) == NULL) {
+    devname = defaultdev;
+  }
+  
   /* open device */
-  if ((audio_fd = open("/dev/dsp", O_RDONLY)) == -1) {
-    perror("adin_mic_standby: open /dev/dsp");
+  if ((audio_fd = open(devname, O_RDONLY)) == -1) {
+    perror("adin_mic_standby: open");
     return(FALSE);
   }
 
@@ -93,8 +137,8 @@ adin_mic_standby(int sfreq, void *arg)
 
   /* re-open for recording */
   /* open device */
-  if ((audio_fd = open("/dev/dsp", O_RDONLY)) == -1) {
-    perror("adin_mic_standby: open /dev/dsp");
+  if ((audio_fd = open(devname, O_RDONLY)) == -1) {
+    perror("adin_mic_standby: open");
     return(FALSE);
   }
   /* set format, samplerate, channels */
@@ -138,7 +182,11 @@ adin_mic_standby(int sfreq, void *arg)
   return TRUE;
 }
  
-/* start recording */
+/** 
+ * Start recording.
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 adin_mic_start()
 {
@@ -150,7 +198,11 @@ adin_mic_start()
   return(TRUE);
 }
 
-/* stop recording */
+/** 
+ * Stop recording.
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 adin_mic_stop()
 {
@@ -163,8 +215,18 @@ adin_mic_stop()
   return TRUE;
 }
 
-/* read samples from audio device */
-/* try to read `sampnum' samples and returns actual sample num recorded */
+/**
+ * @brief  Read samples from device
+ * 
+ * Try to read @a sampnum samples and returns actual number of recorded
+ * samples currently available.  This function will block until at least one
+ * sample can be obtained.
+ * 
+ * @param buf [out] samples obtained in this function
+ * @param sampnum [in] wanted number of samples to be read
+ * 
+ * @return actural number of read samples, -2 if error.
+ */
 int
 adin_mic_read(SP16 *buf, int sampnum)
 {
@@ -178,18 +240,11 @@ adin_mic_read(SP16 *buf, int sampnum)
     perror("adin_mic_read: sndctl_dsp_getispace");
     return(-2);
   }
-  /*printf("%d\n",info.bytes);*/
   
   /* get them as much as possible */
   size = sampnum * sizeof(SP16);
   if (size > info.bytes) size = info.bytes;
   cnt = read(audio_fd, buf, size);
-/* 
- *   do {
- *     errno = 0;
- *     cnt = read(audio_fd, buf, sampnum*sizeof(short));
- *   } while ( cnt < 0 && ( errno == EINTR || errno == EAGAIN ) );
- */
   if ( cnt < 0 ) {
     perror("adin_mic_read: read error\n");
     return ( -2 );
