@@ -1,24 +1,63 @@
-/*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
- * All rights reserved
+/**
+ * @file   adin_mic_linux_oss.c
+ * @author Akinobu LEE
+ * @date   Sun Feb 13 16:18:26 2005
+ *
+ * <JA>
+ * @brief  ޥ (Linux/OSS)
+ *
+ * ޥϤΤ٥ؿǤ󥿥եȤ OSS
+ * ɥɥ饤ФѤ硤Υե뤬Ѥޤ
+ * ͥɸΥɥ饤СOSS/LinuxΥɥ饤С ALSA 
+ * OSSߴ⡼ɤбƤޤ
+ *
+ * configure ǥޥפμưȽ̤
+ * ԤʤʥǥեȡˡLinux ǤϤOSSѥ󥿥ե򤵤ޤ
+ * ¾Υ󥿥ե (ALSA, esd, portaudio, spAudio) Ѥ
+ * configure  "--with-mictype=TYPE" Ū˻ꤷƲ
+ *
+ * ɥɤ 16bit Υ ϿǤ뤳ȤɬܤǤ
+ * LinuxǤϡƥ쥪ϿǤʤǥХξ硤
+ * ͥΤߤϤȤƼФǧ뤳ȤǤޤ
+ *
+ * JuliusLinuxǤϥߥǥХڹԤޤϿǥХ
+ * ʥޥ/饤ˤϿܥ塼Ĵ xmixer ʤ¾Υġ
+ * ԤʤäƲ
+ *
+ * ǥեȤΥǥХ̾ "/dev/dsp" ǤĶѿ AUDIODEV 
+ * ǥХ̾ꤹ뤳Ȥǡ¾ΥǥХ̾ѤǤޤ
+ * </JA>
+ * <EN>
+ * @brief  Microphone input on Linux/OSS
+ *
+ * Low level I/O functions for microphone input on Linux using OSS API.
+ * Works on kernel driver, OSS/Linux, ALSA OSS compatible device, and
+ * other OSS compatible drivers.  This will work on kernel 2.0.x and later.
+ *
+ * This OSS API is used by default on Linux machines.  If you want
+ * another API (like ALSA native, esd, portaudio, spAudio), please specify
+ * "--with-mictype=TYPE" options at compilation time to configure script.
+ *
+ * Sound card should support 16bit monaural recording.  On Linux, however,
+ * if you unfortunately have a device with only stereo recording support,
+ * Julius will try to get input from the left channel of the stereo input.
+ *
+ * Julius does not alter any mixer device setting at all on Linux.  You should
+ * configure the mixer for recording source (mic/line) and recording volume
+ * correctly using other audio tool such as xmixer.
+ *
+ * The default device name is "/dev/dsp", which can be changed by setting
+ * environment variable AUDIODEV.
+ * </EN>
+ *
+ * $Revision: 1.2 $
+ * 
  */
-
-/* adin_mic_linux_oss.c --- adin microphone library for OSS API */
-
-/* $Id: adin_mic_linux_oss.c,v 1.8 2003/10/04 03:12:33 ri Exp $ */
-
-/* for standard sound drivers in linux-2.0.x, 2.2.x */
-/* for OSS/Linux, OSS/Free or other OSS compatible API */
-/* see http://www.opensound.com/ for OSS/Linux */
-
-/* Tested on kernel-2.2.14 with OSS/Linux / ALSA emulation mode */
-
-/* Changes from rev.3.0 --- strictly conforming to the programming guide */
-
 /*
- * Use other mixer program to setup mic device
- * (mute/unmute, volume control, etc.)
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * All rights reserved
  */
 
 #include <sent/stddefs.h>
@@ -27,37 +66,55 @@
 #include <sys/ioctl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/time.h>
 #include <fcntl.h>
-#include <unistd.h>
 
 /* sound header */
 #include <sys/soundcard.h>
 
-#define FREQALLOWRANGE 200	/* acceptable sampling frequency width around 16kHz */
-#define MAXPOLLINTERVAL 300	/* adin read timeout interval (msec) */
-#define FRAGMENT_POWER 10	/* data fragment size (= input delay)
-				  2^10 = 1024 byte = 32ms in 16kHz */
+/// Default device name, can be overridden by AUDIODEV environment variable
+#define DEFAULT_DEVICE "/dev/dsp"
 
-static int audio_fd;		/* audio descriptor */
-static boolean need_swap;	/* whether samples need byte swap */
-static int fmt;			/* signed 16bit */
-static int frag_size;		/* data fragment size */
-static int poll_interval;
-static boolean stereo_rec;	/* stereo recording for some cards...(use left only) */
+#define FREQALLOWRANGE 200	///< Acceptable width of sampling frequency
+#define MAXPOLLINTERVAL 300	///< Read timeout in msec.
+/**
+ * Data fragment size.  Input will be delayed by 2^X bytes.
+ * The default value of 10 means we treat input per a unit of 2~10=1024 bytes,
+ * which corresponds to 32 msec in 16kHz sampling.
+ * 
+ */
+#define FRAGMENT_POWER 10
 
-/* check audio port resource and initialize */
-/* will be called once at startup time */
+static int audio_fd;		///< Audio descriptor
+static boolean need_swap;	///< Whether samples need byte swap
+static int frag_size;		///< Actual data fragment size
+static boolean stereo_rec;	///< TRUE if stereo recording (use left only)
+
+/** 
+ * Device initialization: check device capability and open for recording.
+ * 
+ * @param sfreq [in] required sampling frequency.
+ * @param dummy [in] a dummy data
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 adin_mic_standby(int sfreq, void *dummy)
 {
-  int fmt_can;
-  int fmt1, fmt2;                /* sampling format */
-  int rfmt;
+  int fmt, fmt_can, fmt1, fmt2, rfmt; /* sampling format */
   int samplerate;	/* 16kHz */
+  char *defaultdev = DEFAULT_DEVICE; /* default device */
+  char *devname;
+
+  /* set device name */
+  if ((devname = getenv("AUDIODEV")) == NULL) {
+    devname = defaultdev;
+  }
 
   /* open device */
-  if ((audio_fd = open("/dev/dsp", O_RDONLY|O_NONBLOCK)) == -1) {
-    perror("adin_mic_standby: open /dev/dsp");
+  if ((audio_fd = open(devname, O_RDONLY|O_NONBLOCK)) == -1) {
+    j_printerr("adin_mic_standby: device=%s\n", devname);
+    perror("adin_mic_standby: open device");
     return(FALSE);
   }
 
@@ -97,8 +154,9 @@ adin_mic_standby(int sfreq, void *dummy)
 
   /* re-open for recording */
   /* open device */
-  if ((audio_fd = open("/dev/dsp", O_RDONLY)) == -1) {
-    perror("adin_mic_standby: open /dev/dsp");
+  if ((audio_fd = open(devname, O_RDONLY)) == -1) {
+    j_printerr("adin_mic_standby: device=%s\n", devname);
+    perror("adin_mic_standby: open device");
     return(FALSE);
   }
 
@@ -231,7 +289,11 @@ adin_mic_standby(int sfreq, void *dummy)
   return TRUE;
 }
  
-/* start recording */
+/** 
+ * Start recording.
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 adin_mic_start()
 {
@@ -247,7 +309,11 @@ adin_mic_start()
   return(TRUE);
 }
 
-/* stop recording */
+/** 
+ * Stop recording.
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 adin_mic_stop()
 {
@@ -260,8 +326,23 @@ adin_mic_stop()
   return TRUE;
 }
 
-/* read samples from audio device */
-/* try to read `sampnum' samples and returns actual sample num recorded */
+/**
+ * @brief  Read samples from device
+ * 
+ * Try to read @a sampnum samples and returns actual number of recorded
+ * samples currently available.  This function will block at most
+ * MAXPOLLINTERVAL msec, until at least one sample can be obtained.
+ * If no data has been obtained after waiting for MAXPOLLINTERVAL msec,
+ * returns 0.
+ * 
+ * When stereo input, only left channel will be used.
+ * 
+ * @param buf [out] samples obtained in this function
+ * @param sampnum [in] wanted number of samples to be read
+ * 
+ * @return actural number of read samples, 0 of no sample has been captured in
+ * MAXPOLLINTERVAL msec, -2 if error.
+ */
 int
 adin_mic_read(SP16 *buf, int sampnum)
 {
@@ -295,6 +376,7 @@ adin_mic_read(SP16 *buf, int sampnum)
     size = sampnum * sizeof(SP16);
     if (size > info.bytes) size = info.bytes;
     if (size < frag_size) size = frag_size;
+    size &= ~ 1;		/* Force 16bit alignment */
     cnt = read(audio_fd, buf, size);
     if ( cnt < 0 ) {
       perror("adin_mic_read: read error\n");
