@@ -1,16 +1,41 @@
-/* Copyright (c) 1991-2002 Doshita Lab. Speech Group, Kyoto University */
-/* Copyright (c) 2000-2002 Speech and Acoustics Processing Lab., NAIST */
-/*   All rights reserved   */
-
-/* adin_mic_o2.c --- adin microphone library for IRIX6.x */
-
-/* $Id: adin_mic_o2.c,v 1.2 2002/09/11 22:01:50 ri Exp $ */
-
-/* test on IRIX6.4 (SGI O2 workstation) */
-
+/**
+ * @file   adin_mic_o2.c
+ * @author Akinobu LEE
+ * @date   Sun Feb 13 18:42:22 2005
+ * 
+ * <JA>
+ * @brief  ޥ (SGI IRIX)
+ *
+ * SGI IRIX ΥޥϤѤ٥벻ϴؿǤ
+ * IRIXޥǤϥǥեȤǤ줬Ѥޤ
+ *
+ * SGI O2 Workstation (IRIX6.3) ưǧ򤷤Ƥޤ
+ *
+ * ư奪ǥϤϥޥ˼ưŪڤؤޤ
+ * ܥ塼ϼưĴᤵޤapanelޥɤĴᤷƤ 
+ * </JA>
+ * <EN>
+ * @brief  Microphone input on SGI IRIX machine
+ *
+ * Low level I/O functions for microphone input on a SGI IRIX machine.
+ * This file is used as default on IRIX machines.
+ *
+ * Tested on IRIX 6.3, SGI O2 Workstation.
+ *
+ * The microphone input device will be automatically selected by Julius
+ * on startup.  Please note that the recoding volue will not be
+ * altered by Julius, and appropriate value should be set by another tool
+ * such as apanel.
+ * </EN>
+ *
+ * $Revision: 1.2 $
+ * 
+ */
 /*
- * Other mixer program is needed to setup mic device
- * (mute/unmute, volume control, etc.)
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * All rights reserved
  */
 
 #include <sent/stddefs.h>
@@ -18,10 +43,16 @@
 
 /* sound header */
 #include <audio.h>
-static ALconfig ac;		/* local port settings */
-static ALport aport;		/* audio port */
+static ALconfig ac;		///< Local port settings
+static ALport aport;		///< Audio port
 
-/* sub function: initialize global audio interface */
+/** 
+ * Initialize global audio interface to use microphone input.
+ * 
+ * @param rate [in] sampling rate in Hz
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 static boolean
 adin_o2_setup_global(double rate)
 {
@@ -42,8 +73,14 @@ adin_o2_setup_global(double rate)
   }
 }
 
-/* check audio port resource and initialize */
-/* will be called once at startup time */
+/** 
+ * Device initialization: check device capability and open for recording.
+ * 
+ * @param sfreq [in] required sampling frequency.
+ * @param dummy [in] a dummy data
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 adin_mic_standby(int sfreq, void *dummy)
 {
@@ -73,8 +110,11 @@ adin_mic_standby(int sfreq, void *dummy)
   return(TRUE);
 }
   
-
-/* start recording */
+/** 
+ * Start recording.
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 adin_mic_start()
 {
@@ -88,7 +128,11 @@ adin_mic_start()
   return(TRUE);
 }
 
-/* stop recording */
+/** 
+ * Stop recording.
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 adin_mic_stop()
 {
@@ -97,8 +141,18 @@ adin_mic_stop()
   return(TRUE);
 }
 
-/* read samples from audio device */
-/* try to read `sampnum' samples and returns actual sample num recorded */
+/**
+ * @brief  Read samples from device
+ * 
+ * Try to read @a sampnum samples and returns actual number of recorded
+ * samples currently available.  This function will block until
+ * at least one sample can be obtained.
+ * 
+ * @param buf [out] samples obtained in this function
+ * @param sampnum [in] wanted number of samples to be read
+ * 
+ * @return actural number of read samples, -2 if an error occured.
+ */
 int
 adin_mic_read(SP16 *buf, int sampnum)
 {
