@@ -44,13 +44,13 @@
  * @bug Does not work between different machine byte order.
  * </EN>
  *
- * $Revision:$
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -247,3 +247,21 @@ adin_tcpip_send_resume()
   if (count < 0) j_printerr("adin_tcpip_send_resume: cannot send command to client\n");
   return TRUE;
 }
+
+/** 
+ * Tell the adinnet client to terminate transfer.
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
+boolean
+adin_tcpip_send_terminate()
+{
+   int count;
+   char com;
+   /* send terminate command to adinnet client */
+   com = '2';
+   count = wt(adinnet_asd, &com, 1);
+   if (count < 0) j_printerr("adin_tcpip_send_terminate: cannot send command to client\n");
+   return TRUE;
+}
+
