@@ -1,21 +1,38 @@
+/**
+ * @file   paramtypes.c
+ * @author Akinobu LEE
+ * @date   Tue Feb 15 00:06:26 2005
+ *
+ * <JA>
+ * @brief  ħѥ᡼ʸɽȥХʥɽѴ
+ *
+ * Υեδؿϡħѥ᡼ʸɽ"MFCC_E_D_Z" ʤɡ
+ *  HTK  short ɽХʥȤѴԤʤޤ
+ * </JA>
+ * <EN>
+ * @brief  Convert between string and binary expression of parameter type
+ *
+ * The functions in this file converts the expression of parameter type,
+ * between string (ex. "MFCC_E_D_Z") and internal binary format used in HTK.
+ * </EN>
+ *
+ * $Revision: 1.2 $
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* paramtypes.c --- HTK Parameter type utility functions */
-/*                  convert string like "MFCC_E_D" with internal code */
-
-/* $Id: paramtypes.c,v 1.3 2003/09/29 06:01:23 ri Exp $ */
-
 #include <sent/stddefs.h>
 #include <sent/htk_defs.h>
 #include <sent/htk_param.h>
 
-
-/* below are used both in HMM and in ParameterFile */
-static OptionStr pbase[] = {	/* parameter base types */
+
+/// Database that relates base type strings to binary code and description string.
+static OptionStr pbase[] = {
   {"WAVEFORM", F_WAVEFORM, "sampled waveform", FALSE},
   {"DISCRETE", F_DISCRETE, "Discrete", FALSE},
   {"LPC", F_LPC, "LPC", TRUE},
@@ -28,7 +45,8 @@ static OptionStr pbase[] = {	/* parameter base types */
   {"USER", F_USER, "user defined sample kind", TRUE},
   {NULL,0,NULL,FALSE}
 };
-static OptionStr pqual[] = {	/* parameter qualifier */
+/// Database that relates qualifier type strings to binary code and description string.
+static OptionStr pqual[] = {
   {"_E", F_ENERGY, "log energy coef.", TRUE},
   {"_N", F_ENERGY_SUP, "uppress absolute energy", TRUE},
   {"_D", F_DELTA, "delta coef.", TRUE},
@@ -40,10 +58,14 @@ static OptionStr pqual[] = {	/* parameter qualifier */
   {NULL,0,NULL,FALSE}
 };
 
-
-/* from string to code */
-
-short				/* returns param_type code for qualifiers */
+/** 
+ * Convert a qualifier string to a binary type code.
+ * 
+ * @param s [in] a string that contains qualifier strings like "_E_D_Z"
+ * 
+ * @return the converted internal binary type code, F_ERR_INVALID if failed.
+ */
+short
 param_qualstr2code(char *s)
 {
   int i, qlen;
@@ -72,7 +94,14 @@ param_qualstr2code(char *s)
   return(qual_type);
 }
 
-short				/* returns param_type code */
+/** 
+ * Convert a type string that contains basename and qualifiers to a binary type code.
+ * 
+ * @param s [in] a string that contains base and qualifier string like "MFCC_E_D_Z"
+ * 
+ * @return the converted internal binary type code, F_ERR_INVALID if failed.
+ */
+short
 param_str2code(char *s)
 {
   int i;
@@ -105,13 +134,18 @@ param_str2code(char *s)
   return(F_ERR_INVALID);
 }
 
-
-/* from code to str */
-char *				/* return pointer to given buffer (NULL on err)*/
-param_qualcode2str(
-     char *buf,			/* buffer to store result (must have enough length) */
-     short type,		/* qualifier type code */
-     boolean descflag)		/* set TRUE if want detailed description */
+/** 
+ * Convert the qualifier part of a binary type code to string.
+ *
+ * @param buf [out] buffer to store the resulting string (must have enough length)
+ * @param type [in] binary type code to convert.
+ * @param descflag [in] set to TRUE if you want result in description string
+ * instead of qualifier string.
+ * 
+ * @return @a buf on success, NULL on failure.
+ */
+char *
+param_qualcode2str(char *buf, short type, boolean descflag)
 {
   int i;
 
@@ -129,12 +163,18 @@ param_qualcode2str(
   return(buf);
 }
 
-
-char *				/* return pointer to given buffer (NULL on err)*/
-param_code2str(
-     char *buf,			/* buffer to store result (must have enough length) */
-     short type,		/* parameter type code */
-     boolean descflag)		/* set TRUE if want detailed description */
+/** 
+ * Convert a binary type code to string.
+ *
+ * @param buf [out] buffer to store the resulting string (must have enough length)
+ * @param type [in] binary type code to convert.
+ * @param descflag [in] set to TRUE if you want result in description string
+ * instead of base and qualifier string.
+ * 
+ * @return @a buf on success, NULL on failure.
+ */
+char *
+param_code2str(char *buf, short type, boolean descflag)
 {
   int i;
   short btype;
@@ -162,4 +202,3 @@ param_code2str(
 
   return(buf);
 }
-
