@@ -1,22 +1,42 @@
+/**
+ * @file   strip.c
+ * @author Akinobu LEE
+ * @date   Tue Feb 15 00:30:38 2005
+ *
+ * <JA>
+ * @brief  ǡ饼ץ
+ * </JA>
+ * <EN>
+ * @brief  Strip zero samples from speech data
+ * </EN>
+ *
+ * $Revision: 1.2 $
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* strip.c --- strip zero segments from speech data */
-
-/* $Id: strip.c,v 1.3 2003/09/29 06:01:23 ri Exp $ */
-
 #include <sent/stddefs.h>
 #include <sent/speech.h>
 
-/* distinction function: sequence of 0 and -2^15 is invalid */
+/// Distinction function: sequence of 0 and -2^15 are invalid
 #define IS_INVALID_SAMPLE(A) ((A) == 0 || (A) == -32767)
-/* length to detect as invalid */
-#define WINDOWLEN 16		/* 1ms in 16kHz */
+/// Length of zero sample to detect as invalid sequence.
+#define WINDOWLEN 16
 
-int				/* return length after stripping */
+/** 
+ * Strip zero samples from speech data.
+ * 
+ * @param a [I/O] speech data
+ * @param len [in] length of above
+ * 
+ * @return new length after stripping.
+ */
+int
 strip_zero(SP16 a[], int len)
 {
   int src,dst;
@@ -64,4 +84,3 @@ strip_zero(SP16 a[], int len)
   
   return(dst);
 }
-
