@@ -41,13 +41,13 @@
  * misdetected.  When the misdetection frequently occurs on your MFCC file,
  * please try "-nostrip" option to turn off the stripping.
  *
- * $Revision:$
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -81,7 +81,7 @@ guess_abs_e_location(HTK_Param *param)
     }
   } else {
     /* absolute energy not included */
-    j_printerr("Info: absolute energy coef. not found, stripping disabled\n");
+    //j_printerr("Info: absolute energy coef. not found, stripping disabled\n");
     abs_e_num = -1;
   }
   return abs_e_num;
@@ -108,7 +108,7 @@ param_strip_zero(HTK_Param *param)
   dst = 0;
   for(src=0;src<param->samplenum;src++) {
     if (IS_INVALID_FRAME_MFCC(param->parvec[src][eloc])) {
-      j_printf("Warning: frame %d is invalid, stripped\n", src);
+      j_printf("Warning: frame %d has invalid energy, stripped\n", src);
       free(param->parvec[src]);
       continue;
     }
@@ -118,6 +118,7 @@ param_strip_zero(HTK_Param *param)
     dst++;
   }
   if (dst != param->samplenum) {
+    j_printf("Warning: input shrinked from %d to %d frames\n", param->samplenum, dst);
     param->header.samplenum = param->samplenum = dst;
   }
 
