@@ -1,36 +1,60 @@
+/**
+ * @file   mkterminfo.c
+ * @author Akinobu LEE
+ * @date   Tue Feb 15 14:47:27 2005
+ * 
+ * <JA>
+ * @brief  ƥꤴȤñΥꥹȤ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Make a word list for each category
+ * </EN>
+ * 
+ * $Revision: 1.2 $
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* mkterminfo.c --- extract word index of words from DFA and WORD_INFO */
-/* word IDs must be upword-sorted */
-
-/* $Id: mkterminfo.c,v 1.5 2003/09/29 06:01:23 ri Exp $ */
-
 #include <sent/stddefs.h>
 #include <sent/dfa.h>
 #include <sent/vocabulary.h>
 
 /* construct table: terminal(category) ID -> word list */
+
+/** 
+ * Make a word list for each category.
+ * 
+ * @param tinfo [i/o] terminal data structure to hold the result
+ * @param dinfo [in] DFA gammar to supply the number of category in the grammar
+ * @param winfo [in] word dictionary.
+ */
 void
 make_terminfo(TERM_INFO *tinfo, DFA_INFO *dinfo, WORD_INFO *winfo)
 {
   int i,w,t;
   int tnum;
-  
-  tnum = tinfo->term_num = dinfo->term_num;
 
+  /* set terminal number */
+  tnum = tinfo->term_num = dinfo->term_num;
+  /* allocate memory for the list of word num */
   tinfo->wnum = (int *)mymalloc(sizeof(int) * tnum);
+  /* count number of belonging words for each category */
   for(i=0;i<tnum;i++) tinfo->wnum[i]=0;
   for(w=0;w<winfo->num;w++) {
     tinfo->wnum[winfo->wton[w]]++;
   }
+  /* allocate memory for the list body */
   tinfo->tw = (WORD_ID **)mymalloc(sizeof(WORD_ID *) * tnum);
   for(i=0;i<tnum;i++) {
     tinfo->tw[i] = (WORD_ID *)mymalloc(sizeof(WORD_ID) * tinfo->wnum[i]);
   }
+  /* store the word to each corresponding list */
   for(i=0;i<tnum;i++) tinfo->wnum[i]=0;
   for(w=0;w<winfo->num;w++) {
     t = winfo->wton[w];
@@ -39,13 +63,20 @@ make_terminfo(TERM_INFO *tinfo, DFA_INFO *dinfo, WORD_INFO *winfo)
   }
 
 }
-  
+
+/** 
+ * Append the terminal(category) word list.
+ * 
+ * @param dst [i/o] category data
+ * @param src [i/o] category data to be appended to @a dst
+ * @param coffset [in] category id offset in @a dst where the new data should be stored
+ * @param woffset [in] word id offset where the new data should be stored
+ */
 void
 terminfo_append(TERM_INFO *dst, TERM_INFO *src, int coffset, int woffset)
 {
   int t, new_termnum;
   int i, j;
-  WORD_ID w;
 
   new_termnum = coffset + src->term_num;
   if (dst->tw == NULL) {
