@@ -1,17 +1,36 @@
-/* Copyright (c) 1991-2002 Doshita Lab. Speech Group, Kyoto University */
-/* Copyright (c) 2000-2002 Speech and Acoustics Processing Lab., NAIST */
-/*   All rights reserved   */
-
-/* rddfa.c --- read in DFA definition file to DFA_INFO */
-
-/* $Id: rddfa.c,v 1.6 2002/09/11 22:01:50 ri Exp $ */
+/**
+ * @file   rddfa.c
+ * @author Akinobu LEE
+ * @date   Tue Feb 15 14:54:40 2005
+ * 
+ * <JA>
+ * @brief  DFAʸˡɤ߹
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Read DFA grammar from a file
+ * </EN>
+ * 
+ * $Revision: 1.4 $
+ * 
+ */
+/*
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * All rights reserved
+ */
 
 #include <sent/stddefs.h>
 #include <sent/dfa.h>
 
-static char buf[MAXLINELEN];
+static char buf[MAXLINELEN];	///< Local text buffer
 
-/* initialize state buffer */
+/** 
+ * Initialize and allocate DFA state information list in the grammar.
+ * 
+ * @param dinfo [i/o] DFA grammar
+ */
 void
 dfa_state_init(DFA_INFO *dinfo)
 {
@@ -27,7 +46,12 @@ dfa_state_init(DFA_INFO *dinfo)
   dinfo->sp_id = WORD_INVALID;
 }
 
-/* expand state buffer by DFA_STATESTEP (at least needed) */
+/** 
+ * Expand the state information list to the required length.
+ * 
+ * @param dinfo [i/o] DFA grammar
+ * @param needed [in] required new length
+ */
 void
 dfa_state_expand(DFA_INFO *dinfo, int needed)
 {
@@ -43,7 +67,14 @@ dfa_state_expand(DFA_INFO *dinfo, int needed)
   }
 }
 
-/* read DFA info from fp */
+/** 
+ * Top loop function to read DFA grammar via file pointer
+ * 
+ * @param fp [in] file pointer that points to the DFA grammar data
+ * @param dinfo [out] the read data will be stored in this DFA grammar structure
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 rddfa(FILE *fp, DFA_INFO *dinfo)
 {
@@ -66,6 +97,14 @@ rddfa(FILE *fp, DFA_INFO *dinfo)
   return(TRUE);
 }
 
+/** 
+ * Top loop function to read DFA grammar via file descriptor
+ * 
+ * @param fd [in] file descriptor that points to the DFA grammar data
+ * @param dinfo [out] the read data will be stored in this DFA grammar structure
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 rddfa_fd(int fd, DFA_INFO *dinfo)
 {
@@ -87,15 +126,54 @@ rddfa_fd(int fd, DFA_INFO *dinfo)
   dinfo->term_num = terminal_max + 1;
   return(TRUE);
 }
-  
 
+/** 
+ * Top loop function to read DFA grammar via socket descriptor
+ * 
+ * @param sd [in] socket descriptor that points to the DFA grammar data
+ * @param dinfo [out] the read data will be stored in this DFA grammar structure
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
+boolean
+rddfa_sd(int sd, DFA_INFO *dinfo)
+{
+  int state_max, arc_num, terminal_max;
+
+  /* initialize */
+  dfa_state_init(dinfo);
+  state_max = 0;
+  arc_num = 0;
+  terminal_max = 0;
+
+  while(getl_sd(buf, MAXLINELEN, sd) != NULL) {
+    if (rddfa_line(buf, dinfo, &state_max, &arc_num, &terminal_max) == FALSE) {
+      break;
+    }
+  }
+  dinfo->state_num = state_max + 1;
+  dinfo->arc_num = arc_num;
+  dinfo->term_num = terminal_max + 1;
+  return(TRUE);
+}
+
+/** 
+ * Parse the input line and set grammar information, one by line.
+ * 
+ * @param line [in] text buffer that holds a line of DFA file
+ * @param dinfo [i/o] the read data will be appended to this DFA data
+ * @param state_max [i/o] maximum number of state id appeared, will be updated
+ * @param arc_num [i/o] number of read arcs, will be updated
+ * @param terminal_max [i/o] maximum number of state id appended, will be updated
+ * 
+ * @return TRUE if the line was successfully parsed, FALSE if failed.
+ */
 boolean
 rddfa_line(char *line, DFA_INFO *dinfo, int *state_max, int *arc_num, int *terminal_max)
 {
   DFA_ARC *newarc;
   int state, terminal, next_state;
   unsigned int status;
-  int i;
 
   if (strmatch(buf, "DFAEND")) return(FALSE);
   /* format: state terminalID nextstate statuscode_of_state */
@@ -139,6 +217,15 @@ rddfa_line(char *line, DFA_INFO *dinfo, int *state_max, int *arc_num, int *termi
 
 /* append dfa info to other */
 /* soffset: state offset  coffset: category(terminal) offset */
+
+/** 
+ * Append the DFA state information to other
+ * 
+ * @param dst [i/o] DFA grammar
+ * @param src [i/o] DFA grammar to be appended to @a dst
+ * @param soffset [in] offset state number in @a dst where the new state should be stored
+ * @param coffset [in] category id offset in @a dst where the new data should be stored
+ */
 void
 dfa_append(DFA_INFO *dst, DFA_INFO *src, int soffset, int coffset)
 {
@@ -175,8 +262,8 @@ dfa_append(DFA_INFO *dst, DFA_INFO *src, int soffset, int coffset)
       dst->st[state].arc = newarc;
 
       dst->arc_num++;
-      if (dst->state_num < state + 1) dst->state_num = state + 1;
       if (dst->term_num < terminal + 1) dst->term_num = terminal + 1;
     }
+    if (dst->state_num < state + 1) dst->state_num = state + 1;
   }
 }
