@@ -1,38 +1,58 @@
+/**
+ * @file   check_hmm_restriction.c
+ * @author Akinobu LEE
+ * @date   Tue Feb 15 19:00:58 2005
+ * 
+ * <JA>
+ * @brief  Ϳ줿 %HMM ܤѲǽʷɤå
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Check if the given %HMM definition file can be used
+ * </EN>
+ *
+ * TRANSITION RESTRICTIONS:
+ *
+ *   - for HTK and Julius:
+ *      - no arc to initial state
+ *      - no arc from final state
+ * 
+ *   - Normal version of Julius:
+ *      - should have at least one output state
+ *      - allow only one arc from initial state
+ *      - allow only one arc to final state
+ *        (internal skip/loop is allowed)
+ *
+ *   - Multipath version of Julius:
+ *      - should have at least one output state
+ *
+ * In multipath version, all the transitions including model-skipping
+ * transition is allowed.  However, in normal version, their transition
+ * is restricted as above.
+ * 
+ * If such transition is found, Julius output warning and
+ * proceed by modifying transition to suite for the restriction.
+ *
+ * $Revision: 1.3 $
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* check_hmm_restriction.c --- check if Julius can handle the hmmdef */
-
-/* $Id: check_hmm_restriction.c,v 1.5 2003/09/29 06:01:23 ri Exp $ */
-
 #include <sent/stddefs.h>
 #include <sent/htk_hmm.h>
 #include <sent/htk_param.h>
 #include <sent/hmm.h>
 
-#define GOOD 0
-#define FIXED 1
-#define BAD 3
+#define GOOD 0			///< Mark transition as allowed
+#define FIXED 1			///< Mark fixed transition 
+#define BAD 3			///< Mark not supported transition
 
-/*
- *
- *  TRANSITION RESTRICTIONS:
- *
- * for HTK and Julius:
- *   no arc to initial state
- *   no arc from final state
- * 
- * for Julius only:
- *   allow only one arc from initial state
- *   allow only one arc to final state
- *   (internal skip/loop is allowed)
- *
- */
-
-/* transition allowance matrix */
+/* transition allowance matrix on normal version */
 /* G=good,F=fixed,B=bad */
 /*  |G|F|B */
 /* -+-+-+- */
@@ -40,18 +60,54 @@
 /* F|F|F|B */
 /* B|B|B|B */
 
-/* When unacceptable transition is found,
-   Julius output only warning and proceed by modifying transition to
-   suite for the restriction.
-*/
-
 /* transition probability is not loganized yet */
 
-/* check transition of t */
+/** 
+ * Scan the transition matrix to test the ristrictions.
+ * 
+ * @param t [in] a transition matrix to be tested
+ * 
+ * @return 0 if it conforms, 1 if unacceptable transition was found and
+ * modification forced, 3 if totally unsupported transition as included and
+ * cannot by handled.
+ */
 static int
-trans_ok_p(t)
-     HTK_HMM_Trans *t;
+trans_ok_p(HTK_HMM_Trans *t)
 {
+#ifdef MULTIPATH_VERSION
+  int i;
+  int tflag;
+  int retflag = GOOD;
+
+  /* no arc to initial state */
+  tflag = GOOD;
+  for (i=0;i<t->statenum;i++) {
+    if (t->a[i][0] != (PROB)0.0) {
+      tflag = BAD;
+      break;
+    }
+  }
+  if (tflag == BAD) {
+    j_printerr("Error: transition to initial state not allowed\n");
+    retflag = BAD;
+  }
+  /* no arc from final state */
+  tflag = GOOD;
+  for (i=0;i<t->statenum;i++) {
+    if (t->a[t->statenum-1][i] != (PROB)0.0) {
+      tflag = BAD;
+      break;
+    }
+  }
+  if (tflag == BAD) {
+    j_printerr("Error: transition from final state not allowed\n");
+    retflag = BAD;
+  }
+    
+  return(retflag);
+  
+#else  /* ~MULTIPATH_VERSION */
+  
   int i, j;
   int tflag = BAD;
   int retflag = BAD;
@@ -138,9 +194,17 @@ trans_ok_p(t)
   }
     
   return(retflag | tflag);
+
+#endif /* MULTIPATH_VERSION */
 }
 
-/* check HMM limitation (call trans_ok_p() above) */
+/** 
+ * Check if the transition matrix conforms the ristrictions of Julius.
+ * 
+ * @param dt [in] HTK %HMM model to check.
+ * 
+ * @return TRUE on success, FALSE if the check failed.
+ */
 boolean
 check_hmm_limit(HTK_HMM_Data *dt)
 {
@@ -158,17 +222,22 @@ check_hmm_limit(HTK_HMM_Data *dt)
     j_printerr("SERIOUS WARNING: This may cause unintended recognition result\n");
     put_htk_trans(dt->tr);
   }
-#if 0
-  if (dt->state_num <= 3) {
+#ifdef MULTIPATH_VERSION
+  if (dt->tr->statenum < 3) {
     return_flag = FALSE;
-    j_printerr("Limit: HMM \"%s\" too short (only %d states)\n", dt->name,
-	    dt->state_num);
+    j_printerr("Limit: HMM \"%s\" has no output state (statenum=%d)\n", dt->name, dt->tr->statenum);
   }
 #endif
   return(return_flag);
 }
 
-/* check all defined HMM */
+/** 
+ * Check all the %HMM definitions in a HTK %HMM definition data.
+ * 
+ * @param hmminfo [in] HTK %HMM data to check.
+ * 
+ * @return TRUE if there was no bad models, FALSE if at least one model is bad.
+ */
 boolean
 check_all_hmm_limit(HTK_HMM_INFO *hmminfo)
 {
@@ -183,3 +252,33 @@ check_all_hmm_limit(HTK_HMM_INFO *hmminfo)
   return(return_flag);
 }
 
+
+#ifdef MULTIPATH_VERSION
+/** 
+ * <JA>
+ * ǥ뤬Ͼ֤ͳϾ֤Ͼ֤ľܤ褦
+ * ܤĤɤå롥
+ * 
+ * @param d [in] HMM
+ * 
+ * @return ϤϤؤľܤľ TRUE, ʤ FALSE ֤
+ * </JA>
+ * <EN>
+ * Check if the model has direct transition from initial state to final state,
+ * skipping all the output state.
+ * 
+ * @param d [in] logical HMM
+ * 
+ * @return TRUE if it has direct transition from initial state to final state,
+ * that is, this is a "skippable" model.  Otherwise, return FALSE.
+ * </EN>
+ */
+boolean
+is_skippable_model(HTK_HMM_Data *d)
+{
+  if (d->tr->a[0][d->tr->statenum-1] != LOG_ZERO) {
+    return TRUE;
+  }
+  return FALSE;
+}
+#endif
