@@ -1,33 +1,54 @@
+/**
+ * @file   chkhmmlist.c
+ * @author Akinobu LEE
+ * @date   Tue Feb 15 19:17:51 2005
+ * 
+ * <JA>
+ * @brief  HMMList Ƥβǽʥȥ饤ե󤬴ޤޤƤ뤫å
+ *
+ * Ϳ줿ǥˤơǧо줷ƤΥȥ饤ե
+ * %HMM뤤HMMListƤ뤫ɤåޤ
+ *
+ * åˤϼθäθޤʤͿ줿äñ
+ * ñ֤ȥ饤եˤĤƤΤߥåޤ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Check existence of all possible triphone in HMMList.
+ *
+ * These functions check whether all the possible triphones that may appear
+ * while recognition process is fully defined or mapped in %HMM definition
+ * file and HMMList file.
+ *
+ * Word dictionary is considered for the test. Only triphones that can
+ * appear as word-internal triphones and cross-word triphones on the given
+ * dictionary will be considered.
+ * </EN>
+ * 
+ * $Revision: 1.3 $
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* chkhmmlit.c --- check existence of all possible triphone in HMMList */
-
-/* $Id: chkhmmlist.c,v 1.4 2003/09/29 06:01:23 ri Exp $ */
-
-/* consider vocabulary: only ones that can be appear
-   in word-internal and cross-word are checked */
-
-/* two main routine:
-   make_base_phone():  build basephone info - basephone list and bgn/end mark
-   test_interword_triphone(): check triphone using the basephone info
- */
-
-
 #include <sent/htk_hmm.h>
 #include <sent/vocabulary.h>
 
-/* make a list of base phone */
-/* by gathering center phone name from logical HMM name */
-static void
+/** 
+ * Build a list of base phones by gathering center phones of the defined %HMM.
+ * 
+ * @param hmminfo [i/o] %HMM definition data
+ */
+void
 make_hmm_basephone_list(HTK_HMM_INFO *hmminfo)
 {
   HMM_Logical *lg;
-  char p[50];
-  BASEPHONE *match, *new;
+  static char p[MAX_HMMNAME_LEN];
+  BASEPHONE *match = NULL, *new;
   APATNODE *root;
   int n;
 
@@ -39,10 +60,10 @@ make_hmm_basephone_list(HTK_HMM_INFO *hmminfo)
       match = aptree_search_data(p, root);
       if (strmatch(match->name, p)) continue;
     }
-    new = (BASEPHONE *)mybmalloc(sizeof(BASEPHONE));
+    new = (BASEPHONE *)mybmalloc2(sizeof(BASEPHONE), &(hmminfo->mroot));
     new->bgnflag = FALSE;
     new->endflag = FALSE;
-    new->name = (char *)mybmalloc(strlen(p)+1);
+    new->name = (char *)mybmalloc2(strlen(p)+1, &(hmminfo->mroot));
     strcpy(new->name, p);
     if (root == NULL) root = aptree_make_root_node(new);
     else aptree_add_entry(new->name, new, match->name, &root);
@@ -52,24 +73,44 @@ make_hmm_basephone_list(HTK_HMM_INFO *hmminfo)
   hmminfo->basephone.root = root;
 }
 
-/* print all basephones */
+/** 
+ * Traverse function callback to output detailed information of a basephone in text to stdout.
+ * 
+ * @param x [in] pointer to a basephone data.
+ */
 static void
 print_callback_detail(void *x)
 {
   BASEPHONE *b = x;
   j_printf("\"%s\": bgn=%d, end=%d\n", b->name, b->bgnflag, b->endflag);
 }
-void
-print_all_basephone_detail(HMM_basephone *base)
-{
-  aptree_traverse_and_do(base->root, print_callback_detail);
-}
+
+/** 
+ * Traverse function callback to output name of a basephone in text to stdout.
+ * 
+ * @param x [in] pointer to a basephone data.
+ */
 static void
 print_callback_name(void *x)
 {
   BASEPHONE *b = x;
   j_printf("%s, ", b->name);
 }
+/** 
+ * Output all basephone informations to stdout.
+ * 
+ * @param base [in] pointer to the top basephone data holder.
+ */
+void
+print_all_basephone_detail(HMM_basephone *base)
+{
+  aptree_traverse_and_do(base->root, print_callback_detail);
+}
+/** 
+ * Output all basephone names to stdout
+ * 
+ * @param base [in] pointer to the top basephone data holder.
+ */
 void
 print_all_basephone_name(HMM_basephone *base)
 {
@@ -77,8 +118,14 @@ print_all_basephone_name(HMM_basephone *base)
   j_printf("\n");
 }
 
-/* count word-beginning / word-edge mark of basephone */
-int bncnt, edcnt;
+static int bncnt;		///< Count of basephone that can appear at the beginning of sentence
+static int edcnt;		///< Count of basephone that can appear at the end of sentence
+/** 
+ * Traverse callback function to increment the number of base phones
+ * that can appear at beginning of word and end of word.
+ * 
+ * @param x [in] pointer 
+ */
 static void
 count_callback(void *x)
 {
@@ -86,6 +133,13 @@ count_callback(void *x)
   if (b->bgnflag) bncnt++;
   if (b->endflag) edcnt++;
 }
+
+/** 
+ * Count the total number of base phones that can appear at beginning of
+ * word and end of word.
+ * 
+ * @param base 
+ */
 static void
 count_all_phone(HMM_basephone *base)
 {
@@ -95,13 +149,17 @@ count_all_phone(HMM_basephone *base)
   base->endnum = edcnt;
 }
 
-
-/* mark basephone that can be either beginning or end of a word */
+/** 
+ * Mark each basephone if it can appear at beginning or end of a word.
+ * 
+ * @param winfo [in] word dictinary
+ * @param base [in] top basephone data holder
+ */
 static void
 mark_word_edge(WORD_INFO *winfo, HMM_basephone *base)
 {
   WORD_ID w;
-  char p[50];
+  static char p[MAX_HMMNAME_LEN];
   char *key;
   BASEPHONE *match;
 
@@ -131,48 +189,64 @@ mark_word_edge(WORD_INFO *winfo, HMM_basephone *base)
   }
 }
 
-
+
 /* check if all possible triphones are exist in logical HMM */
 /* temporal storage for aptree() callback */
-static HTK_HMM_INFO *local_hmminfo;
-static WORD_INFO *local_winfo;
-static APATNODE *local_root;
-static WORD_ID current_w;
-static char gbuf[50];
+static HTK_HMM_INFO *local_hmminfo; ///< Local work area to hold HTK %HMM data
+static WORD_INFO *local_winfo;	///< Local work area to hold word dictionary
+static APATNODE *local_root;	///< Local work area to hold basephone index root
+static WORD_ID current_w;	///< Local work area to hold current word ID
+static char gbuf[MAX_HMMNAME_LEN];		///< Local work area for phone name handling
 
-/* add unknown (error) triphone to error list */
-static APATNODE *error_root;	/* error phone list */
-static int error_num;		/* number of error phone  */
+static APATNODE *error_root;	///< Error phone list
+static int error_num;		///< Number of encountered error phone
+
+/** 
+ * Add unknown (error) triphone to error list.
+ * 
+ * @param lostname [in] name of error triphone.
+ */
 static void
-add_to_error(char *lostname)
+add_to_error(char *lostname, HTK_HMM_INFO *hmminfo)
 {
-  char *match, *new;
+  char *match = NULL, *new;
   if (error_root != NULL) {
     match = aptree_search_data(lostname, error_root);
     if (strmatch(match, lostname)) return;
   }
-  new = (char *)mybmalloc(strlen(lostname)+1);
+  new = (char *)mybmalloc2(strlen(lostname)+1, &(hmminfo->mroot));
   strcpy(new, lostname);
   if (error_root == NULL) error_root = aptree_make_root_node(new);
   else aptree_add_entry(new, new, match, &error_root);
 
   error_num++;
 }
+
+/** 
+ * Traverse callback function to output error phone name.
+ * 
+ * @param x [in] pointer to error phone name
+ */
 static void
 print_error_callback(void *x)
 {
   char *p = x;
   j_printf("%s\n", p);
 }
-
-/* check existence of triphone for "x - current_w" and "current_w + x" */
+
+/** 
+ * Traverse callback function to check if the cross-word triphones
+ * "basephone x - word[current_w]" and "word[current_w] + basephone x" exist,
+ * according to the basephone mark.
+ * 
+ * @param x [in] a basephone
+ */
 static void
 triphone_callback_normal(void *x)
 {
   BASEPHONE *b = x;
   WORD_ID w = current_w;
   HMM_Logical *lg, *found;
-  BASEPHONE *match;
 
   if (b->endflag) {		/* x can appear as end of word */
     lg = local_winfo->wseq[w][0];
@@ -182,7 +256,7 @@ triphone_callback_normal(void *x)
     if ((found = htk_hmmdata_lookup_logical(local_hmminfo, gbuf)) == NULL) {
       if (lg->is_pseudo) {
 	j_printerr("Error: \"%s\" not found, fallback to pseudo {%s}\n", gbuf, lg->name);
-	add_to_error(gbuf);
+	add_to_error(gbuf, local_hmminfo);
       }
     }
   }
@@ -194,7 +268,7 @@ triphone_callback_normal(void *x)
     if ((found = htk_hmmdata_lookup_logical(local_hmminfo, gbuf)) == NULL) {
       if (lg->is_pseudo) {
 	j_printerr("Error: \"%s\" not found, fallback to pseudo {%s}\n", gbuf, lg->name);
-	add_to_error(gbuf);
+	add_to_error(gbuf, local_hmminfo);
       }
     }
   }
@@ -202,14 +276,20 @@ triphone_callback_normal(void *x)
 
 /* for words with only one phone, all combination of "x - current_w + x"
    should be checked */
+/** 
+ * Traverse callback function to check if the cross-word triphone
+ * "basephone x - word[current_w] + basephone x" exist, for
+ * words with only one phone: right part.
+ * 
+ * @param x [in] a basephone
+ */
 static void
 triphone_callback_right(void *x)
 {
   BASEPHONE *b = x;
   WORD_ID w = current_w;
   HMM_Logical *lg, *found;
-  BASEPHONE *match;
-  char buf[50];
+  static char buf[MAX_HMMNAME_LEN];
 
   if (b->bgnflag) {
     lg = local_winfo->wseq[w][0];
@@ -219,18 +299,25 @@ triphone_callback_right(void *x)
     if ((found = htk_hmmdata_lookup_logical(local_hmminfo, buf)) == NULL) {
       if (lg->is_pseudo) {
 	j_printerr("Error: \"%s\" not found, fallback to pseudo {%s}\n", buf, lg->name);
-	add_to_error(buf);
+	add_to_error(buf, local_hmminfo);
       }
     }
   }
 }
+
+/** 
+ * Traverse callback function to check if the cross-word triphone
+ * "basephone x - word[current_w] + basephone x" exist, for
+ * words with only one phone: left part.
+ * 
+ * @param x [in] a basephone
+ */
 static void
 triphone_callback_left(void *x)
 {
   BASEPHONE *b = x;
   WORD_ID w = current_w;
-  HMM_Logical *lg, *found;
-  BASEPHONE *match;
+  HMM_Logical *lg;
 
   if (b->endflag) {
     lg = local_winfo->wseq[w][0];
@@ -241,8 +328,13 @@ triphone_callback_left(void *x)
   }
 }
 
-/* check if all possible triphones are exist in logical HMM */
-/* possible triphones are generated using base phones */
+/** 
+ * Top function to check if all the possible triphones on given word
+ * dictionary actually exist in the logical %HMM.
+ * 
+ * @param hmminfo [in] %HMM definition information, with basephone list.
+ * @param winfo [in] word dictionary information
+ */
 void
 test_interword_triphone(HTK_HMM_INFO *hmminfo, WORD_INFO *winfo)
 {
@@ -274,17 +366,30 @@ test_interword_triphone(HTK_HMM_INFO *hmminfo, WORD_INFO *winfo)
 }
 
 
-/* build basephone info */
+
+/** 
+ * @brief  Build basephone information
+ *
+ * Extract base phones from %HMM definition, mark them whether they appear
+ * on word head or word tail, and count the number.
+ * 
+ * @param hmminfo [i/o] %HMM definition information, basephone list will be added.
+ * @param winfo [in] word dictionary information
+ */
 void
 make_base_phone(HTK_HMM_INFO *hmminfo, WORD_INFO *winfo)
 {
   /* gather base phones and word-{head,tail} phones */
   j_printf("Exploring HMM database and lexicon tree:\n");
-  make_hmm_basephone_list(hmminfo);
   mark_word_edge(winfo, &(hmminfo->basephone));
   count_all_phone(&(hmminfo->basephone));
 }
-  
+
+/** 
+ * Output general information concerning phone mapping in %HMM definition.
+ * 
+ * @param hmminfo [in] %HMM definition data.
+ */
 void
 print_phone_info(HTK_HMM_INFO *hmminfo)
 {
