@@ -11,20 +11,19 @@
  * @brief  Load %HMM definition file and HMMList mapping file into memory and set it up.
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.4 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
 #include <sent/stddefs.h>
 #include <sent/htk_hmm.h>
 
-/* initialize phoneme HMM structure */
 /** 
  * Allocate memory for a new %HMM definition data.
  * 
@@ -37,9 +36,77 @@ hmminfo_new()
 
   new = (HTK_HMM_INFO *)mymalloc(sizeof(HTK_HMM_INFO));
 
+  new->mroot = NULL;
+  new->lroot = NULL;
+
+  new->opt.stream_info.num = 1;
+  new->opt.cov_type = C_DIAG_C;
+  new->opt.dur_type = D_NULL;
+
+  new->trstart = NULL;
+  new->vrstart = NULL;
+  new->ststart = NULL;
+  new->dnstart = NULL;
+  new->start   = NULL;
+  new->lgstart = NULL;
+  new->physical_root = NULL;
+  new->logical_root = NULL;
+  new->tr_root = NULL;
+  new->vr_root = NULL;
+  new->dn_root = NULL;
+  new->st_root = NULL;
+  new->codebooknum = 0;
+  new->codebook_root = NULL;
+  new->maxcodebooksize = 0;
+  new->totalmixnum = 0;
+  new->totalstatenum = 0;
+  new->totalhmmnum = 0;
+  new->totallogicalnum = 0;
+  new->is_triphone = FALSE;
+  new->is_tied_mixture = FALSE;
+  new->cdset_method = IWCD_NBEST;
+  new->cdmax_num = 3;
+  new->totalpseudonum = 0;
+  new->sp = NULL;
+  new->basephone.root = NULL;
+  new->cdset_info.cdtree = NULL;
+
   return(new);
 }
 
+/** 
+ * Free memory of an %HMM
+ * 
+ * @param hmm [i/o] %HMM definition data
+ * 
+ * @return TRUE on success, or FALSE if failed.
+ */
+boolean
+hmminfo_free(HTK_HMM_INFO *hmm)
+{
+  /* cdset does not use bmalloc, so free them separately */
+  free_cdset(&(hmm->cdset_info.cdtree));
+
+  /* free lookup indexes */
+  free_aptree(hmm->tr_root);
+  free_aptree(hmm->vr_root);
+  free_aptree(hmm->dn_root);
+  free_aptree(hmm->st_root);
+  free_aptree(hmm->physical_root);
+  free_aptree(hmm->logical_root);
+  free_aptree(hmm->codebook_root);
+  free_aptree(hmm->basephone.root);
+  
+  /* free all memory that has been allocated by bmalloc2() */
+  if (hmm->mroot != NULL) mybfree2(&(hmm->mroot));
+  if (hmm->lroot != NULL) mybfree2(&(hmm->lroot));
+
+  /* free whole */
+  free(hmm);
+
+  return(TRUE);
+}
+
 /** 
  * @brief Load HTK %HMM definition file and HMMList file, and setup phone %HMM information.
  *
@@ -74,7 +141,7 @@ init_hmminfo(HTK_HMM_INFO *hmminfo, char *hmmfilename, char *namemapfile)
     if ((fp = fopen_readfile(hmmfilename)) == NULL) {
       j_error("failed to open %s\n",hmmfilename);
     }
-    if (read_binhmm(fp, hmminfo) == TRUE) {
+    if (read_binhmm(fp, hmminfo, TRUE) == TRUE) {
       ok_p = TRUE;
     }
     if (fclose_readfile(fp) < 0) {
@@ -118,8 +185,6 @@ init_hmminfo(HTK_HMM_INFO *hmminfo, char *hmmfilename, char *namemapfile)
     hmminfo->is_triphone = FALSE;
   }
 
-  hmminfo->sp = NULL;
-
   j_printerr("done\n");
 }
 
