@@ -1,22 +1,39 @@
+/**
+ * @file   put_htkdata_info.c
+ * @author Akinobu LEE
+ * @date   Tue Feb 15 23:36:00 2005
+ * 
+ * <JA>
+ * @brief  %HMM ħѥ᡼ξƥȽϤ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Output %HMM and parameter information to standard out
+ * </EN>
+ * 
+ * $Revision: 1.2 $
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* put_htkdata_data.c --- output hmm/param info to stdout (for debug) */
-
-/* $Id: put_htkdata_info.c,v 1.4 2003/09/29 06:01:23 ri Exp $ */
-
 #include <sent/stddefs.h>
 #include <sent/htk_hmm.h>
 #include <sent/htk_param.h>
 #include <sent/hmm.h>
 
-static char buf[512];
+static char buf[512];		///< Local work area for text handling
 
 
-/* HTK HMM INFO */
+/** 
+ * Output transition matrix.
+ * 
+ * @param t [in] pointer to a transion matrix
+ */
 void
 put_htk_trans(HTK_HMM_Trans *t)
 {
@@ -34,6 +51,11 @@ put_htk_trans(HTK_HMM_Trans *t)
   }
 }
 
+/** 
+ * Output variance vector (diagonal).
+ * 
+ * @param v [in] pointer to a variance data
+ */
 void
 put_htk_var(HTK_HMM_Var *v)
 {
@@ -50,6 +72,11 @@ put_htk_var(HTK_HMM_Var *v)
   }
 }
 
+/** 
+ * Output a density information, mean and variance.
+ * 
+ * @param d [in] pointer to a density data
+ */
 void
 put_htk_dens(HTK_HMM_Dens *d)
 {
@@ -68,6 +95,11 @@ put_htk_dens(HTK_HMM_Dens *d)
   }
 }
 
+/** 
+ * Output mixture component of a state.
+ * 
+ * @param s [in] pointer to %HMM state
+ */
 void
 put_htk_state(HTK_HMM_State *s)
 {
@@ -84,6 +116,11 @@ put_htk_state(HTK_HMM_State *s)
   }
 }
 
+/** 
+ * Output %HMM model, number of states and information for each state.
+ * 
+ * @param h [in] pointer to %HMM model
+ */
 void
 put_htk_hmm(HTK_HMM_Data *h)
 {
@@ -98,20 +135,27 @@ put_htk_hmm(HTK_HMM_Data *h)
   put_htk_trans(h->tr);
 }
 
+/** 
+ * Output logical %HMM data and its mapping status.
+ * 
+ * @param logical [in] pointer to a logical %HMM
+ */
 void
-put_logical_hmm(HMM_Logical *l)
+put_logical_hmm(HMM_Logical *logical)
 {
-  int i;
-  
-  j_printf("name: %s\n", l->name);
-  if (l->is_pseudo) {
-    j_printf("mapped to: %s (pseudo)\n", l->body.pseudo->name);
+  j_printf("name: %s\n", logical->name);
+  if (logical->is_pseudo) {
+    j_printf("mapped to: %s (pseudo)\n", logical->body.pseudo->name);
   } else {
-    j_printf("mapped to: %s\n", l->body.defined->name);
+    j_printf("mapped to: %s\n", logical->body.defined->name);
   }
 }
 
-/* HMM (internal) */
+/** 
+ * Output transition arcs of an HMM instance.
+ * 
+ * @param d [in] pointer to a HMM instance.
+ */
 void
 put_hmm_arc(HMM *d)
 {
@@ -125,9 +169,16 @@ put_hmm_arc(HMM *d)
       j_printf(" arc: %d %f (%f)\n",ac->arc, ac->a, pow(10.0, ac->a));
     }
   }
+#ifndef MULTIPATH_VERSION
   j_printf("last arc to accept state: %f\n", d->accept_ac_a);
+#endif
 }
 
+/** 
+ * Output output probability information of an HMM instance.
+ * 
+ * @param d [in] pointer to a HMM instance.
+ */
 void
 put_hmm_outprob(HMM *d)
 {
@@ -144,6 +195,11 @@ put_hmm_outprob(HMM *d)
   }
 }
 
+/** 
+ * Output an HMM instance.
+ * 
+ * @param d [in] pointer to a HMM instance.
+ */
 void
 put_hmm(HMM *d)
 {
@@ -151,7 +207,11 @@ put_hmm(HMM *d)
   put_hmm_outprob(d);
 }
 
-
+/** 
+ * Output parameter header.
+ * 
+ * @param h [in] pointer to a parameter header information
+ */
 void
 put_param_head(HTK_Param_Header *h)
 {
@@ -161,6 +221,13 @@ put_param_head(HTK_Param_Header *h)
   j_printf("parameter type: %s\n", param_code2str(buf, h->samptype, FALSE));
 }
 
+/** 
+ * Output array of vectors.
+ * 
+ * @param p [in] pointer to vector array represented as [0..num-1][0...veclen-1]
+ * @param num [in] number of vectors in @a p
+ * @param veclen [in] length of each vector
+ */
 void
 put_vec(VECT **p, int num, short veclen)
 {
@@ -176,6 +243,11 @@ put_vec(VECT **p, int num, short veclen)
   }
 }
 
+/** 
+ * Output the whole parameter information, including header and all vectors.
+ * 
+ * @param pinfo [in] pointer to parameter structure.
+ */
 void
 put_param(HTK_Param *pinfo)
 {
@@ -183,6 +255,11 @@ put_param(HTK_Param *pinfo)
   put_vec(pinfo->parvec, pinfo->samplenum, pinfo->veclen);
 }
 
+/** 
+ * Output the length of an input parameter by number of frames and seconds.
+ * 
+ * @param pinfo [in] pointer to parameter structure.
+ */
 void
 put_param_info(HTK_Param *pinfo)
 {
@@ -194,6 +271,13 @@ put_param_info(HTK_Param *pinfo)
   j_printf("length: %d frames (%.2f sec.)\n", h->samplenum, sec);
 }
 
+/** 
+ * Get the maximum number of mixtures per state in a %HMM definition.
+ * 
+ * @param hmminfo [in] %HMM definition data.
+ * 
+ * @return the maximum number of mixtures per state.
+ */
 static int
 get_max_mixture_num(HTK_HMM_INFO *hmminfo)
 {
@@ -207,6 +291,11 @@ get_max_mixture_num(HTK_HMM_INFO *hmminfo)
   return(maxmixnum);
 }
 
+/** 
+ * Output total statistic informations of the %HMM definition data.
+ * 
+ * @param hmminfo [in] %HMM definition data.
+ */
 void
 print_hmmdef_info(HTK_HMM_INFO *hmminfo)
 {
@@ -226,4 +315,23 @@ print_hmmdef_info(HTK_HMM_INFO *hmminfo)
   j_printf("\tcov. matrix type = %s\n", get_cov_str(hmminfo->opt.cov_type));
   j_printf("\t   duration type = %s\n", get_dur_str(hmminfo->opt.dur_type));
   j_printf("\t     mixture num = %d\n", get_max_mixture_num(hmminfo));
+  j_printf("\t   max state num = %d\n", hmminfo->maxstatenum);
+#ifdef MULTIPATH_VERSION
+  j_printf("      skippable models =");
+  {
+    HTK_HMM_Data *dtmp;
+    int n = 0;
+    for (dtmp = hmminfo->start; dtmp; dtmp = dtmp->next) {
+      if (is_skippable_model(dtmp)) {
+	j_printf(" %s", dtmp->name);
+ 	n++;
+      }
+    }
+    if (n == 0) {
+      j_printf(" none\n");
+    } else {
+      j_printf(" (%d model(s))\n", n);
+    }
+  }
+#endif
 }
