@@ -1,32 +1,61 @@
+/**
+ * @file   rdhmmdef.c
+ * @author Akinobu LEE
+ * @date   Wed Feb 16 00:17:18 2005
+ * 
+ * <JA>
+ * @brief  HTK %HMM եɤ߹ߡᥤ
+ *
+ * ˤ HTK  %HMM եɤ߹िδؿƤӽФ
+ * ᥤؿƤޤ
+ *
+ * ΥեϤޤɤ߹ߴؿǶ̤Ѥȡñ̤
+ * եɤ߹ߴؿ󶡤ޤ
+ * %HMM ե read_token() ˤäƥȡñ̤ǽ缡ɤ߹ޤ졤
+ * Хѿ rdhmmdef_token ˳ǼޤƴؿϤ
+ * rdhmmdef_token 򻲾ȤƸߤΥȡޤ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Read HTK %HMM definition file: the main
+ *
+ * This file includes the main routine to read the %HMM definition file in
+ * HTK format.
+ *
+ * This file also contains functions and global variables for per-token
+ * reading tailored for reading HTK %HMM definition file.  The read_token()
+ * will read the file per token, and the read token is stored in a global
+ * variable rdhmmdef_token.  The other reading function will refer to this
+ * variable to read the current token.
+ * </EN>
+ * 
+ * $Revision: 1.3 $
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* rdhmmdef.c --- read in HTK format HMM definition file (main) */
-
-/* $Id: rdhmmdef.c,v 1.7 2003/12/22 09:37:40 ri Exp $ */
-
 #include <sent/stddefs.h>
 #include <sent/htk_param.h>
 #include <sent/htk_hmm.h>
 
-#define MAXBUFLEN  4096
-#ifdef DELM
-#undef DELM
-#endif
-#define DELM " \t\n<>"
+#define MAXBUFLEN  4096		///< Maximum length of a line in the input
+
+char *rdhmmdef_token;		///< Current token string (GLOBAL)
+static char *buf = NULL;	///< Local work area for token reading
+static int line;		///< Input Line count
 
-/* parsing strategy: token look-ahead */
-char *rdhmmdef_token;		/* current token */
-				/* referred by other rdhmmdef_*.c */
-
 /* global functions for rdhmmdef_*.c */
-static char *buf = NULL;
-static int line;
 
-/* error exit */
+/** 
+ * Output error message, with current reading status.
+ * 
+ * @param str [in] error string
+ */
 void
 rderr(char *str)
 {
@@ -37,17 +66,19 @@ rderr(char *str)
   }
 }
 
-/* read next token and set it to rdhmmdef_token */
-/* return NULL on end of file */
+/** 
+ * Read next token and ste it to rdhmmdef_token.
+ * 
+ * @param fp [in] file pointer
+ * 
+ * @return the pointer to the read token, or NULL on end of file or error.
+ */
 char *
 read_token(FILE *fp)
 {
-  /* the quotation handler is moved to
-     mystrtok_quote() in util/mystrtok.c */
-  
   if (buf != NULL) {
     /* already have buffer */
-    if ((rdhmmdef_token = mystrtok_quote(NULL, DELM)) != NULL) {
+    if ((rdhmmdef_token = mystrtok_quote(NULL, HMMDEF_DELM)) != NULL) {
       /* return next token */
       return rdhmmdef_token;
     }
@@ -60,14 +91,17 @@ read_token(FILE *fp)
   if (getl(buf, MAXBUFLEN, fp) == NULL) {
     rdhmmdef_token = NULL;
   } else {
-    rdhmmdef_token = mystrtok_quote(buf, DELM);
+    rdhmmdef_token = mystrtok_quote(buf, HMMDEF_DELM);
     line++;
   }
   return rdhmmdef_token;
 }
 
-
-/* lognize transition prob */
+/** 
+ * Convert all the transition probabilities to log10 scale.
+ * 
+ * @param hmm [i/o] %HMM definition data to modify.
+ */
 static void
 conv_log_arc(HTK_HMM_INFO *hmm)
 {
@@ -79,58 +113,29 @@ conv_log_arc(HTK_HMM_INFO *hmm)
     for(i=0;i<tr->statenum;i++) {
       for(j=0;j<tr->statenum;j++) {
 	l = tr->a[i][j];
-	tr->a[i][j] = (l != 0.0) ? log10(l) : LOG_ZERO;
+	tr->a[i][j] = (l != 0.0) ? (float)log10(l) : LOG_ZERO;
       }
     }
   }
 }
 
-
-/* init data structure */
-void
-init_hmm(HTK_HMM_INFO *hmm)
-{
-  /* set default options */
-  hmm->opt.stream_info.num = 1;
-  hmm->opt.cov_type = C_DIAG_C;
-  hmm->opt.dur_type = D_NULL;
-
-  hmm->trstart = NULL;
-  hmm->vrstart = NULL;
-  hmm->ststart = NULL;
-  hmm->dnstart = NULL;
-  hmm->start   = NULL;
-  hmm->lgstart = NULL;
-  hmm->physical_root = NULL;
-  hmm->logical_root = NULL;
-  hmm->tr_root = NULL;
-  hmm->vr_root = NULL;
-  hmm->dn_root = NULL;
-  hmm->st_root = NULL;
-  hmm->codebooknum = 0;
-  hmm->codebook_root = NULL;
-  hmm->maxcodebooksize = 0;
-  hmm->totalmixnum = 0;
-  hmm->totalstatenum = 0;
-  hmm->totalhmmnum = 0;
-  hmm->totallogicalnum = 0;
-  hmm->is_triphone = FALSE;
-  hmm->is_tied_mixture = FALSE;
-  hmm->cdset_method = IWCD_NBEST;
-  hmm->cdmax_num = 3;
-  hmm->totalpseudonum = 0;
-}
-
-/* main routine */
+/** 
+ * @brief  Main top routine to read in HTK %HMM definition file.
+ *
+ * A HTK %HMM definition file will be read from @a fp.  After reading,
+ * the parameter type is checked and calculate some statistics.
+ * 
+ * @param fp [in] file pointer
+ * @param hmm [out] pointer to a %HMM definition structure to store data.
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 rdhmmdef(FILE *fp, HTK_HMM_INFO *hmm)
 {
   char macrosw;
   char *name;
 
-  /* initialize hmm data */
-  init_hmm(hmm);
-  
   /* read the first token */
   read_token(fp);
   
@@ -146,32 +151,38 @@ rdhmmdef(FILE *fp, HTK_HMM_INFO *hmm)
       set_global_opt(fp,hmm);
       break;
     case 't':			/* transition macro */
-      name = mybstrdup(rdhmmdef_token);
+      name = mybstrdup2(rdhmmdef_token, &(hmm->mroot));
+      if (strlen(name) >= MAX_HMMNAME_LEN) rderr("Macro name too long");
       read_token(fp);
       def_trans_macro(name, fp, hmm);
       break;
     case 's':			/* state macro */
-      name = mybstrdup(rdhmmdef_token);
+      name = mybstrdup2(rdhmmdef_token, &(hmm->mroot));
+      if (strlen(name) >= MAX_HMMNAME_LEN) rderr("Macro name too long");
       read_token(fp);
       def_state_macro(name, fp, hmm);
       break;
     case 'm':			/* density (mixture) macro */
-      name = mybstrdup(rdhmmdef_token);
+      name = mybstrdup2(rdhmmdef_token, &(hmm->mroot));
+      if (strlen(name) >= MAX_HMMNAME_LEN) rderr("Macro name too long");
       read_token(fp);
       def_dens_macro(name, fp, hmm);
       break;
     case 'h':			/* HMM define */
-      name = mybstrdup(rdhmmdef_token);
+      name = mybstrdup2(rdhmmdef_token, &(hmm->mroot));
+      if (strlen(name) >= MAX_HMMNAME_LEN) rderr("Macro name too long");
       read_token(fp);
       def_HMM(name, fp, hmm);
       break;
     case 'v':			/* Variance macro */
-      name = mybstrdup(rdhmmdef_token);
+      name = mybstrdup2(rdhmmdef_token, &(hmm->mroot));
+      if (strlen(name) >= MAX_HMMNAME_LEN) rderr("Macro name too long");
       read_token(fp);
       def_var_macro(name, fp, hmm);
       break;
     case 'r':			/* Regression class macro (ignore) */
-      name = mybstrdup(rdhmmdef_token);
+      name = mybstrdup2(rdhmmdef_token, &(hmm->mroot));
+      if (strlen(name) >= MAX_HMMNAME_LEN) rderr("Macro name too long");
       read_token(fp);
       def_regtree_macro(name, fp, hmm);
       break;
@@ -211,12 +222,17 @@ rdhmmdef(FILE *fp, HTK_HMM_INFO *hmm)
     hmm->totalstatenum = n;
     hmm->maxmixturenum = max;
   }
-  /* compute total number of HMM models */
+  /* compute total number of HMM models and maximum length */
   {
     HTK_HMM_Data *dtmp;
-    int n;
+    int n, maxlen;
     n = 0;
-    for (dtmp = hmm->start; dtmp; dtmp = dtmp->next) n++;
+    maxlen = 0;
+    for (dtmp = hmm->start; dtmp; dtmp = dtmp->next) {
+      if (maxlen < dtmp->state_num) maxlen = dtmp->state_num;
+      n++;
+    }
+    hmm->maxstatenum = maxlen;
     hmm->totalhmmnum = n;
   }
   /* compute total number of mixtures */
@@ -228,6 +244,15 @@ rdhmmdef(FILE *fp, HTK_HMM_INFO *hmm)
     }
     hmm->totalmixnum = n;
   }
+  /* check of HMM name length exceed the maximum */
+  {
+    HTK_HMM_Dens *dtmp;
+    int n = 0;
+    for (dtmp = hmm->dnstart; dtmp; dtmp = dtmp->next) {
+      n++;
+    }
+    hmm->totalmixnum = n;
+  }
 
   return(TRUE);			/* success */
 }
