@@ -11,13 +11,13 @@
  * @brief  Read HTK %HMM definition file: %HMM model
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -32,11 +32,11 @@ extern char *rdhmmdef_token;	///< Current token
  * @return pointer to newly allocated data.
  */
 HTK_HMM_Data *
-htk_hmmdata_new()
+htk_hmmdata_new(HTK_HMM_INFO *hmminfo)
 {
   HTK_HMM_Data *new;
 
-  new = (HTK_HMM_Data *)mybmalloc(sizeof(HTK_HMM_Data));
+  new = (HTK_HMM_Data *)mybmalloc2(sizeof(HTK_HMM_Data), &(hmminfo->mroot));
 
   new->name = NULL;
   new->state_num = 0;
@@ -102,7 +102,7 @@ htk_hmmdata_read(FILE *fp, HTK_HMM_INFO *hmm)
   int i;
   short sid;
 
-  new = htk_hmmdata_new();
+  new = htk_hmmdata_new(hmm);
 
   /* begin tag */
   if (!currentis("BEGINHMM")) rderr("<BEGINHMM> not found");
@@ -119,7 +119,7 @@ htk_hmmdata_read(FILE *fp, HTK_HMM_INFO *hmm)
   read_token(fp);
 
   /* malloc state */
-  new->s = (HTK_HMM_State **)mybmalloc(sizeof(HTK_HMM_State *) * new->state_num);
+  new->s = (HTK_HMM_State **)mybmalloc2(sizeof(HTK_HMM_State *) * new->state_num, &(hmm->mroot));
   for(i=0;i<new->state_num;i++) {
     new->s[i] = NULL;
   }
