@@ -1,25 +1,42 @@
+/**
+ * @file   rdhmmdef_state.c
+ * @author Akinobu LEE
+ * @date   Wed Feb 16 03:07:44 2005
+ * 
+ * <JA>
+ * @brief  HTK %HMM եɤ߹ߡ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Read HTK %HMM definition file: state
+ * </EN>
+ * 
+ * $Revision: 1.3 $
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* rdhmmdef_state.c --- read in hmmdefs (state) */
-
-/* $Id: rdhmmdef_state.c,v 1.7 2003/12/08 03:31:17 ri Exp $ */
-
 #include <sent/stddefs.h>
 #include <sent/htk_hmm.h>
 
-extern char *rdhmmdef_token;	/* defined in rdhmmdef.c */
+extern char *rdhmmdef_token;	///< Current token
 
-/* malloc new */
+/** 
+ * Allocate a new data area and return it.
+ * 
+ * @return pointer to newly allocated data.
+ */
 static HTK_HMM_State *
-state_new()
+state_new(HTK_HMM_INFO *hmm)
 {
   HTK_HMM_State *new;
 
-  new = (HTK_HMM_State *)mybmalloc(sizeof(HTK_HMM_State));
+  new = (HTK_HMM_State *)mybmalloc2(sizeof(HTK_HMM_State), &(hmm->mroot));
 
   new->name = NULL;
   new->mix_num = 0;
@@ -31,7 +48,12 @@ state_new()
   return(new);
 }
 
-/* register */
+/** 
+ * Add a new data to the global structure.
+ * 
+ * @param hmm [i/o] %HMM definition data to store it
+ * @param new [in] new data to be added
+ */
 void
 state_add(HTK_HMM_INFO *hmm, HTK_HMM_State *new)
 {
@@ -58,7 +80,14 @@ state_add(HTK_HMM_INFO *hmm, HTK_HMM_State *new)
   
 }
 
-/* lookup by name */
+/** 
+ * Look up a data macro by the name.
+ * 
+ * @param hmm [in] %HMM definition data
+ * @param keyname [in] macro name to find
+ * 
+ * @return pointer to the found data, or NULL if not found.
+ */
 HTK_HMM_State *
 state_lookup(HTK_HMM_INFO *hmm, char *keyname)
 {
@@ -72,7 +101,20 @@ state_lookup(HTK_HMM_INFO *hmm, char *keyname)
   }
 }
 
-/* read in state data and return new (malloced) HTK_HMM_State */
+/** 
+ * @brief  Read one new data and returns the pointer
+ *
+ * If a sub-component of this data is directly defined at here, they
+ * will be read from here and assigned to this data.  If a sub-component
+ * is not defined here but a macro name referencing to the component previously
+ * defined in other place, the data will be searched by the macro name and
+ * the pointer to the found component will be assigned to this model.
+ * 
+ * @param fp [in] file pointer
+ * @param hmm [in] %HMM definition data
+ * 
+ * @return pointer to the newly read data.
+ */
 static HTK_HMM_State *
 state_read(FILE *fp, HTK_HMM_INFO *hmm)
 {
@@ -81,7 +123,7 @@ state_read(FILE *fp, HTK_HMM_INFO *hmm)
   int i;
   boolean single_gaussian;
 
-  new = state_new();
+  new = state_new(hmm);
 
   if (currentis("NUMMIXES")) {
     read_token(fp);
@@ -100,8 +142,8 @@ state_read(FILE *fp, HTK_HMM_INFO *hmm)
     tmix_read(fp, new, hmm);
   } else {
     
-    new->b = (HTK_HMM_Dens **) mybmalloc(sizeof(HTK_HMM_Dens *) * new->mix_num);
-    new->bweight = (PROB *) mybmalloc(sizeof(PROB) * new->mix_num);
+    new->b = (HTK_HMM_Dens **) mybmalloc2(sizeof(HTK_HMM_Dens *) * new->mix_num, &(hmm->mroot));
+    new->bweight = (PROB *) mybmalloc2(sizeof(PROB) * new->mix_num, &(hmm->mroot));
     for (i=0;i<new->mix_num;i++) {
       new->b[i] = NULL;
       new->bweight[i] = LOG_ZERO;
@@ -119,7 +161,7 @@ state_read(FILE *fp, HTK_HMM_INFO *hmm)
 	mid = atoi(rdhmmdef_token) - 1;
 	read_token(fp);
 	NoTokErr("missing MIXTURE weight");
-	new->bweight[mid] = log(atof(rdhmmdef_token));
+	new->bweight[mid] = (PROB)log(atof(rdhmmdef_token));
 	read_token(fp);
 	new->b[mid] = get_dens_data(fp, hmm);
       }
@@ -141,8 +183,19 @@ state_read(FILE *fp, HTK_HMM_INFO *hmm)
   return (new);
 }
 
-/* read in state data at the current point, and
-   return pointer to the data */
+/** 
+ * @brief  Return a pointer to the data located at the current point.
+ *
+ * If the current point is a macro reference, the pointer to the
+ * already defined data will be searched and returned.
+ * Otherwise, the definition of the data will be read from the current
+ * point and pointer to the newly allocated data will be returned.
+ * 
+ * @param fp [in] file pointer
+ * @param hmm [i/o] %HMM definition data
+ * 
+ * @return pointer to the data located at the current point.
+ */
 HTK_HMM_State *
 get_state_data(FILE *fp, HTK_HMM_INFO *hmm)
 {
@@ -173,7 +226,13 @@ get_state_data(FILE *fp, HTK_HMM_INFO *hmm)
 
 
 
-/* define state data */
+/** 
+ * Read a new data and store it as a macro.
+ * 
+ * @param name [in] macro name
+ * @param fp [in] file pointer
+ * @param hmm [i/o] %HMM definition data
+ */
 void
 def_state_macro(char *name, FILE *fp, HTK_HMM_INFO *hmm)
 {
