@@ -1,27 +1,47 @@
+/**
+ * @file   rdhmmdef_trans.c
+ * @author Akinobu LEE
+ * @date   Wed Feb 16 03:50:55 2005
+ * 
+ * <JA>
+ * @brief  HTK %HMM եɤ߹ߡܹ
+ *
+ * ܳΨϥեɤ߹߽λ log10 Ѵޤ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Read HTK %HMM definition file: transition matrix
+ *
+ * The transition probabilities will be converted to log10 scale
+ * after finished reading the while %HMM definition file.
+ * </EN>
+ * 
+ * $Revision: 1.3 $
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* rdhmmdef_trans.c --- read in hmmdefs (transition) */
-
-/* $Id: rdhmmdef_trans.c,v 1.5 2003/12/08 03:31:17 ri Exp $ */
-
-/* read data is linear scale (will be converted later) */
-
 #include <sent/stddefs.h>
 #include <sent/htk_hmm.h>
 
-extern char *rdhmmdef_token;	/* defined in rdhmmdef.c */
+extern char *rdhmmdef_token;	///< Current token
 
-/* malloc new */
+/** 
+ * Allocate a new data area and return it.
+ * 
+ * @return pointer to newly allocated data.
+ */
 static HTK_HMM_Trans *
-trans_new()
+trans_new(HTK_HMM_INFO *hmm)
 {
   HTK_HMM_Trans *new;
 
-  new = (HTK_HMM_Trans *)mybmalloc(sizeof(HTK_HMM_Trans));
+  new = (HTK_HMM_Trans *)mybmalloc2(sizeof(HTK_HMM_Trans), &(hmm->mroot));
   new->name = (char *)NULL;
   new->statenum = 0;
   new->a = (PROB **)NULL;
@@ -30,7 +50,12 @@ trans_new()
   return(new);
 }
 
-/* register */
+/** 
+ * Add a new data to the global structure.
+ * 
+ * @param hmm [i/o] %HMM definition data to store it
+ * @param new [in] new data to be added
+ */
 void
 trans_add(HTK_HMM_INFO *hmm, HTK_HMM_Trans *new)
 {
@@ -56,7 +81,14 @@ trans_add(HTK_HMM_INFO *hmm, HTK_HMM_Trans *new)
   }
 }  
 
-/* lookup by name */
+/** 
+ * Look up a data macro by the name.
+ * 
+ * @param hmm [in] %HMM definition data
+ * @param keyname [in] macro name to find
+ * 
+ * @return pointer to the found data, or NULL if not found.
+ */
 static HTK_HMM_Trans *
 trans_lookup(HTK_HMM_INFO *hmm, char *keyname)
 {
@@ -70,9 +102,21 @@ trans_lookup(HTK_HMM_INFO *hmm, char *keyname)
   }
 }
 
-/* read in transition data and return new (malloced) HTK_HMM_Trans */
+/** 
+ * @brief  Read one new data and returns the pointer
+ *
+ * If a sub-component of this data is directly defined at here, they
+ * will be read from here and assigned to this data.  If a sub-component
+ * is not defined here but a macro name referencing to the component previously
+ * defined in other place, the data will be searched by the macro name and
+ * the pointer to the found component will be assigned to this model.
+ * 
+ * @param fp [in] file pointer
+ * 
+ * @return pointer to the newly read data.
+ */
 static HTK_HMM_Trans *
-trans_read(FILE *fp)
+trans_read(FILE *fp, HTK_HMM_INFO *hmm)
 {
   HTK_HMM_Trans *new;
   int i,j;
@@ -84,14 +128,14 @@ trans_read(FILE *fp)
   read_token(fp);
 
   /* read statenum */
-  new = trans_new();
+  new = trans_new(hmm);
   NoTokErr("missing TRANSP state num");
   new->statenum = atoi(rdhmmdef_token);
   read_token(fp);
 
   /* allocate array */
-  new->a = (PROB **)mybmalloc(sizeof(PROB *) * new->statenum);
-  atmp = (PROB *)mybmalloc(sizeof(PROB) * new->statenum * new->statenum);
+  new->a = (PROB **)mybmalloc2(sizeof(PROB *) * new->statenum, &(hmm->mroot));
+  atmp = (PROB *)mybmalloc2(sizeof(PROB) * new->statenum * new->statenum, &(hmm->mroot));
   new->a[0] = &(atmp[0]);
   for (i=1;i<new->statenum;i++) {
     new->a[i] = &(atmp[i*new->statenum]);
@@ -101,7 +145,7 @@ trans_read(FILE *fp)
   for (i=0;i<new->statenum; i++) {
     for (j=0;j<new->statenum; j++) {
       NoTokErr("missing some TRANSP value");
-      prob = atof(rdhmmdef_token);
+      prob = (PROB)atof(rdhmmdef_token);
       new->a[i][j] = prob;
       read_token(fp);
     }
@@ -110,8 +154,19 @@ trans_read(FILE *fp)
   return(new);
 }
 
-/* read in transition data at the current point, and
-   return pointer to the data */
+/** 
+ * @brief  Return a pointer to the data located at the current point.
+ *
+ * If the current point is a macro reference, the pointer to the
+ * already defined data will be searched and returned.
+ * Otherwise, the definition of the data will be read from the current
+ * point and pointer to the newly allocated data will be returned.
+ * 
+ * @param fp [in] file pointer
+ * @param hmm [i/o] %HMM definition data
+ * 
+ * @return pointer to the data located at the current point.
+ */
 HTK_HMM_Trans *
 get_trans_data(FILE *fp, HTK_HMM_INFO *hmm)
 {
@@ -119,7 +174,7 @@ get_trans_data(FILE *fp, HTK_HMM_INFO *hmm)
   
   if (currentis("TRANSP")) {
     /* definition: define transition data, and return the pointer */
-    tmp = trans_read(fp);
+    tmp = trans_read(fp, hmm);
     tmp->name = NULL; /* no name */
     trans_add(hmm, tmp);
     return(tmp);
@@ -140,14 +195,20 @@ get_trans_data(FILE *fp, HTK_HMM_INFO *hmm)
   }
 }
 
-/* define transition data */
+/** 
+ * Read a new data and store it as a macro.
+ * 
+ * @param name [in] macro name
+ * @param fp [in] file pointer
+ * @param hmm [i/o] %HMM definition data
+ */
 void
 def_trans_macro(char *name, FILE *fp, HTK_HMM_INFO *hmm)
 {
   HTK_HMM_Trans *new;
 
   /* read in data and return newly malloced data */
-  new = trans_read(fp);
+  new = trans_read(fp, hmm);
 
   /* register it to the grobal HMM structure */
   new->name = name;
