@@ -1,27 +1,44 @@
+/**
+ * @file   rdhmmdef_var.c
+ * @author Akinobu LEE
+ * @date   Wed Feb 16 04:01:38 2005
+ * 
+ * <JA>
+ * @brief  HTK %HMM եɤ߹ߡʬۤʬ٥ȥ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Read HTK %HMM definition file: variance vector of Gaussian
+ * </EN>
+ * 
+ * $Revision: 1.3 $
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* rdhmmdef_var.c --- read in hmmdefs (variance) */
-
-/* $Id: rdhmmdef_var.c,v 1.5 2003/12/08 03:31:17 ri Exp $ */
-
 /* currenty cannot treat other sub macros (~u,~i,~x) */
 
 #include <sent/stddefs.h>
 #include <sent/htk_hmm.h>
 
-extern char *rdhmmdef_token;
+extern char *rdhmmdef_token;	///< Current token
 
-/* malloc new */
+/** 
+ * Allocate a new data area and return it.
+ * 
+ * @return pointer to newly allocated data.
+ */
 static HTK_HMM_Var *
-var_new()
+var_new(HTK_HMM_INFO *hmm)
 {
   HTK_HMM_Var *new;
 
-  new = (HTK_HMM_Var *)mymalloc(sizeof(HTK_HMM_Var));
+  new = (HTK_HMM_Var *)mybmalloc2(sizeof(HTK_HMM_Var), &(hmm->mroot));
 
   new->name = NULL;
   new->vec = NULL;
@@ -31,7 +48,12 @@ var_new()
   return(new);
 }
 
-/* register */
+/** 
+ * Add a new data to the global structure.
+ * 
+ * @param hmm [i/o] %HMM definition data to store it
+ * @param new [in] new data to be added
+ */
 void
 var_add(HTK_HMM_INFO *hmm, HTK_HMM_Var *new)
 {
@@ -58,7 +80,14 @@ var_add(HTK_HMM_INFO *hmm, HTK_HMM_Var *new)
   
 }
 
-/* lookup by name */
+/** 
+ * Look up a data macro by the name.
+ * 
+ * @param hmm [in] %HMM definition data
+ * @param keyname [in] macro name to find
+ * 
+ * @return pointer to the found data, or NULL if not found.
+ */
 static HTK_HMM_Var *
 var_lookup(HTK_HMM_INFO *hmm, char *keyname)
 {
@@ -72,15 +101,26 @@ var_lookup(HTK_HMM_INFO *hmm, char *keyname)
   }
 }
 
-
-/* read in var data and return new (malloced) HTK_HMM_Var */
+/** 
+ * @brief  Read one new data and returns the pointer
+ *
+ * If a sub-component of this data is directly defined at here, they
+ * will be read from here and assigned to this data.  If a sub-component
+ * is not defined here but a macro name referencing to the component previously
+ * defined in other place, the data will be searched by the macro name and
+ * the pointer to the found component will be assigned to this model.
+ * 
+ * @param fp [in] file pointer
+ * 
+ * @return pointer to the newly read data.
+ */
 static HTK_HMM_Var *
-var_read(FILE *fp)
+var_read(FILE *fp, HTK_HMM_INFO *hmm)
 {
   HTK_HMM_Var *new;
   int i;
 
-  new = var_new();
+  new = var_new(hmm);
 
   /* read covariance matrix (diagonal vector) */
 
@@ -92,11 +132,11 @@ var_read(FILE *fp)
     NoTokErr("missing VARIANCE vector length");
     new->len = atoi(rdhmmdef_token);
     read_token(fp);
-    new->vec = (VECT *)mybmalloc(sizeof(VECT) * new->len);
+    new->vec = (VECT *)mybmalloc2(sizeof(VECT) * new->len, &(hmm->mroot));
     /* needs comversion if integerized */
     for (i=0;i<new->len;i++) {
       NoTokErr("missing some VARIANCE element");
-      new->vec[i] = atof(rdhmmdef_token);
+      new->vec[i] = (VECT)atof(rdhmmdef_token);
       read_token(fp);
     }
   }
@@ -104,8 +144,19 @@ var_read(FILE *fp)
   return (new);
 }
 
-/* read in variance data at the current point, and
-   return pointer to the data */
+/** 
+ * @brief  Return a pointer to the data located at the current point.
+ *
+ * If the current point is a macro reference, the pointer to the
+ * already defined data will be searched and returned.
+ * Otherwise, the definition of the data will be read from the current
+ * point and pointer to the newly allocated data will be returned.
+ * 
+ * @param fp [in] file pointer
+ * @param hmm [i/o] %HMM definition data
+ * 
+ * @return pointer to the data located at the current point.
+ */
 HTK_HMM_Var *
 get_var_data(FILE *fp, HTK_HMM_INFO *hmm)
 {
@@ -124,7 +175,7 @@ get_var_data(FILE *fp, HTK_HMM_INFO *hmm)
     return tmp;
   } else if (currentis("VARIANCE")){
     /* definition: define variance data, and return the pointer */
-    tmp = var_read(fp);
+    tmp = var_read(fp, hmm);
     tmp->name = NULL; /* no name */
     var_add(hmm, tmp);
     return tmp;
@@ -134,8 +185,13 @@ get_var_data(FILE *fp, HTK_HMM_INFO *hmm)
   }
 }
 
-
-/* define density data */
+/** 
+ * Read a new data and store it as a macro.
+ * 
+ * @param name [in] macro name
+ * @param fp [in] file pointer
+ * @param hmm [i/o] %HMM definition data
+ */
 void
 def_var_macro(char *name, FILE *fp, HTK_HMM_INFO *hmm)
 {
@@ -143,7 +199,7 @@ def_var_macro(char *name, FILE *fp, HTK_HMM_INFO *hmm)
 
   
   /* read in data and return newly malloced data */
-  new = var_read(fp);
+  new = var_read(fp, hmm);
 
   /* register it to the grobal HMM structure */
   new->name = name;
