@@ -12,13 +12,13 @@
  * @brief  Get N-gram probability of a word/class sequence.
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.4 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -62,7 +62,9 @@ search_bigram(NGRAM_INFO *ndata, WORD_ID w_l, WORD_ID w_r)
 
 /** 
  * Search for a reverse 3-gram tuple (@a wkey, w1, w2), where the context
- * 2-gram tuple (w1, w2) should exist on 2-gram data part at @a n2.  
+ * 2-gram tuple (w1, w2) should exist on 2-gram data part at @a n2.
+ *
+ * (for v3 struture format)
  * 
  * @param ndata [in] word/class N-gram
  * @param n2 [in] 2-gram data part where context 2-gram data exists.
@@ -71,8 +73,46 @@ search_bigram(NGRAM_INFO *ndata, WORD_ID w_l, WORD_ID w_r)
  * @return corresponding index to the 3-gram data part if found, or
  * NNID_INVALID if the 3-gram does not exist.
  */
-NNID
-search_trigram(NGRAM_INFO *ndata, NNID n2, WORD_ID wkey)
+static NNID
+search_trigram_v3(NGRAM_INFO *ndata, NNID n2, WORD_ID wkey)
+{
+  /* do binary search */
+  /* assume that data in (trigrams) are ordered by wid */
+  int left,right,mid;
+
+  if ((left = ndata->n3_bgn[n2]) == NNID_INVALID) 	/* has no bigram */
+    return (NNID_INVALID);
+  right = left + ndata->n3_num[n2] - 1;
+  while(left < right) {
+    mid = (left + right) / 2;
+    if (ndata->n3tonid[mid] < wkey) {
+      left = mid + 1;
+    } else {
+      right = mid;
+    }
+  }
+  if (ndata->n3tonid[left] == wkey) {
+    return (left);
+  } else {
+    return (NNID_INVALID);
+  }
+}
+
+/** 
+ * Search for a reverse 3-gram tuple (@a wkey, w1, w2), where the context
+ * 2-gram tuple (w1, w2) should exist on 2-gram data part at @a n2.  
+ * 
+ * (for v4 struture format)
+ * 
+ * @param ndata [in] word/class N-gram
+ * @param n2 [in] 2-gram data part where context 2-gram data exists.
+ * @param wkey [in] left word/class ID in N-gram
+ * 
+ * @return corresponding index to the 3-gram data part if found, or
+ * NNID_INVALID if the 3-gram does not exist.
+ */
+static NNID
+search_trigram_v4(NGRAM_INFO *ndata, NNID n2, WORD_ID wkey)
 {
   /* do binary search */
   /* assume that data in (trigrams) are ordered by wid */
@@ -202,7 +242,15 @@ tri_prob_rl(NGRAM_INFO *ndata, WORD_ID w1, WORD_ID w2, WORD_ID w3)
   int boid;
   
   if ((n2 = search_bigram(ndata, w2, w3)) != NNID_INVALID) {
-    if ((n3 = search_trigram(ndata, n2, w1)) != NNID_INVALID) {
+    switch(ndata->version) {
+    case 4:
+      n3 = search_trigram_v4(ndata, n2, w1);
+      break;
+    case 3:
+      n3 = search_trigram_v3(ndata, n2, w1);
+      break;
+    }
+    if (n3 != NNID_INVALID) {
       /* trigram exist */
       if (w1 != ndata->unk_id) {
 	return(ndata->p_rrl[n3]);
@@ -213,15 +261,21 @@ tri_prob_rl(NGRAM_INFO *ndata, WORD_ID w1, WORD_ID w2, WORD_ID w3)
       /* return back-off prob */
       /* bo_wt_rl(w2,w3) * p(w1|w2) */
       /* unk will be discounted at bi-gram */
-      if ((boid = ndata->n2bo_upper[n2]) == NNID_INVALID_UPPER) {	/* has no bigram */
-	return(bi_prob_rl(ndata, w1, w2));
-      } else {
-	boid = (boid << 16) + (NNID)(ndata->n2bo_lower[n2]);
-	return(ndata->bo_wt_rrl[boid] + bi_prob_rl(ndata, w1, w2)); 
+      switch(ndata->version) {
+      case 4:
+	if ((boid = ndata->n2bo_upper[n2]) == NNID_INVALID_UPPER) {	/* has no bigram */
+	  return(bi_prob_rl(ndata, w1, w2));
+	} else {
+	  boid = (boid << 16) + (NNID)(ndata->n2bo_lower[n2]);
+	  return(ndata->bo_wt_rrl[boid] + bi_prob_rl(ndata, w1, w2)); 
+	}
+	break;
+      case 3:
+	return(ndata->bo_wt_rrl[n2] + bi_prob_rl(ndata, w1, w2));
+	break;
       }
     }
-  } else {
-    /* context not exist, so return bigram prob */
-    return(bi_prob_rl(ndata, w1, w2));
   }
+  /* context not exist, so return bigram prob */
+  return(bi_prob_rl(ndata, w1, w2));
 }
