@@ -1,14 +1,36 @@
+/**
+ * @file   calc_tied_mix.c
+ * @author Akinobu LEE
+ * @date   Thu Feb 17 14:22:44 2005
+ * 
+ * <JA>
+ * @brief  祬ʬۤνŤߤĤ¤η׻tied-mixtureѡåͭ
+ *
+ * Tied-mixture ѤΥʬ۷׻Ǥϥå夬θޤ
+ * ׻줿ʬۤβ٤ϥɥ֥åñ̤ǥե졼ऴȤ
+ * å夵졤Ʊɥ֥åƱ֤ǥ줿
+ * Υå夫֤ͤޤ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Compute weighed sum of Gaussian mixture for tied-mixture model (cache enabled)
+ *
+ * In tied-mixture computation, the computed output probability of each
+ * Gaussian component will be cache per codebook, for each input frame.
+ * If the same codebook of the same time is accessed later, the cached
+ * value will be returned.
+ * </EN>
+ * 
+ * $Revision: 1.3 $
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* calc_tied_mix.c --- calculate outprob of tied mixture */
-/*                     codebook-level cache is enabled */
-
-/* $Id: calc_tied_mix.c,v 1.4 2003/09/29 06:01:23 ri Exp $ */
-
 #include <sent/stddefs.h>
 #include <sent/htk_hmm.h>
 #include <sent/htk_param.h>
@@ -16,31 +38,37 @@
 #include <sent/gprune.h>
 #include "globalvars.h"
 
-/* control */
-static int allocframenum;
 
 /* book cache */
-static MIXCACHE ***mixture_cache = NULL;/* [time][bookid][#] */
-static MIXCACHE **tcache;		/* [bookid][#] of current time */
-static MIXCACHE *ttcache;		/* [#] of current time and book */
-static MIXCACHE **last_tcache;
-static MIXCACHE *last_ttcache;		/* [#] of current time and book */
-static int *last_id;
+static MIXCACHE ***mixture_cache = NULL;///< Codebook cache: [time][book_id][0..computed_mixture_num]
+static MIXCACHE **tcache; ///< Local work area that holds pointer to the cache array [bookid][0..computed_mixture_num] of the current time
+static MIXCACHE *ttcache; ///< Local work area that holds pointer to the cache list [0..computed_mixture_num] of the current time and the current codebook
+static int *last_id;		///< List of computed mixture id on the previous input frame
+static int allocframenum;	///< Allocated frame length of codebook cache
+static MIXCACHE **last_tcache; ///< Pointer to the cache array of previous frame, to pass to compute_gaussset()
+static MIXCACHE *last_ttcache; ///< Pointer to the cache list of current codebook on previous frame
 
-/* results */
-static LOGPROB **fallback_score = NULL; /* [t][gssetid], LOG_ZERO if selected*/
-
-/* initialize cache area (should be called once on startup) */
+/** 
+ * Initialize codebook cache area.
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 calc_tied_mix_init()
 {
   mixture_cache = NULL;
   allocframenum = -1;
-  last_id = (int *)mybmalloc(sizeof(int) * OP_hmminfo->maxmixturenum);
+  last_id = (int *)mymalloc(sizeof(int) * OP_hmminfo->maxmixturenum);
   return TRUE;
 }
 
-/* prepare cache area (should be called for each sample) */
+/** 
+ * Setup codebook cache for the next incoming input.
+ * 
+ * @param framenum [in] length of the next input.
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 calc_tied_mix_prepare(int framenum)
 {
@@ -77,8 +105,35 @@ calc_tied_mix_prepare(int framenum)
   return TRUE;
 }
 
+/** 
+ * Free work area for tied-mixture calculation.
+ * 
+ */
+void
+calc_tied_mix_free()
+{
+  int t;
+  if (mixture_cache != NULL) {
+    for(t=0;t<allocframenum;t++) {
+      free(mixture_cache[t][0]);
+      free(mixture_cache[t]);
+    }
+    free(mixture_cache);
+  }
+  free(last_id);
+}
 
-
+/** 
+ * @brief  Compute the output probability of current state OP_State on
+ * tied-mixture model
+ * 
+ * This function assumes that the OP_state is assigned to a tied-mixture
+ * codebook.  Here the output probability of Gaussian mixture component
+ * referred by OP_state is consulted to the book level cache, and if not
+ * computed yet on that input frame time, it will be computed here.
+ *
+ * @return the computed output probability in log10.
+ */
 LOGPROB
 calc_tied_mix()
 {
