@@ -1,13 +1,26 @@
+/**
+ * @file   gms_gprune.c
+ * @author Akinobu LEE
+ * @date   Thu Feb 17 15:05:08 2005
+ * 
+ * <JA>
+ * @brief  Gaussian Mixture Selection Τ Gaussian pruning ѤΥեHMMη׻
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Calculate the GMS monophone %HMM for Gaussian Mixture Selection using Gaussian pruning
+ * </EN>
+ * 
+ * $Revision: 1.3 $
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* gms_gprune.c --- compute GMS HMM with Gaussian pruning */
-
-/* $Id: gms_gprune.c,v 1.4 2003/09/29 06:01:23 ri Exp $ */
-
 #include <sent/stddefs.h>
 #include <sent/htk_hmm.h>
 #include <sent/htk_param.h>
@@ -16,39 +29,48 @@
 #include "globalvars.h"
 
 /* activate experimental methods */
-#define GS_MAX_PROB		/* compute only max for GS states */
-#define LAST_BEST		/* compute last best Gaussian first */
-#undef BEAM			/* enable beam pruning */
-#define BEAM_OFFSET 10.0	/* envelope offset */
+#define GS_MAX_PROB		///< Compute only max for GS states
+#define LAST_BEST		///< Compute last best Gaussians first
+#undef BEAM			///< Enable beam pruning
+#define BEAM_OFFSET 10.0	///< Beam offset, used when BEAM enabled
 
 #ifdef BEAM
-#define LAST_BEST
+#define LAST_BEST		///< Compute last best Gaussians first
 #endif
 #ifdef LAST_BEST
-#define GS_MAX_PROB
+#define GS_MAX_PROB		///< Compute only max for GS states
 #endif
 
 /* local cache */
-static int my_gsset_num;	/* num of gsset states (local copy) */
-static int *last_max_id;	/* maximum mixture id of last call for each states */
+static int my_gsset_num;	///< num of gsset states (local copy)
+static int *last_max_id;	///< maximum mixture id of last call for each states
 #ifdef BEAM
-static VECT *dimthres;	/* threshold for each dimension (reversed,base=1) */
-static int dimthres_num;
+static VECT *dimthres;	///< threshold for each dimension (reversed,base=1)
+static int dimthres_num;	///< number of above
 #endif
 
 /************************************************************************/
-/* initialization */
+/** 
+ * Initialization of GMS %HMM likelihood computation.
+ * 
+ * @param hmminfo [in] GMS %HMM definition
+ * @param gsset_num [in] number of states in GMS %HMM
+ */
 void
 gms_gprune_init(HTK_HMM_INFO *hmminfo, int gsset_num)
 {
   my_gsset_num = gsset_num;
-  last_max_id = (int *)mybmalloc(sizeof(int) * gsset_num);
+  last_max_id = (int *)mymalloc(sizeof(int) * gsset_num);
 #ifdef BEAM
   dimthres_num = hmminfo->opt.vec_size;
-  dimthres = (LOGPROB *)mybmalloc(sizeof(LOGPROB) * dimthres_num);
+  dimthres = (LOGPROB *)mymalloc(sizeof(LOGPROB) * dimthres_num);
 #endif
 }
 
+/** 
+ * Prepare GMS %HMM computation for the next speech input.
+ * 
+ */
 void
 gms_gprune_prepare()
 {
@@ -58,10 +80,30 @@ gms_gprune_prepare()
   }
 }
 
+/**
+ * Free GMS related work area.
+ * 
+ */
+void
+gms_gprune_free()
+{
+  free(last_max_id);
+#ifdef BEAM
+  free(dimthres);
+#endif
+}
+
 /**********************************************************************/
-/* compute only max by (safe|beam) pruning */
 /* LAST_BEST ... compute the maximum component in last frame first */
 /* BEAM ... use beam pruning */
+/** 
+ * Compute only max by safe pruning
+ * 
+ * @param binfo [in] Gaussian density
+ * @param thres [in] constant pruning threshold
+ * 
+ * @return the computed likelihood.
+ */
 static LOGPROB
 calc_contprob_with_safe_pruning(HTK_HMM_Dens *binfo, LOGPROB thres)
 {
@@ -89,6 +131,14 @@ calc_contprob_with_safe_pruning(HTK_HMM_Dens *binfo, LOGPROB thres)
 
 #ifdef BEAM
 
+/** 
+ * Compute only max by beam pruning, when no mixture has not
+ * been computed to the last.
+ * 
+ * @param binfo [in] Gaussian density
+ * 
+ * @return the computed likelihood.
+ */
 static LOGPROB
 calc_contprob_with_beam_pruning_pre(HTK_HMM_Dens *binfo)
 {
@@ -112,6 +162,15 @@ calc_contprob_with_beam_pruning_pre(HTK_HMM_Dens *binfo)
   }
   return((tmp + binfo->gconst) / -2.0);
 }
+
+/** 
+ * Compute only max by beam pruning, after at least one mixture has 
+ * been computed to the last.
+ * 
+ * @param binfo [in] Gaussian density
+ * 
+ * @return the computed likelihood.
+ */
 static LOGPROB
 calc_contprob_with_beam_pruning_post(HTK_HMM_Dens *binfo)
 {
@@ -140,6 +199,17 @@ calc_contprob_with_beam_pruning_post(HTK_HMM_Dens *binfo)
 #endif /* BEAM */
 
 #ifdef LAST_BEST
+
+/** 
+ * Compute log output likelihood of a state.  Only maximum Gaussian will be
+ * computed.
+ * 
+ * @param stateinfo [in] %HMM state to compute
+ * @param last_maxi [in] the mixture id that got the maximum value at the previous frame, or -1 if not exist.
+ * @param maxi_ret [out] tue mixture id that get the maximum value at this call.
+ * 
+ * @return the log likelihood.
+ */
 static LOGPROB
 compute_g_max(HTK_HMM_State *stateinfo, int last_maxi, int *maxi_ret)
 {
@@ -191,6 +261,14 @@ compute_g_max(HTK_HMM_State *stateinfo, int last_maxi, int *maxi_ret)
   
 #else  /* ~LAST_BEST */
   
+/** 
+ * Compute log output likelihood of a state.  Only maximum Gaussian will be
+ * computed.
+ * 
+ * @param stateinfo [in] %HMM state to compute
+ * 
+ * @return the log likelihood.
+ */
 static LOGPROB
 compute_g_max(HTK_HMM_State *stateinfo)
 {
@@ -213,6 +291,17 @@ compute_g_max(HTK_HMM_State *stateinfo)
 /**********************************************************************/
 /* main function: compute all gshmm scores */
 /* *** assume to be called for sequencial frame (using last result) */
+
+/** 
+ * Main function to compute all the GMS %HMM states in a frame
+ * with the input vectore specified by OP_vec.  This function assumes
+ * that this will be called for sequencial frame, since it utilizes the
+ * result of previous frame for faster pruning.
+ * 
+ * @param gsset [in] list of GMS %HMM state set.
+ * @param gsset_num [in] length of above
+ * @param scores_ret [out] array of scores for each GMS %HMM state
+ */
 void
 compute_gs_scores(GS_SET *gsset, int gsset_num, LOGPROB *scores_ret)
 {
