@@ -37,13 +37,13 @@
  * calc_tied_mix() or calc_mix().
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -247,6 +247,19 @@ gprune_heu_init()
   return TRUE;
 }
 
+/**
+ * Free gprune_heu related work area.
+ * 
+ */
+void
+gprune_heu_free()
+{
+  free(OP_calced_score);
+  free(OP_calced_id);
+  free(mixcalced);
+  free(backmax);
+}
+
 /** 
  * @brief  Compute a set of Gaussians with heuristic pruning.
  *
