@@ -29,13 +29,13 @@
  * calc_tied_mix() or calc_mix().
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.4 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -92,6 +92,17 @@ gprune_none_init()
   return TRUE;
 }
 
+/**
+ * Free gprune_none related work area.
+ * 
+ */
+void
+gprune_none_free()
+{
+  free(OP_calced_score);
+  free(OP_calced_id);
+}
+
 /** 
  * @brief  Compute a set of Gaussians with no pruning
  *
@@ -116,7 +127,6 @@ gprune_none(HTK_HMM_Dens **g, int num, int *last_id)
   OP_calced_num = 0;
   for(i=0; i<num; i++) {
     dens = *(g++);
-    if (!dens) continue;	/* skip if density = NULL */
     *(prob++) = compute_g_base(dens);
     *(id++) = i;
     OP_calced_num++;
