@@ -33,13 +33,13 @@
  *      -# use outprob(t, hmmstate, param) to get output probability of a state
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -71,18 +71,22 @@ outprob_init(HTK_HMM_INFO *hmminfo,
   case GPRUNE_SEL_NONE:
     compute_gaussset = gprune_none;
     compute_gaussset_init = gprune_none_init;
+    compute_gaussset_free = gprune_none_free;
     break;
   case GPRUNE_SEL_SAFE:
     compute_gaussset = gprune_safe;
     compute_gaussset_init = gprune_safe_init;
+    compute_gaussset_free = gprune_safe_free;
     break;
   case GPRUNE_SEL_HEURISTIC:
     compute_gaussset = gprune_heu;
     compute_gaussset_init = gprune_heu_init;
+    compute_gaussset_free = gprune_heu_free;
     break;
   case GPRUNE_SEL_BEAM:
     compute_gaussset = gprune_beam;
     compute_gaussset_init = gprune_beam_init;
+    compute_gaussset_free = gprune_beam_free;
     break;
   }
   /* select caching function to compute output probability of a mixture */
@@ -149,3 +153,24 @@ outprob_prepare(int framenum)
   OP_last_time = OP_time = -1;
   return TRUE;
 }
+
+/**
+ * Free all work area for outprob computation.
+ * 
+ */
+void
+outprob_free()
+{
+  compute_gaussset_free();
+  if (OP_hmminfo->is_tied_mixture) {
+    calc_tied_mix_free();
+  }
+  if (OP_gshmm != NULL) {
+    gms_free();
+  }
+  outprob_cache_free();
+  if (OP_hmminfo->cdset_method == IWCD_NBEST) {
+    outprob_cd_nbest_free();
+  }
+
+}
