@@ -1,17 +1,33 @@
+/**
+ * @file   voca_load_htkdict.c
+ * @author Akinobu LEE
+ * @date   Fri Feb 18 19:43:06 2005
+ * 
+ * <JA>
+ * @brief  HTKñ켭ǡɤ߹
+ *
+ * ȥ饤եǥѤ硤Υեɽȥ饤եؤ
+ * Ѵӥǥ¸ߥåϤμɤ߹߻˹Ԥʤޤ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Read word dictionary from a file in HTK format
+ *
+ * When using triphone model, conversion from monophone expression
+ * in dictionary to triphone and the existence check of word-internal
+ * triphone will be done here.
+ * </EN>
+ * 
+ * $Revision: 1.2 $
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* voca_load_htkdict.c --- read in vocabulary data */
-
-/* $Id: voca_load_htkdict.c,v 1.15 2003/10/02 04:11:45 ri Exp $ */
-
-/* format is HTK Dictionary format */
-
-/* word-internal context dependency is considered in this function */
-
 #include <sent/stddefs.h>
 #include <sent/vocabulary.h>
 #include <sent/htk_hmm.h>
@@ -41,17 +57,24 @@
  *                 to express the pronunciation
  */
 
-#define PHONEMELEN_STEP  10	/* malloc base */
-static char buf[MAXLINELEN];	/* read buffer */
-static char bufbak[MAXLINELEN];	/* read buffer (for debug output) */
+#define PHONEMELEN_STEP  10	///< Memory allocation step for phoneme sequence
+static char buf[MAXLINELEN];	///< Local work area for input text processing
+static char bufbak[MAXLINELEN];	///< Local work area for debug message
 
-static char trbuf[3][20];
-static char chbuf[30];
-static char nophone[1];
-static int  trp_l, trp, trp_r;
+static char trbuf[3][20];	///< Local buffer for triphone convertion
+static char chbuf[30];	     ///< Another local buffer for triphone convertion
+static char nophone[1];		///< Local buffer to indicate 'no phone'
+static int  trp_l;		///< Triphone cycle index
+static int  trp;		///< Triphone cycle index
+static int  trp_r;		///< Triphone cycle index
 
-/* return string of triphone name composed from last 3 call */
-/* returns NULL on end */
+/** 
+ * Return string of triphone name composed from last 3 call.
+ * 
+ * @param p [in] next phone string
+ * 
+ * @return the composed triphone name, or NULL on end.
+ */
 char *
 cycle_triphone(char *p)
 {
@@ -92,14 +115,72 @@ cycle_triphone(char *p)
 
   return(chbuf);
 }
+
+/** 
+ * Flush the triphone buffer and return the last biphone.
+ * 
+ * @return the composed last bi-phone name.
+ */
 char *
 cycle_triphone_flush()
 {
   return(cycle_triphone(nophone));
 }
 
+/** 
+ * Add a triphone name to the missing error list in WORD_INFO.
+ * 
+ * @param winfo [i/o] word dictionary to add the error phone to error list
+ * @param name [in] phone name to be added
+ */
+static void
+add_to_error(WORD_INFO *winfo, char *name)
+{
+  char *buf;
+  char *match;
 
-/* parse winfo and set maximum state length */
+  buf = (char *)mymalloc(strlen(name) + 1);
+  strcpy(buf, name);
+  if (winfo->errph_root == NULL) {
+    winfo->errph_root = aptree_make_root_node(buf);
+  } else {
+    match = aptree_search_data(buf, winfo->errph_root);
+    if (!strmatch(match, buf)) {
+      aptree_add_entry(buf, buf, match, &(winfo->errph_root));
+    }
+  }
+}
+
+/** 
+ * Traverse callback function to output a error phone.
+ * 
+ * @param x [in] error phone string of the node
+ */
+static void
+callback_list_error(void *x)
+{
+  char *name;
+  name = x;
+  j_printf("%s\n", name);
+}
+/** 
+ * Output all error phones appeared while readin a word dictionary.
+ * 
+ * @param winfo [in] word dictionary data
+ */
+static void
+list_error(WORD_INFO *winfo)
+{
+  j_printf("////// Missing phones:\n");
+  aptree_traverse_and_do(winfo->errph_root, callback_list_error);
+  j_printf("//////////////////////\n");
+}
+
+/** 
+ * Parse a word dictionary and set the maximum state length per word.
+ * 
+ * @param winfo [i/o]
+ */
 static void
 set_maxwn(WORD_INFO *winfo)
 {
@@ -117,7 +198,11 @@ set_maxwn(WORD_INFO *winfo)
   winfo->maxwn = maxwn;
 }
 
-/* parse winfo and set maximum word length */
+/** 
+ * Parse the word dictionary to set the maximum word length.
+ * 
+ * @param winfo [i/o] word dictionary data
+ */
 static void
 set_maxwlen(WORD_INFO *winfo)
 {
@@ -129,15 +214,19 @@ set_maxwlen(WORD_INFO *winfo)
   }
   winfo->maxwlen = maxwlen;
 }
-
 
-/* read in vocabulary file */
-boolean				/* TRUE on success, FALSE on any error word */
-voca_load_htkdict(
-     FILE *fp,
-     WORD_INFO *winfo,
-     HTK_HMM_INFO *hmminfo,	/* if NULL, phonemes are ignored */
-     boolean ignore_tri_conv)	/* TRUE if convert to triphone should be ignored */
+/** 
+ * Top function to read word dictionary via file pointer
+ * 
+ * @param fp [in] file pointer
+ * @param winfo [out] pointer to word dictionary to store the read data.
+ * @param hmminfo [in] HTK %HMM definition data.  if NULL, phonemes are ignored.
+ * @param ignore_tri_conv [in] TRUE if triphone conversion is ignored
+ * 
+ * @return TRUE on success, FALSE on any error word.
+ */
+boolean
+voca_load_htkdict(FILE *fp, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo, boolean ignore_tri_conv)
 {
   boolean ok_flag = TRUE;
   WORD_ID vnum;
@@ -151,11 +240,13 @@ voca_load_htkdict(
   vnum = 0;
   while (getl(buf, sizeof(buf), fp) != NULL) {
     if (vnum >= winfo->maxnum) winfo_expand(winfo);
-    if (voca_load_htkdict_line(buf, vnum, winfo, hmminfo, ignore_tri_conv, do_conv, &ok_flag) == FALSE) break;
+    if (voca_load_htkdict_line(buf, vnum, winfo, hmminfo, do_conv, &ok_flag) == FALSE) break;
     vnum++;
   }
   winfo->num = vnum;
 
+  if (winfo->errph_root != NULL) list_error(winfo);
+
   /* compute maxwn */
   set_maxwn(winfo);
   set_maxwlen(winfo);
@@ -164,13 +255,18 @@ voca_load_htkdict(
 }
 
 
-/* read in vocabulary file (file descriptor version) */
-boolean				/* TRUE on success, FALSE on any error word */
-voca_load_htkdict_fd(
-     int fd,
-     WORD_INFO *winfo,
-     HTK_HMM_INFO *hmminfo,	/* if NULL, phonemes are ignored */
-     boolean ignore_tri_conv)	/* TRUE if convert to triphone should be ignored */
+/** 
+ * Top function to read word dictionary via file descriptor.
+ * 
+ * @param fd [in] file descriptor
+ * @param winfo [out] pointer to word dictionary to store the read data.
+ * @param hmminfo [in] HTK %HMM definition data.  if NULL, phonemes are ignored.
+ * @param ignore_tri_conv [in] TRUE if triphone conversion is ignored
+ * 
+ * @return TRUE on success, FALSE on any error word.
+ */
+boolean
+voca_load_htkdict_fd(int fd, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo, boolean ignore_tri_conv)
 {
   boolean ok_flag = TRUE;
   WORD_ID vnum;
@@ -184,11 +280,13 @@ voca_load_htkdict_fd(
   vnum = 0;
   while(getl_fd(buf, MAXLINELEN, fd) != NULL) {
     if (vnum >= winfo->maxnum) winfo_expand(winfo);
-    if (voca_load_htkdict_line(buf, vnum, winfo, hmminfo, ignore_tri_conv, do_conv, &ok_flag) == FALSE) break;
+    if (voca_load_htkdict_line(buf, vnum, winfo, hmminfo, do_conv, &ok_flag) == FALSE) break;
     vnum++;
   }
   winfo->num = vnum;
 
+  if (winfo->errph_root != NULL) list_error(winfo);
+
   /* compute maxwn */
   set_maxwn(winfo);
   set_maxwlen(winfo);
@@ -196,13 +294,57 @@ voca_load_htkdict_fd(
   return(ok_flag);
 }
 
-/* append a single entry to the existing dictionary */
-boolean				/* TRUE on success, FALSE on any error word */
-voca_append_htkdict(
-     char *entry,               /* dictionary entry string to be appended */
-     WORD_INFO *winfo,
-     HTK_HMM_INFO *hmminfo,	/* if NULL, phonemes are ignored */
-     boolean ignore_tri_conv)	/* TRUE if convert to triphone should be ignored */
+/** 
+ * Top function to read word dictionary via socket descriptor.
+ * 
+ * @param sd [in] socket descriptor
+ * @param winfo [out] pointer to word dictionary to store the read data.
+ * @param hmminfo [in] HTK %HMM definition data.  if NULL, phonemes are ignored.
+ * @param ignore_tri_conv [in] TRUE if triphone conversion is ignored
+ * 
+ * @return TRUE on success, FALSE on any error word.
+ */
+boolean
+voca_load_htkdict_sd(int sd, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo, boolean ignore_tri_conv)
+{
+  boolean ok_flag = TRUE;
+  WORD_ID vnum;
+  boolean do_conv = FALSE;
+  
+  if (hmminfo != NULL && hmminfo->is_triphone && (! ignore_tri_conv))
+    do_conv = TRUE;
+  
+  winfo_init(winfo);
+  
+  vnum = 0;
+  while(getl_sd(buf, MAXLINELEN, sd) != NULL) {
+    if (vnum >= winfo->maxnum) winfo_expand(winfo);
+    if (voca_load_htkdict_line(buf, vnum, winfo, hmminfo, do_conv, &ok_flag) == FALSE) break;
+    vnum++;
+  }
+  winfo->num = vnum;
+  
+  if (winfo->errph_root != NULL) list_error(winfo);
+
+  /* compute maxwn */
+  set_maxwn(winfo);
+  set_maxwlen(winfo);
+  
+  return(ok_flag);
+}
+
+/** 
+ * Append a single entry to the existing word dictionary.
+ * 
+ * @param entry [in] dictionary entry string to be appended.
+ * @param winfo [out] pointer to word dictionary to append the data.
+ * @param hmminfo [in] HTK %HMM definition data.  if NULL, phonemes are ignored.
+ * @param ignore_tri_conv [in] TRUE if triphone conversion is ignored
+ * 
+ * @return TRUE on success, FALSE on any error word.
+ */
+boolean
+voca_append_htkdict(char *entry, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo, boolean ignore_tri_conv)
 {
   boolean ok_flag = TRUE;
   boolean do_conv = FALSE;
@@ -212,32 +354,37 @@ voca_append_htkdict(
 
   if (winfo->num >= winfo->maxnum) winfo_expand(winfo);
   strcpy(buf, entry);		/* const buffer not allowed in voca_load_htkdict_line() */
-  voca_load_htkdict_line(buf, winfo->num, winfo, hmminfo, ignore_tri_conv, do_conv, &ok_flag);
+  voca_load_htkdict_line(buf, winfo->num, winfo, hmminfo, do_conv, &ok_flag);
 
   if (ok_flag == TRUE) {
     winfo->num++;
     /* re-compute maxwn */
     set_maxwn(winfo);
     set_maxwlen(winfo);
+  } else {
+    if (winfo->errph_root != NULL) list_error(winfo);
   }
 
   return(ok_flag);
 }
 
-
-/* process dic entry in buf */
-/* will set ok_flag to FALSE if there is an error in processing entry */
-/* return TRUE on continuing next reading, FALSE for read termination */
+/** 
+ * Sub function to Add a dictionary entry line to the word dictionary.
+ * 
+ * @param buf [i/o] buffer to hold the input string, will be modified in this function
+ * @param vnum [in] current number of words in @a winfo
+ * @param winfo [out] pointer to word dictionary to append the data.
+ * @param hmminfo [in] HTK %HMM definition data.  if NULL, phonemes are ignored.
+ * @param do_conv [in] TRUE if performing triphone conversion
+ * @param ok_flag [out] will be set to FALSE if an error occured for this input.
+ * 
+ * @return FALSE if buf == "DICEND", else TRUE will be returned.
+ */
 boolean
-voca_load_htkdict_line(char *buf, int vnum,
-		       WORD_INFO *winfo,
-		       HTK_HMM_INFO *hmminfo,
-		       boolean ignore_tri_conv,
-		       boolean do_conv,
-		       boolean *ok_flag)
+voca_load_htkdict_line(char *buf, int vnum, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo, boolean do_conv, boolean *ok_flag)
 {
   char *ptmp, *lp = NULL, *p;
-  static char cbuf[50];
+  static char cbuf[MAX_HMMNAME_LEN];
   int tmpmaxlen, len;
   HMM_Logical **tmpwseq;
   HMM_Logical *tmplg;
@@ -386,16 +533,22 @@ voca_load_htkdict_line(char *buf, int vnum,
 	  /* both defined or pseudo phone are not found */
 	  if (len == 0 && lp == NULL) {
 	    j_printerr("line %d: triphone \"*-%s+*\" or monophone \"%s\" not found\n", vnum+1, p, p);
+	    snprintf(cbuf,MAX_HMMNAME_LEN,"*-%s+* or monophone %s", p, p);
 	  } else if (len == 0) {
 	    j_printerr("line %d: triphone \"*-%s\" or biphone \"%s\" not found\n", vnum+1, p, p);
+	    snprintf(cbuf,MAX_HMMNAME_LEN,"*-%s or biphone %s", p, p);
 	  } else if (lp == NULL) {
 	    j_printerr("line %d: triphone \"%s+*\" or biphone \"%s\" not found\n", vnum+1, p, p);
+	    snprintf(cbuf,MAX_HMMNAME_LEN,"%s+* or biphone %s", p, p);
 	  } else {
 	    j_printerr("line %d: triphone \"%s\" not found\n", vnum+1, p);
+	    snprintf(cbuf,MAX_HMMNAME_LEN,"%s", p);
 	  }
 	} else {
 	  j_printerr("line %d: phone \"%s\" not found\n", vnum+1, p);
+	  snprintf(cbuf, MAX_HMMNAME_LEN, "%s", p);
 	}
+	add_to_error(winfo, cbuf);
 	pok = FALSE;
       } else {
 	/* found */
@@ -431,8 +584,17 @@ voca_load_htkdict_line(char *buf, int vnum,
   return(TRUE);
 }
 
-
-/* convert monophone dictionary to word-internal triphone */
+/** 
+ * @brief Convert whole words in word dictionary to word-internal triphone.
+ *
+ * Normally triphone conversion will be performed directly when reading
+ * dictionary file.  This function is for post conversion only.
+ * 
+ * @param winfo [i/o] word dictionary information
+ * @param hmminfo [in] HTK %HMM definition
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 voca_mono2tri(WORD_INFO *winfo, HTK_HMM_INFO *hmminfo)
 {
@@ -463,14 +625,19 @@ voca_mono2tri(WORD_INFO *winfo, HTK_HMM_INFO *hmminfo)
   return (ok_flag);
 }
 
-/* append whole winfo to other */
-/* assume both use the same hmminfo */
-void				/* TRUE on success, FALSE on any error word */
-voca_append(
-	    WORD_INFO *dstinfo,	/* append to this word_info */
-	    WORD_INFO *srcinfo,	/* append this to dst */
-	    int coffset,	/* category ID offset */
-	    int woffset)	/* word ID offset */
+/** 
+ * Append one word dictionary to other, for multiple grammar handling.
+ * Assumes that the same %HMM definition is used on both word dictionary.
+ * 
+ * @param dstinfo [i/o] word dictionary
+ * @param srcinfo [in] word dictionary to be appended to @a dst
+ * @param coffset [in] category id offset in @a dst where the new data
+ * should be stored
+ * @param woffset [in] word id offset in @a dst where the new data
+ * should be stored
+ */
+void
+voca_append(WORD_INFO *dstinfo, WORD_INFO *srcinfo, int coffset, int woffset)
 {
   WORD_ID n, w;
   int i;
