@@ -1,17 +1,43 @@
-/* Copyright (c) 1991-2002 Doshita Lab. Speech Group, Kyoto University */
-/* Copyright (c) 2000-2002 Speech and Acoustics Processing Lab., NAIST */
-/*   All rights reserved   */
-
-/* voca_lookup.c --- vocabulary lookup function */
-
-/* $Id: voca_lookup.c,v 1.3 2002/09/11 22:01:50 ri Exp $ */
+/**
+ * @file   voca_lookup.c
+ * @author Akinobu LEE
+ * @date   Fri Feb 18 21:24:01 2005
+ * 
+ * <JA>
+ * @brief  ñ켭ñθ
+ *
+ * ñ򡤡ָ쥨ȥ̾פ뤤ϡָ쥨ȥ̾[ʸ]
+ * 뤤ϡ#ñֹפ鸡ޤ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Look up a word on dictionary by string
+ *
+ * String can be "langentry" or "langentry[outputstring]", or
+ * "#number".
+ * </EN>
+ * 
+ * $Revision: 1.2 $
+ * 
+ */
+/*
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * All rights reserved
+ */
 
 #include <sent/stddefs.h>
 #include <sent/vocabulary.h>
 
-/* search for word (for debug) */
-/* keyword: "wname" or "wname[woutput]" or "#num" */
-/* returns WORD_INVALID if not found */
+/** 
+ * Look up a word on dictionary by string.
+ * 
+ * @param keyword [in] keyword to search
+ * @param winfo [in] word dictionary
+ * 
+ * @return the word id if found, or WORD_INVALID if not found.
+ */
 WORD_ID
 voca_lookup_wid(char *keyword, WORD_INFO *winfo)
 {
@@ -75,8 +101,17 @@ voca_lookup_wid(char *keyword, WORD_INFO *winfo)
 
 /* convert space-separated words string -> array of wid */
 /* return malloced array */
-#define WSSTEP 10
+#define WSSTEP 10 ///< Allocation step 
 
+/** 
+ * Convert string of space-separated word strings to array of word ids.
+ * 
+ * @param winfo [in] word dictionary
+ * @param s [in] string of space-separated word strings
+ * @param len_return [out] number of found words
+ * 
+ * @return pointer to a newly allocated word list.
+ */
 WORD_ID *
 new_str2wordseq(WORD_INFO *winfo, char *s, int *len_return)
 {
