@@ -10,7 +10,7 @@
  * Ƥޤե졼ñ̤ФԤؿ wav2mfcc-pipe.c 
  * ҤƤޤ
  * 
- * ФǤΤ MFCC_{0|E}_D(_Z) ηΤߤǤ
+ * ФǤΤ MFCC[_0][_E][_D][_A][_Z] ηǤ
  * </JA>
  * 
  * <EN>
@@ -20,10 +20,10 @@
  * to MFCC vector array.  The frame-wise MFCC computation needed for
  * real-time recognition is defined in wav2mfcc-pipe.c.
  *
- * The only supported format is MFCC_{0|E}_D(_Z).
+ * The supported format is MFCC[_0][_E][_D][_A][_Z].
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.3 $
  * 
  */
 
@@ -75,6 +75,7 @@ make_costbl_hamming(int framesize)
 {
   int i;
   float a;
+
   if (costbl_hamming_len  == framesize) return;
   if (costbl_hamming_len != 0) {
     j_printerr("WARNING!! frame size changed!! (%d -> %d)\n",
@@ -83,7 +84,7 @@ make_costbl_hamming(int framesize)
   }
   costbl_hamming = (double *)mymalloc(sizeof(double) * framesize);
 
-  a = 2 * PI / (framesize - 1);
+  a = 2.0 * PI / (framesize - 1);
   for(i=1;i<=framesize;i++) {
     /*costbl_hamming[i-1] = 0.54 - 0.46 * cos(2 * PI * (i - 1) / (float)(framesize - 1));*/
     costbl_hamming[i-1] = 0.54 - 0.46 * cos(a * (i - 1));
@@ -201,7 +202,7 @@ make_sintbl_wcep(int lifter, int mfcc_dim)
 
 
 /** 
- * Convert wave data to MFCC_(0|E)_D(_Z).  Also does spectral subtraction
+ * Convert wave data to MFCC.  Also does spectral subtraction
  * if @a ssbuf specified.
  * 
  * @param wave [in] waveform data
@@ -213,37 +214,37 @@ make_sintbl_wcep(int lifter, int mfcc_dim)
  * 
  * @return the number of processed frames.
  */
-int Wav2MFCC_E_D(SP16 *wave, float **mfcc, Value para, int nSamples, float *ssbuf, int ssbuflen)
+int Wav2MFCC(SP16 *wave, float **mfcc, Value para, int nSamples, float *ssbuf, int ssbuflen)
 {
   float *bf;                        /* Work space for FFT */
   double *fbank;                    /* Filterbank */
-  float *energy;            /* Raw Energy */ 
   int i, k, t;
   int end = 0, start = 1;
   int frame_num;                    /* Number of samples in output file */
   FBankInfo fb;
+  float energy = 0.0;
+  float c0 = 0.0;
+  int p;
 
   /* Get filterbank information */
   fb = InitFBank(para);
   
   if((fbank = (double *)mymalloc((para.fbank_num+1)*sizeof(double))) == NULL){
-    j_error("Error: Wav2MFCC_E_D: failed to malloc\n");
+    j_error("Error: Wav2MFCC: failed to malloc\n");
   }
   if((bf = (float *)mymalloc(fb.fftN * sizeof(float))) == NULL){
-    j_error("Error: Wav2MFCC_E_D: failed to malloc\n");
+    j_error("Error: Wav2MFCC: failed to malloc\n");
   }
 
   if (ssbuf != NULL) {
     /* check ssbuf length */
     if (fb.fftN != ssbuflen) {
-      j_error("Error: Wav2MFCC_E_D: noise spectrum length not match\n");
+      j_error("Error: Wav2MFCC: noise spectrum length not match\n");
     }
   }
 
   frame_num = (int)((nSamples - para.framesize) / para.frameshift) + 1;
 
-  energy = (float *)mymalloc(sizeof(float)*frame_num);
-
   for(t = 0; t < frame_num; t++){
     if(end != 0) start = end - (para.framesize - para.frameshift) - 1;
 
@@ -253,55 +254,53 @@ int Wav2MFCC_E_D(SP16 *wave, float **mfcc, Value para, int nSamples, float *ssbu
     }
     end = i;
     
-    /* Calculate Log Raw Energy */
-    if(para.raw_e)
-      energy[t] = CalcLogRawE(bf, para.framesize); 
-
-    /* Pre-emphasise */
-    PreEmphasise(bf, para);
-    
-    /* Hamming Window */
-    Hamming(bf, para.framesize);
-
-    /* Calculate Log Energy */
-    if(!para.raw_e && !para.c0)
-      energy[t] = CalcLogRawE(bf, para.framesize);
-
-    /* Filterbank */
-    MakeFBank(bf, fbank, fb, para, ssbuf);
-
-    /* MFCC */
-    MakeMFCC(fbank, mfcc[t], para);   
-
-    /* 0'th Cepstral parameter */
-    if (para.c0) {
-      energy[t] = CalcC0(fbank, para);
+    if (para.zmeanframe) {
+      ZMeanFrame(bf, para.framesize);
     }
- 
-    /* Weight Cepstrum */
+
+    /* Calculate base MFCC coefficients */
+
+    if (para.energy && para.raw_e) {
+      /* calculate log raw energy */
+      energy = CalcLogRawE(bf, para.framesize);
+    }
+    /* pre-emphasize */
+    PreEmphasise(bf, para);
+    /* hamming window */
+    Hamming(bf, para.framesize);
+    if (para.energy && ! para.raw_e) {
+      /* calculate log energy */
+      energy = CalcLogRawE(bf, para.framesize);
+    }
+    /* filterbank */
+    MakeFBank(bf, fbank, fb, para, ssbuf);
+    /* 0'th cepstral parameter */
+    if (para.c0) c0 = CalcC0(fbank, para);
+    /* MFCC */
+    MakeMFCC(fbank, mfcc[t], para);
+    /* weight cepstrum */
     WeightCepstrum(mfcc[t], para);
-
+    /* set energy to mfcc */
+    p = para.mfcc_dim;
+    if (para.c0) mfcc[t][p++] = c0;
+    if (para.energy) mfcc[t][p++] = energy;
   }
-
+  
   /* Normalise Log Energy */
-  if(para.enormal)
-    NormaliseLogE(mfcc, energy, frame_num, para);
-  else{
-    for(t = 0; t < frame_num; t++)
-      mfcc[t][para.mfcc_dim] = energy[t];
-  }
+  if (para.energy && para.enormal) NormaliseLogE(mfcc, frame_num, para);
+  
+  /* Delta (consider energy suppress) */
+  if (para.delta) Delta(mfcc, frame_num, para);
 
-  /* Delta */
-  Delta(mfcc, frame_num, para);
+  /* Acceleration */
+  if (para.acc) Accel(mfcc, frame_num, para);
 
   /* Cepstrum Mean Normalization */
-  if(para.cmn)
-    CMN(mfcc, frame_num, para.mfcc_dim);
+  if(para.cmn) CMN(mfcc, frame_num, para.mfcc_dim);
 
   FreeFBank(fb);
   free(fbank);
   free(bf);
-  free(energy);
   return(frame_num);
 }
 
@@ -327,10 +326,25 @@ float CalcLogRawE(float *wave, int framesize)
   return(energy);
 }
 
-
-/* 
- *  
+/** 
+ * Remove DC offset per frame
+ * 
+ * @param wave [i/o] waveform data in the current frame
+ * @param framesize [in] frame size
+ * 
  */
+void
+ZMeanFrame(float *wave, int framesize)
+{		   
+  int i;
+  float mean;
+
+  mean = 0.0;
+  for(i = 1; i <= framesize; i++) mean += wave[i];
+  mean /= framesize;
+  for(i = 1; i <= framesize; i++) wave[i] -= mean;
+}
+
 /** 
  * Apply hamming window.
  * 
@@ -643,29 +657,31 @@ float CalcC0(double *fbank, Value para)
  * Normalise log energy
  * 
  * @param mfcc [i/o] array of MFCC vectors
- * @param energy [in] array of log energy value
  * @param frame_num [in] number of frames
  * @param para [in] configuration parameters
  */
-void NormaliseLogE(float **mfcc, float *energy, int frame_num, Value para)
+void NormaliseLogE(float **mfcc, int frame_num, Value para)
 {  
-  float *p, max, min;
+  float max, min, f;
   int t;
+  int l;
+
+  l = para.mfcc_dim;
+  if (para.c0) l++;
 
   /* find max log energy */
-  p = energy;
-  max = p[0];
+  max = mfcc[0][l];
   for(t = 0; t < frame_num; t++)
-    if(p[t] > max) max = p[t];
+    if(mfcc[t][l] > max) max = mfcc[t][l];
 
   /* set the silence floor */
   min = max - (para.silFloor * log(10.0)) / 10.0;  
 
   /* normalise */
-  p = energy;
   for(t = 0; t < frame_num; t++){
-    if (p[t] < min) p[t] = min;               
-    mfcc[t][para.mfcc_dim] = 1.0 - (max - p[t]) * para.escale;
+    f = mfcc[t][l];
+    if (f < min) f = min;
+    mfcc[t][l] = 1.0 - (max - f) * para.escale;
   }
 }
 
@@ -678,32 +694,83 @@ void NormaliseLogE(float **mfcc, float *energy, int frame_num, Value para)
  */
 void Delta(float **c, int frame, Value para)
 {
-  int theta, t, n, dim, B = 0;
+  int theta, t, n, B = 0;
   float A1, A2, sum;
+  float *ed;
 
   for(theta = 1; theta <= para.delWin; theta++)
     B += theta * theta;
 
-  dim = para.vec_num / 2;
+  if (para.absesup) ed = (float *)mymalloc(sizeof(float) * frame);
 
   for(t = 0; t < frame; t++){
-    for(n = 1; n <= dim; n++){
+    for(n = 0; n < para.baselen; n++){
       sum = 0;
       for(theta = 1; theta <= para.delWin; theta++){
 	/* Replicate the first or last vector */
 	/* at the beginning and end of speech */
-	if(t - theta < 0) A1 = c[0][n - 1];
-	else A1 = c[t - theta][n - 1];
-	if(t + theta >= frame) A2 = c[frame - 1][n - 1];
-	else A2 = c[t + theta][n - 1];
+	if (t - theta < 0) A1 = c[0][n];
+	else A1 = c[t - theta][n];
+	if (t + theta >= frame) A2 = c[frame - 1][n];
+	else A2 = c[t + theta][n];
 	sum += theta * (A2 - A1);
       }
-      c[t][n + para.mfcc_dim] = sum / (2 * B);
+      if (para.absesup && n == para.baselen-1) {
+	ed[t] = sum / (2 * B);
+      } else {
+	c[t][para.baselen + n] = sum / (2 * B);
+      }
     }
   }
+  
+  if (para.absesup) {
+    for (t=0;t<frame;t++) {
+      memmove(&(c[t][para.baselen-1]), &(c[t][para.baselen]), sizeof(float) * para.baselen - 1);
+      c[t][para.baselen * 2 - 2] = ed[t];
+    }
+    free(ed);
+  }
 }
 
 
+/** 
+ * Calculate acceleration coefficients.
+ * 
+ * @param c [i/o] MFCC vectors, in which the delta coeff. will be appended.
+ * @param frame [in] number of frames
+ * @param para [in] configuration parameters
+ */
+void Accel(float **c, int frame, Value para)
+{
+  int theta, t, n, B = 0;
+  int src, dst;
+  float A1, A2, sum;
+
+  for(theta = 1; theta <= para.accWin; theta++)
+    B += theta * theta;
+
+  for(t = 0; t < frame; t++){
+    src = para.baselen * 2 - 1;
+    if (para.absesup) src--;
+    dst = src + para.baselen;
+    for(n = 0; n < para.baselen; n++){
+      sum = 0;
+      for(theta = 1; theta <= para.accWin; theta++){
+	/* Replicate the first or last vector */
+	/* at the beginning and end of speech */
+	if (t - theta < 0) A1 = c[0][src];
+	else A1 = c[t - theta][src];
+	if (t + theta >= frame) A2 = c[frame - 1][src];
+	else A2 = c[t + theta][src];
+	sum += theta * (A2 - A1);
+      }
+      c[t][dst] = sum / (2 * B);
+      src--;
+      dst--;
+    }
+  }
+}
+
 /** 
  * Apply FFT
  * 
