@@ -17,19 +17,19 @@
  * @brief  
  * </EN>
  * 
- * $Revision:$
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
  * All rights reserved
  */
 
 /* mkbingram --- make binary n-gram for JULIUS from ARPA standard format */
 
-/* $Id: mkbingram.c,v 1.4 2004/03/22 04:14:32 ri Exp $ */
+/* $Id: mkbingram.c,v 1.3 2006/03/28 07:29:02 sumomo Exp $ */
 
 #include <sent/stddefs.h>
 #include <sent/ngram2.h>
@@ -121,7 +121,13 @@ main(int argc, char *argv[])
     j_printerr("failed to open %s\n", outfile);
     return -1;
   }
-  printf("Writing to %s...\n", outfile);
+  if (ngram->version < 4) {
+    printf("***********************************************\n");
+    printf("**** number of 3-gram exceeds 24bit limit!! \n");
+    printf("**** will be saved in old v3 format (<= v3.4.2)\n");
+    printf("***********************************************\n");
+  }
+  printf("Writing in v%d format to %s...\n", ngram->version, outfile);
   if (ngram_write_bin(fp, ngram, header) == FALSE){/* failed */
     j_printerr("failed to write %s\n",outfile);
     return -1;
